# Copyright 1999-2003 Josh Glover <jmglov@jmglov.net>
# Distributed under the terms of the GNU General Public License v2

inherit webapp-apache

S=${WORKDIR}/${PN}_${PV}
DESCRIPTION="Web-based system for managing tasks and defects throughout the software development process"
HOMEPAGE="http://www.incogen.com/index.php?type=General&param=bugport"
SRC_URI="http://www.incogen.com/downloads/${PN}/${PN}_${PV}.tar.gz"

LICENSE="BSD"
SLOT="0"
KEYWORDS="~x86"

IUSE="mysql oci8"
RDEPEND=">=virtual/php-4.3
         mysql? ( >=dev-db/mysql-4 )
         dev-php/adodb"

DEPEND="${RDEPEND}"

webapp-detect || NO_WEBSERVER=1

pkg_setup() {
	if [ ! `use mysql` ]; then
		if [ ! `use oci8` ]; then
			einfo "BugPort requires either the 'mysql' or 'oci8' USE " \
			      "flag enabled,"
			einfo "depending on which database you wish to use"
			die "Bad database configuration, see above"
		fi
	fi
	webapp-pkg_setup "${NO_WEBSERVER}"
	einfo "Installing into ${HTTPD_ROOT}."
}

src_compile() {
	einfo "Nothing to compile"
}

src_install() {
	local DocumentRoot=${HTTPD_ROOT}
	local destdir=${DocumentRoot}${PN}
	dodir ${destdir}
	cp -r . ${D}/${destdir}

	# Install documents
	dodoc *.txt
	docinto devel-docs
	dodoc devel-docs/*
	docinto install-gentoo-unsupported
	dodoc install-gentoo-unsupported/INSTALL.gentoo \
	      install-gentoo-unsupported/COPYING

	# Fix ownership and permissions
	chown -R ${HTTPD_USER}:${HTTPD_GROUP} ${D}/${destdir}
	find ${D}/${destdir} -type d | xargs chmod 755
	find ${D}/${destdir} -type f | xargs chmod 644

	# Link in ADOdb
	dosym /usr/local/adodb ${destdir}/ADODB

	# If a config file already exists in the live filesystem, (i.e. we are
	# upgrading), preserve it
	if [ -e ${destdir}/conf/config.conf ]; then
		cp ${destdir}/conf/config.conf ${D}/${destdir}/conf/config.conf
	# Otherwise, setup config file as much as possible
	else
		cp ${D}/${destdir}/conf/config.conf ${D}/${destdir}/conf/config.conf.ORIG
		awk '/^\$home/ { print "#"$0; print "\$home = \"'${destdir}'\";" }; ! /^\$home/ { print }' ${D}/${destdir}/conf/config.conf \
		>${T}/config.conf
		mv ${T}/config.conf ${D}/${destdir}/conf/config.conf
		if [ `use oci8` ]; then
			awk '/^\$type/ { print "#"$0; print "\$type = \"oracle\";" }; ! /^\$type/ { print }' ${D}/${destdir}/conf/config.conf \
			>${T}/config.conf
			mv ${T}/config.conf ${D}/${destdir}/conf/config.conf
		fi
	fi
}

pkg_postinst() {
echo; echo
	einfo 'If this is the first time you have installed BugPort, read the'
	einfo 'INSTALL.txt file in the system documentation directory for'
	einfo 'instructions on how to configure BugPort.'
	echo
	einfo 'If you are upgrading BugPort, you probably do not have to change'
	einfo 'any configuration.'
	echo; echo
}
