<?php

/*
DevelopmentItemListPortlet
$Author: millz $
$Revision: 1.3 $
$Date: 2004/01/07 19:23:53 $
$Log: DevelopmentItemListPortlet.php,v $
Revision 1.3  2004/01/07 19:23:53  millz
- changed to use static creator for DevelopmentItemResultsView

Revision 1.2  2003/07/14 17:00:03  dougqh
- Fix for 3049

Revision 1.1  2003/07/09 20:17:26  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

importClass( 'CollectionUI.AIteratorView' );

define( 'PORTLET_MAX_REPORT_DISPLAY_LENGTH', 300 );

class DevelopmentItemListPortlet extends AIteratorView
{

//========================================================================

   var $numMaxRows;
   var $title;
   var $numSeen = 0;

//========================================================================

   /*!
   @function DevelopmentItemListPortlet( $title, [ $numMaxRows ] )
   @param $title - a user-presentable String
   @param $numMaxRows - a positive integer
   @discussion - DevelopmentItemListPortlet cunstructor

      Since this class does provide a default constructor, it
      canNOT be instantiated by the BugUI.Controller.
   */

   function DevelopmentItemListPortlet( $title, $numMaxRows = UNSPECIFIED )
   {
      $this->AIteratorView();

      $this->title = $title;
      $this->numMaxRows = ( $numMaxRows === UNSPECIFIED ) ? 7 : $numMaxRows;
   }

//========================================================================

   /*!
   @function create( $title, $iterator, [ $numMaxRows ] )
   @param $title - a String - title for this portlet
   @param $iterator - an IIterator< ADevelopmentItem > - containing
      the rows to display
   @param $numMaxRows - a positive integer
   @discussion - convenience static creator method
   @result - returns a DevelopmentItemListPortlet
   */

   function &create( $title, &$iterator, $numMaxRows = UNSPECIFIED )
   {
      $portlet =& new DevelopmentItemListPortlet( $title );
      $portlet->setIterator( $iterator );
      return $portlet;
   }

//========================================================================

   /*!
   @function getTitle()
   */

   function getTitle()
   {
      return $this->title;
   }

//========================================================================

   /*!
   @function _getPrototype()
   @discussion - provides an ADatabaseItem prototype
   @result - returns an ADatabaseItem
   */

   function &_getPrototype()
   {
      importClass( 'Bugs.ADevelopmentItem' );
      return ADevelopmentItem::getPrototype();
   }

//========================================================================

   /*!
   @function getNumRows()
   @discussion - provides an integer containing the maximum number
      of rows to display
   @result - returns an integer
   */

   function getNumRows()
   {
      return $this->numMaxRows;
   }

//========================================================================

   /*!
   @function _startHTMLGeneration
   @discussion - called before the iteration is started to
      prepare for the HTML generation
   */

   function _startHTMLGeneration()
   {
      //TODO: It would be better to use a ResultsTable for this purpose,
      //but that would require supporting Display.DisplaySettingsSystem
      //in some form.

      importClass( 'HTMLComponents.Table.Table' );
      $this->table =& new Table( $this->title.' Portlet' );
      $this->table->setWidth( '100%' );
   }

//========================================================================

   /*!
   @function _handleCurrent( $devItem )
   @param $devItem - an ADevelopmentItem from the IIterator
   @discussion -
   */

   function _handleCurrent( &$devItem )
   {
      importClass( 'HTMLComponents.Text' );
      importClass( 'HTMLComponents.RawHTML' );

      if( $this->numSeen % 2 == 0 )
      {
         $rowColor =& Color::getColor( 255,255,255 );
      }
      else
      {
         $rowColor =& Color::getColor( 230,230,230 );
      }

      $this->numSeen++;
      $this->table->startRow( $rowColor );

      importClass( 'Display.Font' );

      $subjectTextObj =& new Text( $devItem->getSubject() );
      $subjectTextObj->setFont( Font::getFont( true, true ) );

      importClass( 'BugUI.OtherComponents.DevelopmentItemLinks' );
      $linkComp =& new DevelopmentItemLinks( $devItem  );
      $this->table->addCell( $subjectTextObj );
      $this->table->addCell( $linkComp, RIGHT );

      $this->table->startRow( $rowColor );

      $report = $devItem->getReport();
      if ( strlen( $report ) > PORTLET_MAX_REPORT_DISPLAY_LENGTH )
      {
         $report = substr( $report, 0 , PORTLET_MAX_REPORT_DISPLAY_LENGTH ) . '...';
      }

      $reportTextObj =& new Text( $report );
      $reportTextObj->setFont( Font::getFont( 'Tahoma', '-1' ) );

      $this->table->addCell( $reportTextObj, UNSPECIFIED, 2 );
   }

//========================================================================

   /*!
   @function _getGeneratedHTML()
   @discussion - called after the iteration is complete to
      get the HTML that has been produced and to do any necessary
      clean-up
   @result - returns a String of HTML
   */

   function _getGeneratedHTML()
   {
      if( $this->numSeen == 0 )
      {
         $this->table->startRow();
         importClass( 'HTMLComponents.RawHTML' );

         //TODO: Get rid of this non-XHTML compliant code
         $this->table->addCell(
            new RawHTML( '<font color="666666" size="-2">No items were found in this category</font>' ),
            CENTER );
      }
      else
      {
         importClass( 'BugUI.Views.DevelopmentItemResultsView' );
         $view =& DevelopmentItemResultsView::create2( $this->iterator );

         $operation =& $view->getDisplayOperation();

         $count = $this->iterator->getCount();
         $text =
            ( $count > 1 ) ?
            '[View All ' . $count . ' Results]' :
            '[View Item in Results View]';

         importClass( 'Display.Color' );
         $this->table->startRow( Color::getColor( 240, 240, 255 ) );
         $this->table->addCell(
            $operation->getLink(
               new Text( $text ) ),
               UNSPECIFIED,
               INFINITY );
      }

      //TODO: ...
      //Apparently, APortlet is not really abstract
      //It would be better to just create a PortletWindow at some point.

      importClass( 'BugUI.Portlets.APortlet' );
      $portlet =& new APortlet(
         $this->getTitle(),
         $this->table->getHTML() );

      unset( $this->table );

      return $portlet->getHTML();
   }

//========================================================================

};

?>