<?php

/*
ReviewDevelopmentItemAction
$Author: dougqh $
$Revision: 1.9 $
$Date: 2004/05/19 14:18:56 $
$Log: ReviewDevelopmentItemAction.php,v $
Revision 1.9  2004/05/19 14:18:56  dougqh
- Added support for duedate

Revision 1.8  2003/12/12 16:43:29  dougqh
Fixing 3901

Revision 1.7  2003/12/09 15:05:44  dougqh
Refactoring View->Action->Factory classes

Revision 1.6  2003/11/05 14:16:56  dougqh
- Feature 3754

Revision 1.5  2003/10/20 17:50:19  dougqh
- Fixed 3663

Revision 1.4  2003/08/13 23:05:29  dougqh
- Fixing an oversight to my fix to 3154

Revision 1.3  2003/08/13 22:05:32  dougqh
- Fixed 3154 - Method should have been called areAdditionalChecksValid

Revision 1.2  2003/08/13 18:13:50  dawn
- defect 3233 fix

Revision 1.1  2003/07/09 20:09:49  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

importClass( 'BugUI.UIHelpers.DevelopmentItem.Actions.ADevelopmentItemAction' );

//========================================================================

/*!
@class ReviewDevelopmentItemAction
@abstract - a concrete ADevelopmentItemAction that adds a new
   ADevelopmentItem to a Project
*/

class ReviewDevelopmentItemAction extends ADevelopmentItemAction
{

//========================================================================

   /*!
   @function ReviewDevelopmentItemAction()
   @discussion - constructs a new ReviewDevelopmentItemAction object
   */

   function ReviewDevelopmentItemAction()
   {
      $this->ADevelopmentItemAction();
   }

//========================================================================

   /*!
   @function getAvailableFieldNames()
   @discussion - provides an array of field names that are available
      for use with this Action
   @result - returns String[]
   */

   function getAvailableFieldNames()
   {
      return array(
         'priority',
         'report',
         'developer',
         'duedate',
         'estCodeDuration',
         'estMiscDuration',
         'newDesignDuration',
         'developmentPlan',
         'category',
         'subproject' );
   }

//========================================================================

   /*!
   @function isRequired( $fieldName )
   @param $fieldName - a String - specifying a field name
   @discussion - indicates if that field is required for this action
   @result - returns a boolean
   */

   function isRequired( $fieldName )
   {
      switch ( $fieldName )
      {
         case 'priority':
         case 'category':
         case 'subproject':
         return true;

         default:
         return false;
      };
   }

//========================================================================

   /*!
   @function areAdditionalChecksValid()
   @discussion - overridden to do some additional checks that related
      to development plan, newDesignDuration, etc.
   */

   function areAdditionalChecksValid()
   {
      //Need this in case - the design has already been set
      //because the later checks would not pass
      $devItem =& $this->getDbItem();
      if ( $devItem->isDesigned() ) return true;

      //if one of the fields is non-empty all must be non-empty
      //if one field is empty all must be empty

      $names = array(
         'developmentPlan',
         'newDesignDuration',
         'estCodeDuration',
         'estMiscDuration' );

      $emptyNames = array();
      $fullNames = array();

      foreach ( $names as $curName )
      {
         $curField =& $this->getField( $curName );

         if ( $curField->isEmpty() )
         {
            $emptyNames[] = '"' . $curField->getDisplayName() . '"';
         }
         else
         {
            $fullNames[] = '"' . $curField->getDisplayName() . '"';
         }
      }

      $valid =
         ( count( $names ) == count( $emptyNames ) && isEmpty( $fullNames ) ) ||
         ( count( $names ) == count( $fullNames ) && isEmpty( $emptyNames ) );

      if ( ! $valid )
      {
         if ( count( $fullNames ) == 1 )
         {
            $fullNamePart = $fullNames[ 0 ] . ' is set';
         }
         else
         {
            $fullNamePart = implode( ',', $fullNames ) . ' are set';
         }

         if ( count( $emptyNames ) == 1 )
         {
            $emptyNamePart = $emptyNames[ 0 ] . ' is not set';
         }
         else
         {
            $emptyNamePart = implode( ',', $emptyNames ) . ' are not set.';
         }

         $this->_addError(
            $fullNamePart . ', but ' . $emptyNamePart );
      }

      return $valid;
   }

//========================================================================


   /*!
   @function _execute()
   @discussion - actually performs the action

      Only called if all validation checks passed.
   */

   function _execute()
   {
      $devItem =& $this->getDbItem();

      $devItem->_setPriority( $this->getFieldValue( 'priority' ) );
      $devItem->_setDeveloper( $this->getFieldValue( 'developer' ) );

      $devItem->_setDueDate( $this->getFieldValue( 'duedate' ) );

      $devPlanField =& $this->getField( 'developmentPlan' );

      if ( ! $devItem->isDesigned() && ! $devPlanField->isEmpty() )
      {
         $devItem->_setDesign(
            $devPlanField->getValue(),
            $this->getFieldValue( 'newDesignDuration' ),
            $this->getFieldValue( 'estCodeDuration' ),
            $this->getFieldValue( 'estMiscDuration' ),
            $this->getUser() );
      }
      $devItem->_setCategory( $this->getFieldValue( 'category' ) );
      $devItem->_setSubproject( $this->getFieldValue( 'subproject' ) );

      $devItem->_store();
   }

//========================================================================

   /*!
   @function isEnabled()
   */

   function isEnabled()
   {
      $devItem =& $this->getDbItem();
      $project =& $devItem->getProject();

      return (
         $project->isDesigner( $this->getUser() ) &&
         ( ! $devItem->isClosed() ) );
   }

//========================================================================

   /*!
   @function getDisplayName()
   */

   function getDisplayName()
   {
      return 'Review';
   }

//========================================================================

};

?>