<?php

/*
Attachment
$Author: millz $
$Revision: 1.9 $
$Date: 2004/09/14 16:45:53 $
$Log: Attachment.php,v $
Revision 1.9  2004/09/14 16:45:53  millz
- removing a hard coded path

Revision 1.8  2004/04/01 22:05:14  dougqh
- Fixed bug with empty description handling

Revision 1.7  2003/12/18 20:40:41  dougqh
Updated to work under PHP 4.2

Revision 1.6  2003/09/29 17:02:08  dougqh
Updated path handling to work under Windows.

Revision 1.5  2003/08/29 21:55:11  millz
- added getDefaultOrderBy

Revision 1.4  2003/08/18 16:58:49  millz
- added getAbsoluteFileName and corrected getFileSize to use it

Revision 1.3  2003/07/14 13:56:23  dougqh
- Changing printError calls to unreachablePoint calls.

Revision 1.2  2003/07/10 19:15:58  dougqh
- Fixed backwards directory selection logic in getLink

Revision 1.1  2003/07/09 20:34:36  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

importClass( 'Database.ADatabaseItem' );

//========================================================================

define( 'ATTACHMENT_DIRECTORY_NAME', 'attachments' );
define( 'ATTACHMENT_ID_VAR_NAME', 'attachment' );

//========================================================================

/*!
@class Attachment
@abstract -
*/

class Attachment extends ADatabaseItem
{

//========================================================================

   var $name;
   var $dateCreated;
   var $submitter_id;

//========================================================================

   /*!
   @function Attachment()
   @discussion - constructs a new Attachment object
   @result - constructs a new Attachment object
   */

   function Attachment( $primary_id = UNSPECIFIED )
   {
      $this->ADatabaseItem( $primary_id );
   }

//========================================================================

   /*!
   @function getFromPrimaryID( $primary_id )
   */

   /* static */
   function &getFromPrimaryID( $primary_id )
   {
      return Attachment::_getFromPrimaryID(
         Attachment::getPrototype(),
         $primary_id );
   }

//========================================================================

   /*!
   @function getPrototype()
   @discussion - provides the Attachment prototype
   */

   /* static */
   function &getPrototype()
   {
      return Attachment::_getPrototype( __FILE__ );
   }

//========================================================================

   /*!
   @function create( $user, $tempFileHash, $description )
   @param $user - the User - that added the Attachment
   @param $tempFileHash - an entry from the $_FILES array
   @param $description - a String - containing a description of
      the Attachment
   @discussion - creates a new Attachment using the provided
      parameters
   @result - returns an Attachment built using a $_FILES entry
   */

   function &create( &$user, $tempFileHash, $description )
   {
      $attachment =& new Attachment();

      $attachment->_setSubmitter( $user );

      $attachment->_setTempFile(
         $tempFileHash{ 'name' },
         $tempFileHash{ 'tmp_name' } );

      $attachment->_setDescription( $description );

      return $attachment;
   }

//========================================================================

   /*!
   @function getDefaultOrderBy()
   */

   function getDefaultOrderBy()
   {
      return 'date_in DESC';
   }


//========================================================================

   /*!
   @function getAbsoluteFileName()
   @discussion - returns the absolute local directory that this attachment lives in
   */

   function getAbsoluteFileName()
   {
      return $this->getAbsoluteDirectory() . $this->getFileName();
   }

//========================================================================

   /*!
   @function getAbsoluteDirectory()
   @discussion - returns the absolute local directory that this attachment lives in
   */

   function getAbsoluteDirectory()
   {
      $path = $this->getRelativeDirectory();

      //need to modify this to support making parent directories
      makeDirectory( $path );

      return $path;
   }

//========================================================================

   /*!
   @function getAllColumns()
   @discussion - returns the columns for this item's row
   */

   function getAllColumns()
   {
      return array(
         $this->getPrimaryColumn(),
         'date_in',
         'description',
         'user_id',
         'filename',
         'report_id' );
   }

//========================================================================

   /*!
   @function getDescription()
   @discussion - standard get method
   @result - returns a String
   */

   function getDescription()
   {
      if ( isset( $this->description ) )
      {
         return $this->description;
      }
      else
      {
         return '';
      }
   }

//========================================================================

   /*!
   @function _setDescription( $desc )
   @discussion - standard set method
   */

   function _setDescription( $desc )
   {
      $this->description =  $desc;
   }

//========================================================================

   /*!
   @function getTable()
   @discussion - provides the table name (with schema if necessary)
   @result - returns a String
   */

   function getTable()
   {
      return 'attachment';
   }

//========================================================================

	function isFileNameValid( $fileNameToTest )
	{
		return strpos( $fileNameToTest, '..' ) === FALSE;
	}

//========================================================================

   /*!
   @function getFileName()
   @discussion - a normal get function
   @result - returns name item for this object
   */

   function getFileName()
   {
      if( ! $this->isFileNameValid( $this->filename ) )
      {
         return 'Bad attachment fileName - contains bad characters!';
      }

      return $this->filename;
   }

//========================================================================

   /*!
   @function getDateCreated()
   @discussion - a normal get function
   @result - returns dateCreated item for this object
   */

   function getDateCreated()
   {
      return $this->date_in;
   }

//========================================================================

   /*!
   @function _setSubmitter( &$user )
   @param $user - a User
   @discussion - sets the Submitter for this Attachment
   */

   function _setSubmitter( &$user )
   {
      $this->user_id = $user->getPrimaryID();
   }

//========================================================================

   /*!
   @function getSubmitter()
   @discussion - a normal get function
   @result - returns submitter item for this object
   */

   function &getSubmitter()
   {
      importClass( 'Shared.User' );
      return User::getFromPrimaryID( $this->user_id );
   }

//========================================================================

   /*!
   @function getFileSize()
   @discussion - a normal get function
   @result - returns fileSize item for this object
   */

   function getFileSize()
   {
      $realFileName = $this->getAbsoluteFileName();

      if( !file_exists($realFileName) )
      {
         return 'Unknown, file not found';
      }

      // print "realFileName = ".$realFileName.BR_NL;
      // print "filesize = ".filesize($realFileName).BR_NL;

      return number_format( filesize($realFileName) / 1024, 2 ) . ' KB';

   }

//========================================================================

   /*!
   @function getDevelopmentItem()
   @discussion - returns the develItem for this - assumes it is set via report_id member
   */

   function &getDevelopmentItem()
   {
      // we need the project name
      importClass( 'Bugs.ADevelopmentItem' );
      $report =& ADevelopmentItem::getFromPrimaryID( $this->report_id );

      return $report;
   }

//========================================================================

   /*!
   @function getRelativeDirectory()
   @discussion -
   */

   function getRelativeDirectory()
   {
      //getDevelopmentItem may need to be implemented
      $devItem =& $this->getDevelopmentItem();
      //ADevelopmentItem::getProject may need to be implemented
      $project =& $devItem->getProject();
      $path =
         $project->getDirectory() . DIRECTORY_SEPARATOR .
         ATTACHMENT_DIRECTORY_NAME . DIRECTORY_SEPARATOR .
         $devItem->getPrimaryID() . DIRECTORY_SEPARATOR;

      return $path;
   }

//========================================================================

   /*!
   @function _setTempFile( $originalName, $actualFile )
   @param $originalName - the name of the file that the user submitted it under (the name as it was on the client machine) - no path
   @param $actualFile - the local file that is a copy of what was on the client, FULL PATH
   @discussion - names just saved off, no return
   */

   function _setTempFile( $originalName, $actualFile )
   {
      //put those in member variables until _store is called
      if(!is_uploaded_file($actualFile) )
      {
         unreachablePoint(
            'Attempt to set non-uploaded file as ready-to-load attachment not allowed.  Exiting...' );
      }

      $this->originalName = $originalName;
      $this->actualFile = $actualFile;
   }

//========================================================================

   /*!
   @function _setDevelopmentItem( $devItem )
   @param $devItem - the devItem of this attachment
   @discussion - just uses the primary id
   */

   function _setDevelopmentItem( $devItem )
   {
      $this->report_id = $devItem->getPrimaryID();
   }

//========================================================================

   /*!
   @function _store( )
   @discussion - mvs the temp file to its "permament" location (saves it, basically)
   */

   function _store()
   {
      // get name to use for permament storage
      // NOTE!!!! This cannot be called inline as it return value will change after the move has been performed!!!
      $permName = $this->_getStorageFile();

      if( ! $this->isFileNameValid( $permName ) )
      {
         unreachablePoint(
            'Bad attachment fileName [' .
            $permName . '] - contains bad characters!' );
      }

      // Move the temp file to the location returned by _getStorageFile
      $success = move_uploaded_file( $this->actualFile, $permName );

      if ( ! $success )
      {
         unreachablePoint(
            'Problem saving Uploaded File!! ' .
            'Make sure that file to be saved is an uploaded one and that this path is usable for file storage: [ ' . $permName . ']' );
      }

      // Set the actual file variable as needed
      $this->actualFile = $permName;

      $this->filename = getFileNameFromFullPath( $this->actualFile );

      $this->date_in = time();

      return parent::_store();

   }

//========================================================================

		function showAttachmentInBrowser()
		{
		   $ext = strtolower( getExtension( $this->getFileName() ) );
				if( $ext == "jpg"  ||
				    $ext == "gif"  ||
				    $ext == "png"  ||
				    $ext == "log"  ||
				    $ext == "html" ||
				    $ext == "txt" ) return true;
		  
		  	return false;
		}

//========================================================================

   /*!
   @function _getStorageFile( $originalName = UNSPECIFIED )
   @param $originalName - optional name of the file that the user wants, will default to that set in _setTempFile
   @discussion - will attempt to avoid stepping on existing files by putting extra chars in name when needed
   @result - path string returned
   */

   function _getStorageFile( $originalName = UNSPECIFIED )
   {
      if( $originalName === UNSPECIFIED )
      {
         if( isset( $this->originalName ) )
         {
            $originalName = $this->originalName;
         }
         else
         {
            unreachablePoint(
               'No originalName determined in Attachment::_getStorageFile()' );
         }
      }

      $directory = $this->getAbsoluteDirectory() . DIRECTORY_SEPARATOR;

      // keep trying file names until we find one that's not taken

      $tmpOrig = $originalName;

      // trying to avoid possiblity of some weird infinite loop
      for ( $try = 0; file_exists( $directory . $originalName ) && $try < 100; ++$try )
      {
         $originalName = baseNameWithoutExtension($tmpOrig) . '_' . $try . '.' . getExtension($tmpOrig);
      }

      // return it
      return $directory . $originalName;
   }

//========================================================================

   /*!
   @function getLocalPath()
   @discussion -
   */

   function getLocalPath()
   {
      //local directory

      return $this->getRelativeDirectory() . $this->getFileName();
   }

//========================================================================

   /*!
   @function getWindowsDirectory()
   @discussion -
   */

   function getWindowsDirectory()
   {
      //produce path to SAMBA share
   }

//========================================================================

   /*!
   @function getUNIXDirectory()
   @discussion -
   */

   function getUNIXDirectory()
   {
      //produce path for Linux users
   }

//========================================================================

   /*!
   @function getLink()
   @discussion - provides a Link to the Attachment on the server
   @result - returns a Link
   */

   function &getLink()
   {
      //TODO: Get rid this hard coded path
      importClass( 'HTMLComponents.Links.ExternalLink' );


      $link =& new ExternalLink( 'download.php?/&' . ATTACHMENT_ID_VAR_NAME . '=' . $this->getPrimaryID() );


			if( $this->showAttachmentInBrowser() )
			{
      	$link->setTarget( '_blank' );
      }
      
      importClass( 'HTMLComponents.Text' );
      $link->setLinkBody( new Text( $this->getFileName() ) );

      return $link;
   }

	function getMimeType()
	{
		// some of these from: http://www.isi.edu/in-notes/iana/assignments/media-types/media-types
		
		$ext = strtolower(getExtension( $this->getFileName() ));
		
		if( $ext == "jpg" ) return "image/jpeg";
		if( $ext == "png" ) return "image/png";
		if( $ext == "gif" ) return "image/gif";
		
		if( $ext == "xml" || $ext == "vpml" ) return "text/xml";
		
		if( $ext == "txt" || 
				$ext == "php" || 
				$ext == "pl" || 
				$ext == "java" || 
				$ext == "c" || 
				$ext == "cpp" || 
				$ext == "htm" ||
				$ext == "tab" ||
				$ext == "log" || 
				$ext == "fasta" ) return "text/plain";
		
		if( $ext == "html"  ) return "text/html";
		if( $ext == "doc" ) return "application/msword";
		if( $ext == "zip" ) return "application/zip";
		if( $ext == "ps" ) return "application/postscript";
		if( $ext == "pdf" ) return "application/pdf";
		if( $ext == "xls" ) return "application/vnd.ms-excel";
		
		
		return 'application/octet-stream';
	}

//========================================================================

   /*!
   @function _getDefaultOrderBy()
   */

   function _getDefaultOrderBy()
   {
      return 'date_in DESC';
   }

//========================================================================

};

?>