<?php

/*
Version
$Author: dougqh $
$Revision: 1.3 $
$Date: 2003/12/18 20:40:41 $
$Log: Version.php,v $
Revision 1.3  2003/12/18 20:40:41  dougqh
Updated to work under PHP 4.2

Revision 1.2  2003/08/11 17:06:39  dougqh
- Fixing 3275

Revision 1.1  2003/07/09 20:34:36  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

importClass( 'Database.ATrivialDatabaseItem' );

//========================================================================

define( 'CLIENT_VERSION_TYPE', 1 );
define( 'SERVER_VERSION_TYPE', 0 );

//========================================================================

class Version extends ADatabaseItem
{

//========================================================================

   /*!
   @function Version()
   @discussion - constructor for the serverity database item
   */

   function Version( $primary_id = UNSPECIFIED )
   {
      $this->ADatabaseItem( $primary_id );
   }

//========================================================================

   /*!
   @function getPrototype()
   @discussion - constructs a Version prototype
   */

   /* static */
   function &getPrototype()
   {
      return Version::_getPrototype( __FILE__ );
   }

//========================================================================

   /*!
   @function getFromPrimaryID( $primary_id )
   */

   /* static */
   function &getFromPrimaryID( $primary_id )
   {
      return Version::_getFromPrimaryID(
         Version::getPrototype(),
         $primary_id );
   }

//========================================================================

   /*!
   @function _setName( $name )
   @param $name - a String
   @discussion - sets the name of this Version - should only be called
      on Version objects where isNew is true
   */

   function _setName( $name )
   {
      $this->name = $name;
   }

//========================================================================

   /*!
   @function getName()
   @discussion - provides the name of this Version
   @result - returns a String
   */

   function getName()
   {
      return $this->name;
   }

//========================================================================

   /*!
   @function _getRegularIdentifyingString()
   */

   function _getRegularIdentifyingString()
   {
      return $this->getName();
   }

//========================================================================

   /*!
   @function getMoreRecentVersions()
   @discussion - provides a collection of Versions more recent than
      this Version
   @result - returns an IIterator< Version >
   */

   function &getMoreRecentVersions()
   {
      return $this->doSearch(
         'isClient=? AND project_id=? AND name>?',
         array( $this->isClient, $this->project_id, $this->getPrimaryID() ) );
   }

//========================================================================

   /*!
   @function _getClientWherePair()
   @discussion - provides a where pair that retrieves client versions
   @result - returns a WherePair
   */

   /* static */
   function _getClientWherePair()
   {
      return array( 'isClient=?', array( CLIENT_VERSION_TYPE ) );
   }

//========================================================================

   /*!
   @function _getServerWherePair()
   @discussion - provides a where pair that retrieves server versions
   @result - returns a WherePair
   */

   /* static */
   function _getServerWherePair()
   {
      return array( 'isClient=?', array( SERVER_VERSION_TYPE ) );
   }

//========================================================================

   /*!
   @function getTable()
   */

   function getTable()
   {
      return 'version';
   }

//========================================================================

   /*!
   @function getAllColumns()
   */

   function getAllColumns()
   {
      return array(
         $this->getPrimaryColumn(),
         'name',
         'project_id',
         'isClient',
         'date_in' );
   }

//========================================================================

   /*!
   @function _getDefaultValue( $column )
   */

   function _getDefaultValue( $column )
   {
      switch ( $column )
      {
         case 'date_in':
         return time();

         default:
         unreachablePoint( 'Required column is NOT set.' );
      };
   }

//========================================================================

   /*!
   @function _setProjectInfo( $project, $type )
   @param $project - a Project
   @param $type - one of CLIENT_VERSION_TYPE_TYPE or SERVER_VERSION_TYPE_TYPE
   @discussion - called by Project::_add<X>Version
   */

   function _setProjectInfo( &$project, $type )
   {
      $this->project_id = $project->getPrimaryID();
      $this->isClient = $type;
   }

//========================================================================

   /*!
   @function _getProject()
   @discussion - provides the Project with which this Version
      is associated
   @result - returns a Project
   */

   function &_getProject()
   {
      importClass( 'Shared.Project' );
      return Project::getFromPrimaryID( $this->project_id );
   }

//========================================================================

   /*!
   @function getDefaultOrderBy()
   */

   function getDefaultOrderBy()
   {
      return 'name DESC';
   }

//========================================================================

   /*!
   @function _tryFindThisVersion()
   @discussion - indicates if this Version has already been stored
      in the database
   @result - returns NON_EXISTENT_ID if the row does NOT exist;
      otherwise returns the ID of the row that should be used
   */

   function _tryFindThisVersion()
   {
      if ( $this->isClient )
      {
         $wherePairs[] = Version::_getClientWherePair();
      }
      else
      {
         $wherePairs[] = Version::_getServerWherePair();
      }

      $project =& $this->_getProject();
      $wherePairs[] = $project->_getVersionWherePair();
      $wherePairs[] = array( 'name=?', $this->name );

      $finalPair =& ADatabaseItem::_getAndWherePairs( $wherePairs );

      $iterator =& $this->doSearch( $finalPair[ 0 ], $finalPair[ 1 ] );

      $iterator->reset();
      if ( $iterator->isEnd() )
      {
         return NON_EXISTENT_ID;
      }
      else
      {
         $version =& $iterator->getCurrent();
         return $version->getPrimaryID();
      }
   }

//========================================================================

   /*!
   @function _store()
   @discussion - overloaded to prevent duplicate Version insertion
   */

   function _store()
   {
      //TODO: Consider moving this logic into project

      if ( $this->isNew() )
      {
         $version_id = $this->_tryFindThisVersion();

         if ( $version_id == NON_EXISTENT_ID )
         {
            $version_id = parent::_store();
         }
         else
         {
            //NASTY HACK - the primary_id needs to be set correctly
            $this->primary_id = $version_id;
         }
      }
      else
      {
         return parent::_store();
      }
   }

//========================================================================

};

?>