<?php

/*
Object
$Author: dougqh $
$Revision: 1.2 $
$Date: 2003/10/20 21:02:49 $
$Log: Object.php,v $
Revision 1.2  2003/10/20 21:02:49  dougqh
- Added method getObjectID

Revision 1.1  2003/10/20 21:01:13  dougqh
Initial revision

Revision 1.28  2003/07/09 22:19:13  dougqh
Killed unnecessary member variables.

Revision 1.27  2003/07/09 22:14:26  dougqh
Ported from GP and killed the unnecessary code

Revision 1.19  2002/12/13 13:52:18  millz
- added _clearExcludedVariablesCache

Revision 1.18  2002/11/06 20:08:18  dougqh
Removed trailing newline from getDebugging.

Revision 1.17  2002/10/18 21:39:37  dougqh
Added the method _unsetAll in a failed attempt to collect garbage.

Revision 1.16  2002/09/26 18:55:48  dougqh
Restoring the hasObjectLocator method.

Revision 1.15  2002/09/24 20:03:30  dougqh
Changing quotes to ticks.

Revision 1.14  2002/09/03 23:04:06  dougqh
Added default implementations of _strongEquals, _equals, _lessThan, and _greaterThan.

Revision 1.13  2002/08/26 20:13:04  dougqh
Added additional code to allow both the standard Serializer and the ApplicationSerializer to both work.

Revision 1.12  2002/08/17 17:02:21  dougqh
Adding a new equals method.

Revision 1.11  2002/08/16 17:45:57  dougqh
Fixes to getCopy.

Revision 1.10  2002/08/14 22:26:44  dougqh
Added invariant call to Object::_onDeserialize.

Revision 1.9  2002/08/14 13:22:52  dougqh
Adding timestamp to object_id to ensure uniqueness.

Revision 1.8  2002/08/13 13:31:59  dougqh
Modifications to work with new Serializer.

Revision 1.7  2002/08/05 22:09:04  dougqh
Debugging

Revision 1.6  2002/08/05 15:03:41  dougqh
Removing _test static method.

Revision 1.5  2002/07/23 17:34:45  dougqh
Moved the more complicated serialization handling mechanisms into
HeavyObject to reduce the number of method calls done when
unserializing Objects.

Revision 1.4  2002/07/18 17:44:48  dougqh
Added support getting a default ObjectLocator when ObjectLocator has
been set.
Added caching to getClass and getAbsoluteClass.
Added caching to speed up finalize re-registering.
Added caching to speed up wake for Objects without auto-recovered variables.

Revision 1.3  2002/07/12 13:44:30  dougqh
Comment out some debug statements.

Revision 1.2  2002/07/04 18:12:57  dougqh
Added _setAllowSerialized copies method. The functionality provided by this method is needed by ToolOperation.

Revision 1.1  2002/07/01 19:30:21  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

/*!
@class Object
@abstract - the abstract base class for all Objects
@discussion - This class defines the basic mechanisms for serializing
   and finalizing Objects.
*/

/* abstract */
class Object
{

//========================================================================

   var $object_id;

   var $changed;

//========================================================================

   /*!
   @function Object()
   @discussion - constructs a new Object object
   @result - constructs a new Object object
   */

   function Object()
   {
      $this->_resetObjectID();
   }

//========================================================================

   /*!
   @function _strongEquals( $object )
   @param $object -
   @discussion - Called to determine if two objects are the same object.

      Object just assumes two objects with the same object_id are equal.
      This behavior may be overwritten by a child class, but no assumptions
      should be made about the Object type passed to strongEquals.
   @result - returns a boolean
   */

   function _strongEquals( &$object )
   {
      assert( '$this->_invariant()' );

      return ( $this->object_id === $object->object_id );
   }

//========================================================================

   /*!
   @function _equals( $object )
   @discussion - called to determine if two Objects are equal

      This default implementation is true if two objects are equivalent
      under strong equivalence.
   @result - returns a boolean
   */

   function _equals( &$object )
   {
      return $this->_strongEquals( $object );
   }

//========================================================================

   /*!
   @function _lessThan( $object )
   @param $object - an Object
   @discussion - called to determine if $this is less than the specified
      Object

      NOTE: This default implementation always returns false.
   @result - returns a boolean
   */

   function _lessThan( &$object )
   {
      return false;
   }

//========================================================================

   /*!
   @function _greaterThan( $object )
   @param $object - an Object
   @discussion - called to determine if $this is greater than the specified
      Object

      NOTE: This default implementation always returns false.
   @result - returns a boolean
   */

   function _greaterThan( &$object )
   {
      return false;
   }

//========================================================================

   /*!
   @function getCopy()
   @discussion - gets a copy of this Object

      NOTE: Using this method is preferred over using copy semantics.
      This method will do additional clean up that is also necessary.

      The copy is a shallow copy of the Object.
   @result - returns a copy of this Object
   */

   function &getCopy()
   {
      //NOTE: Intentional use of copy semantics
      //  remember this copy is shallow when
      //  references are involved
      $result = $this;

      $result->_resetObjectID();
      unset( $result->objectLocator );
      unset( $result->owner );
      unset( $result->fixedStorageGroup );
      unset( $result->excludedVariables );

      //side effeects of the serializer - must be cleaned up
      unset( $result->serialized );
      unset( $result->deserialized );

      return $result;
   }

//========================================================================

   /*!
   @function getDebugging()
   @discussion - called go get debugging information for
      this Object

      NOTE: Child classes are encouraged to override this method
      to provide better information.
   @result - returns a String
   */

   function getDebugging()
   {
      return 'Object ID: ' . $this->object_id . NL .
             'Class: ' . $this->getClass() . NL .
             'Abs Class: ' . $this->getAbsoluteClass();
   }

//========================================================================

   /*!
   @function getAbsoluteClass()
   @discussion - returns the absolute class specifier of this Object
   @result - returns a String containing the absolute class specifier
      of this Object
   */

   function getAbsoluteClass()
   {
      return classToAbsoluteClass( get_class( &$this ) );
   }

//========================================================================

   /*!
   @function _setChanged( [ $value ] )
   @param $value - a parameter indicated whether or not the object has
      changed - by default true
   @discussion - sets the changed status of this object
   */

   function _setChanged( $value = true )
   {
      $this->changed = $value;
  }

//========================================================================

   /*!
   @function _hasChanged()
   @discussion - indicates if this Object has changed a change in state
   @result - returns true if change has been registered;
      otherwise returns false
   */

   function _hasChanged()
   {
      return $this->changed;
   }

//========================================================================

   /*!
   @function getClass()
   @discussion - returns the name of this Object's class
   @result - returns a string containing the name of this Object's class
   */

   function getClass()
   {
      return get_class( &$this );
   }

//========================================================================

   /*!
   @function _resetObjectID()
   @discussion - sets the object_id of this Object
   */

   function _resetObjectID()
   {
      $this->object_id = get_class( &$this ) . '_' . Object::_getNextObjectID() . '_' . str_replace( array( ' ', '.' ), '_', microtime() );
   }

//========================================================================

   /*!
   @function getObjectID()
   */

   function getObjectID()
   {
      return $this->object_id;
   }

//========================================================================

   /*!
   @function _unsetAll()
   @discussion - called if an Object wants to kill all its member variables
   */

   function _unsetAll()
   {
      foreach ( array_keys( get_object_vars( &$this ) ) as $key )
      {
         unset( $this->$key );
      }
   }

//========================================================================

   /*!
   @function _getNextObjectID()
   @discussion - a static method that produces a sequence integers starting
      at 0.  Each call produces the successor of the previous return
      value.

      This is used by Object to provide a uniquely identifying integer to
      each Object.
   @result - returns an integer
   */

   /* static */
   function _getNextObjectID()
   {
      global $GL_nextObjectID;

      if ( ! isset( $GL_nextObjectID ) )
      {
         if ( isset( $_SESSION{ 'nextObjectID' } ) )
         {
            $GL_nextObjectID = $_SESSION{ 'nextObjectID' };
         }
         else
         {
            $GL_nextObjectID = 0;
         }
      }

      $result  = $GL_nextObjectID++;

      $_SESSION{ 'nextObjectID' } = $result;

      return $result;
   }

//========================================================================

   /*!
   @function _setCircularReference( $memberName, $object )
   @param $memberName - a member variable name
   @param $object - object to assign to the member variable
   @discussion - makes $this->$memberName =& $object, but
      also records the assignment so it can be undone at
      script termination so that a memory leak does NOT occur
   */

   function _setCircularReference( $memberName, &$object )
   {
      $this->$memberName =& $object;

      //just need to allocate, it will be invoked by the clean-up
      //process
      importClass( 'Common.Memory.UnsetMemberFinalizer' );
      new UnsetMemberFinalizer( $this, $memberName );

      //TODO: Provide a means to handle the memory leak here
   }

//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */

   function _invariant()
   {
      return isset( $this->object_id );
   }

//========================================================================

};

?>
