<?php

/*
QueryResultsIterator
$Author: millz $
$Revision: 1.11 $
$Date: 2003/12/17 21:37:08 $
$Log: QueryResultsIterator.php,v $
Revision 1.11  2003/12/17 21:37:08  millz
- removing duplicate preg_split logic

Revision 1.10  2003/12/11 18:09:39  millz
- correcting comment search bugs

Revision 1.9  2003/11/05 14:19:35  millz
- debugging

Revision 1.8  2003/10/20 18:23:31  dougqh
- Removed erroneous this in getCount

Revision 1.7  2003/10/20 18:21:41  dougqh
- Removing debugging

Revision 1.6  2003/10/20 18:21:12  dougqh
- Error checking for 3621

Revision 1.5  2003/07/21 22:21:32  millz
- fixing redundancy bug

Revision 1.4  2003/07/17 20:36:50  millz
- reworked count logic and sql creation

Revision 1.3  2003/07/14 13:56:37  dougqh
- Changing printError calls to unreachablePoint calls.

Revision 1.2  2003/07/10 18:55:39  millz
- updated getOrderFromRequest

Revision 1.1  2003/07/09 21:57:17  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

importClass( 'Collections.IIterator' );

//========================================================================

define( 'QUERY_PARAM', 'colHash' );
define( 'COMPARISON_KEY', 'comp' );
define( 'VALUE_KEY', 'val' );

define( 'WHERE_PARAM', 'where' );
define( 'BIND_PARAM', 'binds' );
define( 'ORDER_BY_PARAM', 'orderBy' );
define( 'ORDER_BY_DIRECTION_PARAM', 'orderByDirection' );



//========================================================================

/*!
@class QueryResultsIterator
@abstract - an implementation of IIterator that traverses
*/

class QueryResultsIterator extends Object /* implements IIterator */
{

//========================================================================

   var $resultSet;
   var $dbPrototype;
   var $whereClause;
   var $binds;
   var $orderBy;
   var $joinStruct;

//========================================================================
   /*!
   @function QueryResultsIterator( $dbPrototype, [ $whereClause, [ $binds , [ $orderBy ,[  $joinStruct ]]]] )
   @param $dbPrototype - an ADatabaseItem prototype
   @param $whereClause - a String - containing a where clause
   @param $binds - a String[] - bind values for where clause - defaults to empty array
   @param $orderBy - an orderBy expression - defaults to empty String
   @param joinStruct - an optional joinStruct from Database.Join
   @discussion - constructs a new QueryResultsIterator object
   @result - constructs a new QueryResultsIterator object
   */

   function QueryResultsIterator(
      &$dbPrototype,
      $whereClause = UNSPECIFIED,
      $binds  = UNSPECIFIED,
      $orderBy = UNSPECIFIED,
      $joinStruct = UNSPECIFIED )
   {
      $this->dbPrototype =& $dbPrototype;
      $this->whereClause = ( $whereClause === UNSPECIFIED ) ? '' : $whereClause;
      $this->binds = ( $binds === UNSPECIFIED ) ? array() : $binds;
      $this->orderBy = ( $orderBy === UNSPECIFIED ) ? '' : $orderBy;
      $this->joinStruct = $joinStruct;

      $this->Object();
   }

//========================================================================

   /*!
   @function getPrototype()
   @discussion - provides the prototype with which this QueryResultsIterator
      was created
   @result - returns an ADatabaseItem
   */

   function &getPrototype()
   {
      return $this->dbPrototype;
   }

//========================================================================

   /*!
   @function getWherePairFromRequest( $dbPrototype, [ $queryVarName ] )
   @param $dbPrototype - an ADatabaseItem
   @param $queryVarName - a key in $_REQUEST to check for query
      description data - defaults to QUERY_PARAM
   @discussion - uses information in the $_REQUEST{ $queryVarName } to
      construct a WherePair

      $_REQUEST{ $queryVarName } should be of the form
      array(
         <column> => array(
            COMPARISON_KEY => <comp>,
            VALUE_KEY => <value> ),
         ... )
   @result - returns a WherePair
   */

   /* static */
   function getWherePairFromRequest(
      &$dbPrototype,
      $queryVarName = UNSPECIFIED )
   {
      $queryVarName =
         ( $queryVarName === UNSPECIFIED ) ?
         QUERY_PARAM :
         $queryVarName;

      if ( ! isset( $_REQUEST{ $queryVarName } ) )
      {
         //bailing early, since the information is NOT present
         return array( '', array() );
      }

      $queryHash = $_REQUEST{ $queryVarName };


      $fullWhere = '';
      $binds = array();

      //TODO: For additional safety, we could check that
      //the column is actually in $dbPrototype->getAllColumns.

      foreach ( $queryHash as $column => $pair )
      {
         //unset each cycle, so isset checks will work later
         unset( $where );
         unset( $bind );

         //
         if (
            ! isset( $pair{ VALUE_KEY } ) ||
            isEmpty( $pair{ VALUE_KEY } ) )
         {
            continue;
         }

         $value = $pair{ VALUE_KEY };

        // if the column is a _id and it has commas or whitespace then
        // lets convert it to an array so that it will be converted to a IN() clause 
         importClass( 'Database.ADatabaseItem' );
         if( endsWith( ID_SUFFIX, $column ) && is_string($value) )
         {
            if( preg_match("/\D+/", $value) )
                { $value = getNumericIDArrayFromString( $value ); }
         }


         if( is_array( $value ) ) // an array of values, do an IN() no matter what the user said
         {
            $comparison = IN_LIST;
         }
         elseif ( ! isset( $pair{ COMPARISON_KEY } ) ) // no compare set, assume =
         {
            $comparison =  EQUALS;
         }
         else
         {
            $comparison = $pair{ COMPARISON_KEY };
         }

         //not using default to be defensive
         switch ( $comparison )
         {
            case CONTAINS:
            $where = $column . ' LIKE ?';
            $bind = '%' . $value . '%';
            break;

            case LESS_THAN:
            $where = $column . '<?';
            $bind = $value;
            break;

            case GREATER_THAN:
            $where = $column . '>?';
            $bind = $value;
            break;

            case LESS_THAN_OR_EQUAL:
            $where = $column . '<=?';
            $bind = $value;
            break;

            case GREATER_THAN_OR_EQUAL:
            $where = $column . '>=?';
            $bind = $value;
            break;

            case IN_LIST:
            $where = $column . ' IN (' . implode( ',', $value ) . ')';
            break;

            case EQUALS:
            default:
            $where = $column . '=?';
            $bind = $value;
            break;
         }

         if ( isset( $where ) )
         {
            $fullWhere .= ' AND ' . $where;

            if ( isset( $bind ) )
            {
               $binds[] = $bind;
            }
         }
      }

      if ( $fullWhere != '' )
      {
         //strip the excess ' AND ' at the start
         //5 is strlen( ' AND ' )
         return array(
            substr( $fullWhere, 5 ),
            $binds );
      }
      else
      {
         return array( '', array() );
      }
   }

//========================================================================

   /*!
   @function getOrderFromRequest( $dbPrototype, [ $orderByVarName , [$orderByDirVarName] ] )
   @parma $dbPrototype - an ADatabaseItem
   @param $orderByVarName - the column name
   @param $orderByDirVarName - ASC or DESC
   @discussion - reads the orderBy settings from $_REQUEST
   @result - returns a String
   */

   /* static */
   function getOrderFromRequest( &$dbPrototype, $orderByVarName = UNSPECIFIED, $orderByDirVarName = UNSPECIFIED )
   {
      //TODO: For additional safety validate the orderBy against a dbPrototype

      $orderByVarName =
         ( $orderByVarName === UNSPECIFIED ) ?
         ORDER_BY_PARAM :
         $orderByVarName;

      $orderByDirVarName =
         ( $orderByDirVarName === UNSPECIFIED ) ?
         ORDER_BY_DIRECTION_PARAM :
         $orderByDirVarName;

      if(! isset( $_REQUEST{ $orderByVarName } )){return UNSPECIFIED;}
      else
      {
         $sort =  $_REQUEST{ $orderByVarName };
         if(isset( $_REQUEST{ $orderByDirVarName } )){ $sort .=  " ".$_REQUEST{ $orderByDirVarName }; }
         return $sort;
      }

   }

//========================================================================

   /*!
   @function createFromRequest( $dbPrototype )
   @param $dbPrototype - an ADatabaseItem
   @discussion - constructs a QueryResultsIterator from information
      in $_REQUEST
   @result - returns a QueryResultsIterator
   */

   /* static */
   function &createFromRequest( &$dbPrototype )
   {
      if ( isset( $_REQUEST{ WHERE_PARAM } ) )
      {
         $where = $_REQUEST{ WHERE_PARAM };
      }
      else
      {
         $where = UNSPECIFIED;
      }

      if ( isset( $_REQUEST{ BIND_PARAM } ) )
      {
         $binds = $_REQUEST{ BIND_PARAM };
      }
      else
      {
         $binds = UNSPECIFIED;
      }

      if ( isset( $_REQUEST{ ORDER_BY_PARAM } ) )
      {
         $orderBy = $_REQUEST{ ORDER_BY_PARAM };
      }
      else
      {
         $orderBy = UNSPECIFIED;
      }

      // print_r( $_REQUEST );

      return new QueryResultsIterator(
         $dbPrototype,
         $where,
         $binds,
         $orderBy );
   }

//========================================================================

   /*!
   @function getParameters()
   @discussion - provides a hash of parameters that can
      be used in URL to represent this QueryResultsIterator

      This QueryResultsIterator can later by restored by calling
      QueryResultsIterator::createFromRequest.
   @result - returns a parameter hash
   */

   function getParameters()
   {
      $params = array();

      if ( ! isEmpty( $this->whereClause ) )
      {
         $params{ WHERE_PARAM } = $this->whereClause;
      }

      if ( ! isEmpty( $this->orderBy ) )
      {
         $params{ ORDER_BY_PARAM } = $this->orderBy;
      }

      foreach ( $this->binds as $key => $curBind )
      {
         $params{ BIND_PARAM . '[' . $key . ']' } = $curBind;
      }

      return $params;
   }

//========================================================================

   /*!
   @function _createRecordSet()
   @discussion - gets an ADO RecordSet and places it in the recordSet
      member variable if its not already set
   */

   function _createRecordSet()
   {
      if ( ! isset( $this->recordSet ) )
      {
         importClass( 'Database.DBConnectionProvider' );
         // if we are joining with another table we need to fully qualify our columns in the select clause
         // otherwise let's avoid the extra characters involved and not do that qualification
         
         $cacheTime = $this->dbPrototype->_getMultipleRowCacheTime();
         
         if( $this->joinStruct !== UNSPECIFIED )
         {
            // determine if we need to make groupBy-based select clause changes
            $selectClause = DBConnectionProvider::createQualifiedAliasedSelectClause( $this->dbPrototype->getTable(), $this->dbPrototype->getReadColumns() );

            // we are assuming that only the main join table has columns in the select statement
            // NOTE: WE ARE NOT ACTUALLY DOING THE GROUP BY STUFF FOR NOW
            if( $this->shouldUseGroupBy() )
            {
               $selectClause = ' DISTINCT '.$selectClause;
            }
            
            // want the min valid cache age
            if( isset( $this->joinStruct{ PROTOTYPE_HASH_KEY } ) )
            {
                // print "Checking cache time for comment table too".BR_NL;
                $cacheTime = min( $cacheTime, $this->joinStruct{ PROTOTYPE_HASH_KEY }->_getMultipleRowCacheTime() );
                // print "Checking cache time for comment table too ($cacheTime)".BR_NL;
            }
            
         }
         else
         {
            $selectClause = implode( ',', $this->dbPrototype->getReadColumns() );
         }

         $query = $this->_getSQL( $selectClause );

         // we need this for escaping purposes later
         //importClass( 'Database.DBConnectionProvider' );
         $conn =& DBConnectionProvider::getReadOnlyConnection();
//         print 'Query: ' . $query . BR_NL;
//         print_r( $this->binds );
//         print BR_NL;

         //Do NOT forget the binds
         $stmt = $conn->Prepare( $query );

         // run the query and cache it for a while (or get it from the cache)
         $this->recordSet =
            $conn->CacheExecute(
               $cacheTime,
               $stmt,
               $this->binds );

         if ( $this->recordSet === false )
         {
            unreachablePoint(
               'Error in running query in QueryResultsIterator::_createRecordSet - tried to run -> ['.$query.']' );
         }

         $this->error = ( $this->recordSet === false );


         //ob_flush();
      }
   }

//========================================================================

   /*!
   @function shouldUseGroupBy()
   @discussion - bool
   */

   function shouldUseGroupBy()
   {
      return $this->joinStruct{'isXtoMany'};
   }

//========================================================================

   /*!
   @function _getSQL( $selectClause, [ $includeOrderBy ] )
   @param $selectClause - the columns to select, looks like: "user_id, col2"
   @param $includeOrderBy - indicates if the order by clause should be included
      - defaults to true
   @discussion - returns a valid SQL for this DB
   */

   function _getSQL( $selectClause, $includeOrderBy = true )
   {

       $query =
         'SELECT ' . $selectClause .
         ' FROM ' . $this->dbPrototype->getTable();

      // here we will add on the join clause info, if our joinStruct == UNSPEC then this call with just return ''
      importClass( 'Database.Join' );
      $query .= Join::createSQL92Clause( $this->joinStruct );

      if ( $this->whereClause != '' )
      {
         $query .= ' WHERE ' . $this->whereClause;
      }

      // WHILE WE ARE DOING the distinct - as long as it works for us - we do not need the group by clause
      // if ( $this->shouldUseGroupBy() )
      // {
      //    $query .= ' GROUP BY ' . $this->dbPrototype->getTable() . '.' . $this->dbPrototype->getPrimaryColumn();
      // }

      if ( $this->orderBy != '' && $includeOrderBy )
      {
         $query .= ' ORDER BY ' . $this->orderBy;
      }

      return $query;
   }

//========================================================================

   /*!
   @function getCount( $goAheadAndDoRealQuery = false )
   @discussion - actually runs the query - gets an ADO record set
   */

   function getCount( $goAheadAndDoRealQuery = false )
   {
      if ( $goAheadAndDoRealQuery )
      {
         //we will need the data soon, go ahead and get it
         $this->_createRecordSet();
         //This will set recordSet
      }

      // we do not need the data soon or it does not exist, simplify the count query
      if (
         ! isset( $this->recordSet ) ||
         $this->recordSet === false )
      {
         // since we have not run the query yet, let's assume a count can be found faster
         // by using a simpler select clause
         $cacheHandler =& $this->dbPrototype->getCacheHandler();
         $query =
            $this->_getSQL(
               'COUNT(DISTINCT ' . $this->dbPrototype->getTable() . '.' . $this->dbPrototype->getPrimaryColumn().') num',
               false );

         $conn =& DBConnectionProvider::getReadOnlyConnection();

         $num = $conn->cacheGetOne(
            $cacheHandler->getMultipleCacheTime(),
            $query,
            $this->binds );

         if( ! isEmpty( $conn->ErrorMsg() ) )
         {
            print 'QueryResultsIterator->getCount' . BR_NL;
            print 'Error running Query "' . $query . '"' . BR_NL;
            print 'Error Message "' . $conn->ErrorMsg() . BR_NL;
         }
      }
      else
      {
         $num = $this->recordSet->RecordCount();
      }

      return $num;
   }

//========================================================================

   /*!
   @function reset()
   @discussion - actually runs the query - gets an ADO record set
   */

   function reset()
   {
      $this->_createRecordSet();

      // set cursor in $this->recordSet to 0
      // this is not supported everywhere
      $this->recordSet->MoveFirst();
   }

//========================================================================

   /*!
   @function isEnd()
   @discussion - reached end if at end of the ADO record set
   */

   function isEnd()
   {
      return $this->error || $this->recordSet->EOF;
   }

//========================================================================

   /*!
   @function moveNext()
   @discussion - advance the ADO record set
   */

   function moveNext()
   {
      $this->recordSet->moveNext();
   }

//========================================================================

   /*!
   @function getCurrent()
   @discussion - use getFromresultSet on the dbPrototype to get
      an appropriate instance
   */

   function &getCurrent()
   {
//      $this->error =
//         ( ! isset( $this->recordSet->fields ) ) ||
//         ( ! is_array( $this->recordSet->fields ) );
//
//      if ( $this->error ) return false;

      return $this->dbPrototype->getFromRecordSet( $this->recordSet->fields );
   }

//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */

   function _invariant()
   {
      return parent::_invariant();
   }

//========================================================================

};

?>