<?php

/*
StandardCacheHandler
$Author: dougqh $
$Revision: 1.1 $
$Date: 2003/07/09 21:57:17 $
$Log: StandardCacheHandler.php,v $
Revision 1.1  2003/07/09 21:57:17  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

// 20000000 secs ~= 231.5 Days
define( 'MAX_SINGLE_SECONDS', 20000000 );

define( 'MAX_MULTIPLE_SECONDS', 20000000 );

// This is to deal with the fact that our time calculation will
// be made a bit before the actual query is run.
define( 'AGE_SECS_BUFFER', 3 );

//========================================================================

importClass( 'Database.ADBCacheHandler' );

//========================================================================

/*!
@class StandardCacheHandler
@discussion - This class is entirely safe when it comes to ACID properties,
   but for this simple application its sufficient.
*/

class StandardCacheHandler extends ADBCacheHandler
{

//========================================================================

   var $table;
   var $maxSingleSeconds;
   var $maxMultipleSeconds;

//========================================================================

   /*!
   @function StandardCacheHandler( $table, $maxSingleSeconds , $maxMultipleSeconds )
   @param - $table - the name of the table in question
   @param - $maxSingleSeconds - the max possible number of secs a single row can be cached
   @param - $maxMultipleSeconds - the max possible number of secs a multiple row set can be cached
   @discussion - constructs a new StandardCacheHandler object
   @result - constructs a new StandardCacheHandler object
   */

   function StandardCacheHandler( $table, $maxSingleSeconds, $maxMultipleSeconds)
   {
      $this->ADBCacheHandler();

      $this->table = $table;
      $this->maxSingleSeconds = $maxSingleSeconds;
      $this->maxMultipleSeconds = $maxMultipleSeconds;
   }

//========================================================================

   /*!
   @function create( $table, [ $maxSingleSeconds  = MAX_SINGLE_SECONDS, [ $maxMultipleSeconds  = MAX_MULTIPLE_SECONDS ] ] )
   @param - $table - the name of the table in question
   @param - $maxSingleSeconds - the max possible number of secs a single row can be cached
   @param - $maxMultipleSeconds - the max possible number of secs a multiple row set can be cached
   @discussion - constructs a new StandardCacheHandler object
   @result - constructs a new StandardCacheHandler object
   */

   function &create( $table, $maxSingleSeconds = MAX_SINGLE_SECONDS,  $maxMultipleSeconds = MAX_MULTIPLE_SECONDS )
   {
      static $instanceHash;

      if ( isEmpty( $table ) )
      {
         unreachablePoint(
            'StandardCacheHandler::create called with invalid (empty) table name!' );
      }

      if ( ! isset( $instanceHash{ $table } )  )
      {
         $instanceHash{ $table } =& new StandardCacheHandler( $table, $maxSingleSeconds, $maxMultipleSeconds );
      }

      return $instanceHash{ $table };
   }

//========================================================================

   /*!
   @function  getSingleCacheTime( $rowID )
   */

   function getSingleCacheTime( $rowID )
   {
      return
         min(
            max(
               time() - $this->_getLastUpdateTime() - AGE_SECS_BUFFER,
               0 ),
            $this->maxSingleSeconds );
   }

//========================================================================

   /*!
   @function  getMultipleCacheTime()
   */

   function getMultipleCacheTime()
   {
      return
         min(
            max(
               time() - $this->_getLastUpdateTime() - AGE_SECS_BUFFER,
               0 ),
            $this->maxMultipleSeconds );
   }

//========================================================================

   /*!
   @function  _getLastUpdateTime()
   @discussion - gets epoch time for last update of this table
   @result - returns a long containing the UNIX Epoch Time
   */

   function _getLastUpdateTime()
   {
      importClass( 'Database.DBConnectionProvider' );
      $dbcon =& DBConnectionProvider::getConnection();

      // we need to update the tablecaching row for this table

      // first let's see if we need to do an insert or not
      $sql =
         'SELECT date_modified ' .
         'FROM ' . $this->getTable() . ' ' .
         'WHERE tableName=?';

      $time = $dbcon->getOne( $sql, array( $this->table ) );

//      print getLocation() . BR_NL;
//      print 'Table: ' . $this->table . BR_NL;
//      print getStringDepiction( $time ) . BR_NL;
//      print BR_NL;

      if ( $time === false )
      {
         //create an entry, so we have a reference point next time
         $this->markAllDirty();

         // If we had a missing or invalid date, let's default to saying that
         // the cache is only valid from this time on this should never happen
         // as long as our dirty catch above works.

         return time();
      }
      else
      {
         return $dbcon->UnixTimeStamp( $time );
      }
   }

//========================================================================

   /*!
   @function markRowDirty( $rowID )
   @discussion - justs mark all rows dirty for now
   */

   function markRowDirty( $rowID )
   {
      // this func is not yet fully realized
      $this->markAllDirty();
   }

//========================================================================

   /*!
   @function  getPrimaryColumn()
   @discussion - returns the name of the primary column for this item
   @result - returns string
   */

   function getPrimaryColumn()
   {
      return 'tableName';
   }

//========================================================================

   /*!
   @function  getTable()
   @discussion - returns the name of the table for this item
   @result - returns string
   */

   function getTable()
   {
      return 'tablecaching';
   }

//========================================================================

   /*!
   @function  markAllDirty()
   @discussion - sets last modify date for this table
   */

   function markAllDirty()
   {
      importClass( 'Database.DBConnectionProvider' );
      $dbcon =& DBConnectionProvider::getConnection();

      // we need to update the tablecaching row for this table

      //None: A Delete and Insert could create a bad interaction
      //with other DB client, since there is no transaction.

      $stmt = $dbcon->Prepare(
         'SELECT * FROM ' . $this->getTable() .
         ' WHERE ' . $this->getPrimaryColumn() . '=?' );

      $recordSet = $dbcon->Execute( $stmt, array( $this->table ) );

      $hash{ 'tableName' } = $this->table;
      $hash{ 'date_modified' } = time();

      if ( $recordSet->EOF )
      {
         //generate an INSERT SQL
         $sql = $dbcon->GetInsertSQL(
            $recordSet,
            $hash );
      }
      else
      {
         //generate an UPDATE SQL
         $sql = $dbcon->GetUpdateSQL(
            $recordSet,
            $hash );
      }

//      print getLocation() . BR_NL;
//      print 'SQL: ' . BR_NL;
//      print $sql . BR_NL;

      $dbcon->Execute( $sql );

   }

//========================================================================

};

?>