<?php

/*
ADbItemFieldFactory
$Author: dougqh $
$Revision: 1.4 $
$Date: 2004/05/19 14:18:48 $
$Log: ADbItemFieldFactory.php,v $
Revision 1.4  2004/05/19 14:18:48  dougqh
- Added support for dates

Revision 1.3  2004/04/01 22:11:25  millz
small changes

Revision 1.2  2004/01/07 22:18:55  millz
- refactor

Revision 1.1  2003/12/09 19:34:11  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

/*!
@class ADbItemFieldFactory
@abstract - an abstract class that produces for ALightFormFields for
   a given Entity class
*/

class ADbItemFieldFactory extends Object
{

//========================================================================

   /*!
   @function ADbItemFieldFactory()
   */

   function ADbItemFieldFactory()
   {
      $this->Object();
      $this->fieldsHash = array();
   }

//========================================================================

   /*!
   @function _setAction( &$action )
   */

   function _setAction( &$action )
   {
      $this->action =& $action;
   }


//========================================================================

   /*!
   @function getUser()
   @discussion - provides the User for which this ADbItemFieldFactory
      was created
   @result - returns a User
   */

   function &getUser()
   {
      return $this->action->getUser();
   }

//========================================================================

   /*!
   @function getDbItem()
   @discussion - provides the ADatabaseItem for which fields are being
     produced
   @result - returns an ADatabaseItem (or null if _creatingNew)
   */

   function &getDbItem()
   {
      return $this->action->getDbItem();
   }

//========================================================================

   /*!
   @function _creatingNew()
   @discussion - indicates if this ADbItemFieldFactory is manipulating
      a new DbItem
   @result - returns a boolean
   */

   function _creatingNew()
   {
      return $this->action->creatingNew();
   }

//========================================================================

   /*!
   @function _getDisplayName( $name, $displayName )
   @param $name - a String
   @param $displayName - a user-presentable String or UNSPECIFIED
   @discussion - if $displayName is UNSPECIFIED returns
      ucfirst( $name ); otherwise just returns $displayName
   @result - returns a user-presentable String
   */

   function _getDisplayName( $name, $displayName )
   {
      return ( $displayName === UNSPECIFIED ) ?
         ucfirst( $name ) :
         $displayName;
   }

//========================================================================

   /*!
   @function createField( $name )
   @param $name - a String - the name of a field
   @discussion - creates the ALightFormField that corresponds to the given
      name
   @result - returns an ALightFormField
   */
    function &createField( $name )
    {
        $field =& $this->_createField( $name );
        if ( ! is_null( $field ) )
        {
        $field->setRequired(
           $this->action->isRequired( $name ) );

        $field->setEnabled(
           $field->isEnabled() &&
           $this->action->isFieldEnabled( $name ) );
        }
        return $field;
    }

//========================================================================

   /*!
   @function _createField( $name )
   @param $name - a String - the name of a field
   @discussion - creates the ALightFormField that corresponds to the given
      name
   @result - returns an ALightFormField
   */

   /* abstract function &_createField( $name ); */

//========================================================================

   /*!
   @function _createText( $name, [ $displayName ] )
   @param $name - an identifying String
   @param $displayName - a user-presentable String - optional
      - defaults to ucfirst( $name )
   @discussion - constructs a LightTextField using the standard size
      for this builder
   @result - returns a new LightTextField
   */

   /* final */
   function &_createText( $name, $displayName = UNSPECIFIED )
   {
      $displayName = $this->_getDisplayName( $name, $displayName );

      importClass( 'HTMLComponents.LightForms.LightTextField' );
      return new LightTextField( $name, $displayName, 200, 106 );
   }

//========================================================================

   /*!
   @function _createShortText( $name, [ $displayName ] )
   @param $name - an identifying String
   @param $displayName - a user-presentable String - optional
      - defaults to ucfirst( $name )
   @discussion - constructs a LightTextField using the shorter size
      for this builder
   @result - returns a new LightTextField
   */

   /* final */
   function &_createShortText( $name, $displayName = UNSPECIFIED )
   {
      $displayName = $this->_getDisplayName( $name, $displayName );

      importClass( 'HTMLComponents.LightForms.LightTextField' );
      return new LightTextField( $name, $displayName, 200, 40 );
   }

//========================================================================

   /*!
   @function _createTextArea( $name, [ $displayName ] )
   @param $name - an identifying String
   @param $displayName - a user-presentable String - optional
      - defaults to ucfirst( $name )
   @discussion - constructs a LightTextAreaField using the standard size
      for this builder
   @result - returns a new LightTextAreaField
   */

   /* final */
   function &_createTextArea( $name, $displayName = UNSPECIFIED )
   {
      $displayName = $this->_getDisplayName( $name, $displayName );

      importClass( 'HTMLComponents.LightForms.LightTextAreaField' );
      return new LightTextAreaField( $name, $displayName, 80, 7 );
   }

//========================================================================

   /*!
   @function _createNumber( $name, [ $displayName ] )
   @param $name - an identifying String
   @param $displayName - a user-presentable String - optional
      - defaults to ucfirst( $name )
   @discussion - constructs a LightNumberField using the standard
      size for this builder
   @result - returns a new LightNumberField
   */

   /* final */
   function &_createNumber( $name, $displayName = UNSPECIFIED )
   {
      $displayName = $this->_getDisplayName( $name, $displayName );

      importClass( 'HTMLComponents.LightForms.LightTextField' );
      return new LightTextField( $name, $displayName, 20, 20 );
   }

//========================================================================

   /*!
   @function _createBoolean( $name, [ $displayName ], $defaultsToTrue = false )
   @param $name - an identifying String
   @param $displayName - a user-presentable String - optional
      - defaults to ucfirst( $name )
   @discussion - constructs a LightCheckBoxField for representing a bool val
   @result - returns a new LightCheckBoxField
   */

   /* final */
   function &_createBoolean( $name, $displayName = UNSPECIFIED, $defaultsToTrue = false )
   {
      $displayName = $this->_getDisplayName( $name, $displayName );

      importClass( 'HTMLComponents.LightForms.LightCheckBoxField' );
      return new LightCheckBoxField( $name, $displayName, $defaultsToTrue );
   }

//========================================================================

   /*!
   @function _createTimeDuration( $name, [ $displayName ] )
   @param $name - an identifying String
   @param $displayName - a user-presentable String - optional
      - defaults to ucfirst( $name )
   @discussion - constructs a TimeDurationField
   @result - returns a new TimeDurationField
   */

   /* final */
   function &_createTimeDuration( $name, $displayName = UNSPECIFIED )
   {
      $displayName = $this->_getDisplayName( $name, $displayName );

      //need a constant
      importClass( 'Bugs.ADevelopmentItem' );
      importClass( 'TimeUI.TimeDurationField' );
      return new TimeDurationField( $name, $displayName, DEFAULT_DEV_ITEM_TIME_UNIT );
   }

//========================================================================

   /*!
   @function _createFutureDate( $name, [ $displayName ] )
   @param $name - an identifying String
   @param $displayName - a user-presentable String - optional
      - defaults to ucfirst( $name )
   @discussion - constructs a FutureDateField
   @result - returns a new FutureDateField
   */
   /* final */
   function &_createFutureDate( $name, $displayName = UNSPECIFIED )
   {
      $displayName = $this->_getDisplayName( $name, $displayName );

      //need a constant
      importClass( 'TimeUI.FutureDateField' );
      return new FutureDateField( $name, $displayName );
   }

//========================================================================

   /*!
   @function _createSelect1( $absClassName, $name, [ $displayName ] )
   @param $absClassName - absolute class specifier for an ADatabaseItem
      class
   @param $name - an identifying String
   @param $displayName - a user-presentable String - optional
      - defaults to ucfirst( $name )
   @discussion - creates a LightIteratorSelectField provided by
      a default search on a $absClassName prototype
   @result - returns a new LightIteratorSelectField
   */

   /* final */
   function &_createSelect1( $absClassName, $name, $displayName = UNSPECIFIED )
   {
      $displayName = $this->_getDisplayName( $name, $displayName );

      importClass( $absClassName );
      $prototype =&
         callStaticMethod(
            absoluteClassToClass( $absClassName ),
            'getPrototype',
            new_array() );

      //print_r( $prototype );

      importClass( 'DatabaseUI.LightIteratorSelectField' );
      return new LightIteratorSelectField(
         $prototype->doSearch(),
         $name,
         $displayName,
         SINGLE_SELECT );
   }

//========================================================================

   /*!
   @function _createSelect2( $iterator, $name, [ $displayName ] )
   @param $iterator - an IIterator< ADatabaseItem >
   @param $name - an identifying String
   @param $displayName - a user-presentable String - optional
      - defaults to ucfirst( $name )
   @discussion - creates a LightIteratorSelectField
   @result - returns a new LightIteratorSelectField
   */

   /* final */
   function &_createSelect2( &$iterator, $name, $displayName = UNSPECIFIED )
   {
      $displayName = $this->_getDisplayName( $name, $displayName );

      importClass( 'DatabaseUI.LightIteratorSelectField' );
      return new LightIteratorSelectField(
         $iterator,
         $name,
         $displayName,
         SINGLE_SELECT );
   }

//========================================================================

};

?>