<?php

/*
Color
$Author: dougqh $
$Revision: 1.1 $
$Date: 2003/07/09 21:37:41 $
$Log: Color.php,v $
Revision 1.1  2003/07/09 21:37:41  dougqh
Initial revision

Revision 1.7  2003/03/17 14:24:37  dougqh
A little clean-up
New serializer

Revision 1.7  2003/03/07 18:22:47  dougqh
Color was needlessly importing Common.StoredObjects.SessionObject.

Revision 1.6  2002/09/24 23:00:48  dougqh
Quotes to ticks

Revision 1.5  2002/08/19 20:29:15  millz
bug fix to getAverageColor assertions

Revision 1.4  2002/08/19 20:08:23  millz
implemented and tested getAverageColor

Revision 1.3  2002/08/19 19:40:10  dougqh
Outlining getAverageColor.

Revision 1.2  2002/07/15 18:44:21  dougqh
Added new method isDark which indicates if a color is considered to be a dark color.

Revision 1.1  2002/07/01 20:07:45  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

//If any component of image exceeds the DARK_THRESHOLD then that Color
//is NOT dark.
define( 'DARK_THRESHOLD', 96 );

//========================================================================

/*!
@class Color
@abstract - a concrete class that represents a color which is an
   RGB triple
*/

class Color extends Object
{

//========================================================================

   var $hexString;

//========================================================================

   /*!
   @function Color( $red, $green, $blue )
   @param $red - an Integer between 0 and 255
   @param $green - an Integer between 0 and 255
   @param $blue - an Integer between 0 and 255
   @discussion - constructs a new Color object
      NOTE: This constructor should not be called directly call the
         static method getColor instead.
   @result - constructs a new Color object
   */

   /* private */
   function Color( $red, $green, $blue )
   {
      //require
      {
         assert( '$red >= 0 && $red <= 255' );
         assert( '$green >= 0 && $green <= 255' );
         assert( '$blue >= 0 && $blue <= 255' );
      }
      //body
      {
         $this->red = $red;
         $this->green = $green;
         $this->blue = $blue;

         //allow copying at serialization time
         $this->Object( true );
      }
      //ensure
      {
         assert( '$this->_invariant()' );
      }
   }

//========================================================================

   /*!
   @function getColor( $red, $green, $blue )
   @param $red - an Integer between 0 and 255
   @param $green - an Integer between 0 and 255
   @param $blue - an Integer between 0 and 255
   @discussion - provides the instance of the specified Color
   @result - returns the instance of the specified Color
   */

   /* static */
   function &getColor( $red, $green, $blue )
   {
      $colorIdentifier = $red . '_' . $green . '_' . $blue;

      static $colorsHash;
      if ( ! isset( $colorsHash{ $colorIdentifier } ) )
      {
         $colorsHash{ $colorIdentifier } =& new Color( $red, $green, $blue );
      }

      return $colorsHash{ $colorIdentifier };
   }

//========================================================================

   function getColorFromHexTriplet( $hexTriplet )
   {
      $r = hexdec( $hexTriplet[0].$hexTriplet[1] );
      $g = hexdec( $hexTriplet[2].$hexTriplet[3] );
      $b = hexdec( $hexTriplet[4].$hexTriplet[5] );

      // print "Will create color $r, $g, $b<br>";

      return Color::getColor( $r, $g, $b );
   }

//========================================================================

   /*!
   @function getAverageColor( $color0, $color1 )
   @param $color0 - a Color
   @param $color1 - a Color
   @discussion -
   @result - returns a Color
   */

   function &getAverageColor( &$color0, &$color1 )
   {
      assert( 'isInstanceOf( $color0, "Color" )' );
      assert( 'isInstanceOf( $color1, "Color" )' );

      $color0RGB = $color0->getRGBTuple();
      $color1RGB = $color1->getRGBTuple();

      $avgRed     = intval ( ($color0RGB[0] + $color1RGB[0]) / 2 );
      $avgBlue    = intval ( ($color0RGB[1] + $color1RGB[1]) / 2 );
      $avgGreen   = intval ( ($color0RGB[2] + $color1RGB[2]) / 2 );

      return Color::getColor( $avgRed,$avgBlue,$avgGreen );
   }

//========================================================================

   /*!
   @function isDark()
   @discussion - indicates if this Color is considered to be a dark Color
   @result - returns true if this Color is a dark Color;
      otherwise returns false
   */

   function isDark()
   {
      return ( $this->red <= DARK_THRESHOLD ) &&
             ( $this->green <= DARK_THRESHOLD ) &&
             ( $this->blue <= DARK_THRESHOLD );
   }

//========================================================================

   /*!
   @function getRed()
   @discussion - provides the red component of this Color
   @result - returns a positive integer
   */

   function getRed()
   {
      assert( '$this->_invariant()' );

      return $this->red;
   }

//========================================================================

   /*!
   @function getGreen()
   @discussion - provides the green component of this Color
   @result - returns a positive integer
   */

   function getGreen()
   {
      assert( '$this->_invariant()' );

      return $this->green;
   }

//========================================================================

   /*!
   @function getBlue()
   @discussion - provides the blue component of this Color
   @result - returns a positive integer
   */

   function getBlue()
   {
      assert( '$this->_invariant()' );

      return $this->blue;
   }

//========================================================================

   /*!
   @function getRGBTuple()
   @discussion - provides a 3-tuple containing the
      red component as the first element, the green component as the second,
      and the component as the third
   @result - returns an array of integers
   */

   function getRGBTuple()
   {
      assert( '$this->_invariant()' );

      return new_array( $this->red, $this->green, $this->blue );
   }

//========================================================================

   /*!
   @function getHexString()
   @discussion - provides an HTML hex String specification of the color
   @result - returns a String
   */

   function getHexString()
   {
      //require
      {
         assert( '$this->_invariant()' );
      }
      //body
      {
         if ( ! isset( $this->hexString ) )
         {
            $this->hexString = sprintf(
               '%02X%02X%02X',
               $this->red,
               $this->green,
               $this->blue );
         }
      }
      //ensure
      {
         assert( '$this->_invariant()' );
      }

      return $this->hexString;
   }

//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */

   function _invariant()
   {
      return parent::_invariant();
   }

//========================================================================

};

?>