<?php

/*
Font
$Author: dougqh $
$Revision: 1.1 $
$Date: 2003/07/09 21:37:41 $
$Log: Font.php,v $
Revision 1.1  2003/07/09 21:37:41  dougqh
Initial revision

Revision 1.5  2002/09/24 18:13:21  dougqh
Changing quotes to ticks.

Revision 1.4  2002/07/12 15:40:46  dougqh
Changed font to generate div tags with style attributes to change
the appearance of text.  Changed the algorithm for computing
absolute sizes from relative sizes.  getHTMLHead only generates
absolute size specifications computed from relative size specifications.

Revision 1.3  2002/07/12 13:49:48  dougqh
Fixed in error in Font::getFont that was causing multiple copies
of the same Font to be created.

Revision 1.2  2002/07/08 17:12:04  dougqh
Fixed bug in getTrueTypeFile.

Revision 1.1  2002/07/01 20:07:45  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

//These constants play double roles
//Role 1 - a constant indicating the use of the default
//Role 2 - the name of the Config variable containing the actual values
define( 'DEFAULT_FONT_FACE', 'defaultFontFace' );
define( 'DEFAULT_FONT_SIZE', 'defaultFontSize' );

//========================================================================

/*!
@class Font
@abstract - a class that represents a Font
*/

class Font extends Object
{
//========================================================================

   var $face;
   var $size;
   var $bold;
   var $italics;

   var $emphasisHead;
   var $emphasisTail;

   var $baseStyle;
   var $usedDivTag;
   var $hasParam;

   var $fontFace;

//========================================================================

   /*!
   @function Font( $face, $size, $bold, $italics )
   @param $face - a String containing the name of the face of this Font
   @param $size - a size either relative or absolute for this Font
   @param $bold - a boolean indicating if this Font should be bold
   @param $italics - a boolean indicating if this Font should be italicized
   @discussion - constructs a new Font object
      - the font constructed will reflect the default Font for the
        system until it is set through the accessor methods.
   @result - constructs a new Font object
   */

   function Font( $face, $size, $bold, $italics )
   {
      //require
      {
         assert( '$face != ""' );
         assert( 'Font::_isRelativeSize( $size ) ||
                  Font::_isAbsoluteSize( $size ) ||
                  ( $size == DEFAULT_FONT_SIZE )' );
      }
      //body
      {
         $arguments = func_get_args();
         debug( 'Font::Font', "Arguments", $arguments );

         $this->face = $face;
         $this->size = $size;
         $this->bold = $bold;
         $this->italics = $italics;

         //allow copying at serialization time
         $this->Object( true );
      }
      //ensure
      {
         assert( '$this->_invariant()' );
      }
   }


//========================================================================

   /*!
   @function getFont( $face, $size, [ $isBold, [ $isItalicized ] ] )
   @param $face - a font face to use
   @param $size - a size either relative or absolute;
      relative - "+2" or "-1"
      absolute - "10pt"
   @param $isBold - optional boolean indicating if this Font should be isBold
      - defaults to false
   @param $isItalicized - optional boolean indicating if this Font should be italicized
      - defaults to false
   @discussion - provides the Font specified
   @result - returns a Font object
   */

   /*!
   @function getFont( $size, [ $isBold, [ $isItalicized ] ] )
   @param $size - a size either relative or absolute;
      relative - "+2" or "-1"
      absolute - "10pt"
   @param $isBold - optional boolean indicating if this Font should be isBold
      - defaults to false
   @param $isItalicized - optional boolean indicating if this Font should be italicized
      - defaults to false
   @discussion - provides a Font object that uses the default face
   @result - returns a Font object
   */

   /*!
   @function getFont( [ $isBold, [ $isItalicized ] ] )
   @param $isBold - optional boolean indicating if this Font should be isBold
      - defaults to false
   @param $isItalicized - optional boolean indicating if this Font should be italicized
      - defaults to false
   @discussion - provides the default Font object
   @result - returns a Font object
   */

   function &getFont(
      $arg0 = UNSPECIFIED, $arg1 = UNSPECIFIED,
      $arg2 = UNSPECIFIED, $arg3 = UNSPECIFIED )
   {
      $arguments = func_get_args();

      if ( is_string( $arg0 ) && ( $arg0 != UNSPECIFIED ) )
      {
         if ( is_string( $arg1 ) && ( $arg1 != UNSPECIFIED ) )
         {
            $face = $arg0;
            $size = $arg1;
            $isBold = $arg2;
            $isItalicized = $arg3;
         }
         else
         {
            $face = DEFAULT_FONT_FACE;
            $size = $arg0;
            $isBold = $arg1;
            $isItalicized = $arg2;
         }
      }
      else
      {
          $face = DEFAULT_FONT_FACE;
          $size = DEFAULT_FONT_SIZE;
          $isBold = $arg0;
          $isItalicized = $arg1;
      }

      if ( $isBold === UNSPECIFIED ) $isBold = false;
      if ( $isItalicized === UNSPECIFIED ) $isItalicized = false;

      return Font::_getFont( $face, $size, $isBold, $isItalicized );
   }


//========================================================================

   /*!
   @function getBold()
   @discussion - convenience method for getting a bold font
      Also, slightly faster than getFont( true )
   */

   function &getBold()
   {
      return Font::_getFont(
         DEFAULT_FONT_FACE,
         DEFAULT_FONT_SIZE,
         true,
         false );
   }

//========================================================================

   /*!
   @function _getFont( $face, $size, $isBold, $isItalicized )
   */

   function &_getFont( $face, $size, $isBold, $isItalicized )
   {
      $fontIdentifier = $face . '_' . $size . '_' .
         intval( $isBold ) . '_' . intval( $isItalicized );

      static $fontsHash;
      if ( ! isset( $fontsHash{ $fontIdentifier } ) )
      {
         $fontsHash{ $fontIdentifier } =& new Font( $face, $size, $isBold, $isItalicized );
      }

      return $fontsHash{ $fontIdentifier };
   }

//========================================================================

   /*!
   @function getDefaultFace()
   @discussion - a static method that provides the default font face
   @result - returns a String
   */

   /* static */
   function getDefaultFace()
   {
      return 'Verdana';
   }

//========================================================================

   /*!
   @function getDefaultSize()
   @discussion - a static method that provide the default ABSOLUTE font size
   @result - returns an integer
   */

   /* static */
   function getDefaultSize()
   {
      return 12;
   }

//========================================================================

   /*!
   @function getFace()
   @discussion - indicates the font face of this Font;
      may return the constant DEFAULT_FONT_FACE
   @result - returns a String containing the name of the Font face
   */

   function getFace()
   {
      assert( '$this->_invariant()' );

      return $this->face;
   }

//========================================================================

   /*!
   @function getSize()
   @discussion - indicates the font face of this Font;
      may return the constant DEFAULT_FONT_SIZE
   @result - returns a String containing the size of this Font
   */

   function getSize()
   {
      assert( '$this->_invariant()' );

      return $this->size;
   }

//========================================================================

   /*!
   @function isBold()
   @discussion - indicates if this Font is bold
   @result - returns true if this Font is bold;
      otherwise returns false
   */

   function isBold()
   {
      assert( '$this->_invariant()' );

      return $this->bold;
   }

//========================================================================

   /*!
   @function isItalicized()
   @discussion - indicates if this Font is italicized
   @result - returns true if this Font is italicized;
      otherwise returns false
   */

   function isItalicized()
   {
      assert( '$this->_invariant()' );

      return $this->italics;
   }

//========================================================================

   /*!
   @function getActualFace()
   @discussion - indicates the actual face of this Font

      Fonts created with DEFAULT_FONT_FACE will provide a String
      containing the default font face recovered from the database
   @result - returns a String
   */

   function getActualFace()
   {
      //require
      {
         assert( '$this->_invariant()' );
      }
      //body
      {
         if ( $this->face == DEFAULT_FONT_FACE )
         {
            $face = Font::getDefaultFace();
         }
         else
         {
            $face = $this->face;
         }
      }
      //ensure
      {
         assert( '$face != ""' );

         assert( '$this->_invariant()' );
      }

      return $face;
   }

//========================================================================

   /*!
   @function getActualSize()
   @discussion - indicates the actual size of this Font

      Fonts created with DEFAULT_FONT_SIZE will provide the actual
      default font size.

      This function does still return relative size specifications.
   */

   function getActualSize()
   {
      //require
      {
         assert( '$this->_invariant()' );
      }
      //body
      {
         if ( $this->size == DEFAULT_FONT_SIZE )
         {
            $size = Font::getDefaultSize();
         }
         else
         {
            $size = $this->size;
         }
      }
      //ensure
      {
         assert( '$size != ""' );

         assert( '$this->_invariant()' );
      }

      return $size;
   }

//========================================================================

   /*!
   @function getRelativeSize()
   @discussion - provides the relative size of this Font
   @result - returns a String containing a relative font size
      specifier i.e. +N -N
   */

   function getRelativeSize()
   {
      //require
      {
         assert( '$this->_invariant()' );
      }
      //body
      {
         if ( Font::_isRelativeSize( $this->size ) )
         {
            $result = $this->size;
         }
         else
         {
            //TODO: Implement conversion actual to a relative size here
            $result = '+0';
         }
      }
      //ensure
      {
         assert( 'Font::_isRelativeSize( $result )' );

         assert( '$this->_invariant()' );
      }

      return $result;
   }

//========================================================================

   /*!
   @function getAbsoluteSize()
   @discussion - if the Font was created with an absolute size specification
      that value is returned; otherwise computes the absolute size from
      the default font size and the relative size
   @result - returns an integer
   */

   function getAbsoluteSize()
   {
      //require
      {
         assert( '$this->_invariant()' );
      }
      //body
      {
         if ( $this->size == DEFAULT_FONT_SIZE )
         {
            $size = Font::getDefaultSize();
         }
         elseif ( Font::_isAbsoluteSize( $this->size ) )
         {
            $size = str_replace( 'pt', '', $this->size );
         }
         elseif ( Font::_isRelativeSize( $this->size ) )
         {
            $increasing = ( strpos( $this->size, '+' ) !== false );
            $amount = str_replace( array( '+', '-' ), '', $this->size );

            $differential = Font::getDefaultSize() * ( $amount * 0.1 );
            $differential = max( intval( $differential ), 1 );

            if ( $increasing )
            {
               $size = Font::getDefaultSize() + $differential;
            }
            else
            {
               $size = Font::getDefaultSize() - $differential;
            }
         }
      }
      //ensure
      {
         assert( '$size >= 0' );

         assert( '$this->_invariant()' );
      }

      return $size;
   }

//========================================================================

   /*!
   @function getHTMLHead( [ $hexColor ] )
   @param $hexColor - optional String containing hex specification of
      text color
   @discussion - generates the opening tag of an HTML representation
      of this Font object
   @result - a string containing the font opening tag
   */

   function getHTMLHead( $hexColor = UNSPECIFIED )
   {
      //require
      {
         assert( '$this->_invariant()' );
      }
      //body
      {
         //debug( 'Font::getHTMLHead', 'this', $this );

         if ( ! isset( $this->emphasisHead ) )
         {
            $this->emphasisHead = '';
            $this->emphasisTail = '';

            if ( $this->bold )
            {
               $this->emphasisHead .= '<b>';
               $this->emphasisTail = '</b>' . $this->emphasisTail;
            }

            if ( $this->italics )
            {
               $this->emphasisHead .= '<i>';
               $this->emphasisTail .= '</i>' . $this->emphasisTail;
            }
         }

         if ( ! isset( $this->baseStyle ) )
         {
            $this->hasParam = false;

            if ( $this->face === DEFAULT_FONT_FACE )
            {
               $faceParam = '';
            }
            else
            {
               $faceParam = ' font-family:' . $this->face .';';
               $this->hasParam = true;
            }

            if ( $this->size === DEFAULT_FONT_SIZE )
            {
               $sizeParam = '';
            }
            else
            {
               $sizeParam = ' font-size:' . $this->getAbsoluteSize() . 'px;';
               $this->hasParam = true;
            }

            $this->baseStyle = $faceParam . $sizeParam;

            //NOTE: baseStyle is not complete by itself after this construction
            //This is done to allow delayed color specification.
         }

         if ( $hexColor === UNSPECIFIED )
         {
            $actualStyle = $this->baseStyle;
         }
         else
         {
            $actualStyle = $this->baseStyle . ' color: #' . $hexColor . ';';
         }

         $this->usedDivTag = ( $actualStyle != '' );
         if ( $this->usedDivTag )
         {
            $actualFontHead = '<div style="' . $actualStyle . '">';
         }
         else
         {
            $actualFontHead = "";
         }

         $result = $this->emphasisHead . $actualFontHead;

         debug( 'Font::getHTMLHead', 'ObjectID', $this->object_id );
      }
      //ensure
      {
         assert( 'isset( $this->emphasisHead )' );
         assert( 'isset( $this->emphasisTail )' );

         assert( 'isset( $this->baseStyle )' );
         assert( 'isset( $this->usedDivTag )' );
         assert( 'isset( $this->hasParam )' );

         assert( '$this->_invariant()' );
      }

      return $result;
   }

//========================================================================

   /*!
   @function getHTMLTail()
   @discussion - generates the closing of an HTML representation of
      this Font object

      NOTE: getHTMLHead MUST be called before the corresponding call
         to getHTMLTail for the HTML generation to work correctly.
   @result - a string containing a font closing tag
   */

   function getHTMLTail()
   {
      //require
      {
         //should have been set by getHTMLHead
         assert( 'isset( $this->emphasisTail )' );
         assert( 'isset( $this->usedDivTag )' );

         assert( '$this->_invariant()' );
      }
      //body
      {
         $result = ( $this->usedDivTag ? '</div>' : '' ) . $this->emphasisTail;

         //debug( 'Font::getHTMLTail', 'ObjectID', $this->object_id );
      }
      //ensure
      {
         assert( '$this->_invariant()' );
      }

      return $result;
   }

//========================================================================

   /*!
   @function getTrueTypeFile()
   @discussion - provides the full path of the file that contains this
      font definition
   @result - a String containing an absolute file path
   */

   function getTrueTypeFile()
   {
      //require
      {
         assert( '$this->_invariant()' );
      }
      //body
      {
         //debug( "Font::getTrueTypeFile", "this", $this );

         if ( ! isset( $this->fontFile ) )
         {
            $this->fontFile = $this->getActualFace();

            $style = "";
            if ( $this->bold ) $style .= 'Bold';
            if ( $this->italics ) $style .= 'Italic';

            if ( $style != '' ) $this->fontFile .= '-' . $style;

            $this->fontFile .= '.ttf';

            $this->fontFile = truePath( 'fonts' ) . $this->fontFile;
         }
      }
      //ensure
      {
         assert( 'isset( $this->fontFile )' );
         assert( '$this->_invariant()' );
      }

      return $this->fontFile;
   }

//========================================================================

   /*!
   @function _isRelativeSize( $size )
   @param $size - a String containing a size specifier
   @discussion - static method that indicates whether or not the provided
      size specifier is relative size specifier
   @result - returns true if $size is a relative size specifier;
      otherwise returns false
   */

   /* static */
   function _isRelativeSize( $size )
   {
      return ( ( strpos( $size, '+' ) === 0 ) ||
               ( strpos( $size, '-' ) === 0 ) );
   }

//========================================================================

   /*!
   @function _isAbsoluteSize( $size )
   @param $size - a String containing a size specifier
   @discussion - static method that indicates whether or not the provided
      size specifier is an absolute size specifier
   @result - returns if $size is an absolute size specifier;
      otherwise returns false
   */

   function _isAbsoluteSize( $size )
   {
      return ( strpos( $size, 'pt' ) === strlen( $size ) - 2 );
   }

//========================================================================

   /*!
   @function _getExcludedVariablesToNeverRestore()
   @discussion - do NOT want to store htmlHead, htmlTail, and fontFile
      because they contain information requested from getActualFace and
      getActualSize which can change if the default font Config Variables
      are modified
   @result - returns an array of Strings
   */

   function _getExcludedVariablesToNeverRestore()
   {
      return array( 'htmlHead', 'fontFile' );
   }

//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */

   function _invariant()
   {
      return parent::_invariant() &&
             ( $this->face != '' ) &&
             ( Font::_isRelativeSize( $this->size ) ||
               Font::_isAbsoluteSize( $this->size ) ||
               ( $this->size == DEFAULT_FONT_SIZE ) );
   }

//========================================================================

};

?>