<?php

/*
AEmail
$Author: dougqh $
$Revision: 1.9 $
$Date: 2004/08/25 13:23:19 $
$Log: AEmail.php,v $
Revision 1.9  2004/08/25 13:23:19  dougqh
- Added a check to suppress the mail call in AEmail::send if there are no recipients

Revision 1.8  2004/04/28 13:53:43  dougqh
- Quick fix to last change

Revision 1.7  2004/04/27 20:35:49  dougqh
- Fix for email issue under Windows

Revision 1.6  2003/10/21 15:14:18  dougqh
- Fixed issue with sender list format.

Revision 1.5  2003/10/21 14:14:15  dougqh
Added exclusion support

Revision 1.4  2003/09/29 18:14:05  dougqh
- Replaced short PHP tag with long PHP tag

Revision 1.3  2003/09/05 17:26:48  dougqh
- Another patch to _getHeaders

Revision 1.2  2003/09/05 17:05:12  dougqh
- Added angle brackets that were missing from the header

Revision 1.1  2003/07/09 22:00:54  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

class AEmail extends Object
{

//========================================================================

   /*!
   @function AEmail()
   @discussion AEmail constructor
   */

   function AEmail()
   {
      $this->Object();

      $this->recipientsHash = array();
      $this->exclusionsHash = array();
   }

//========================================================================

   /*!
   @function _addUserRecipient( $user )
   @param $user - a User
   @discussion - add the specified User as recipient of this Email
   */

   function _addUserRecipient( &$user )
   {
      if ( ! $user->isNull() ) $this->_addRecipient( $user->getEmailAddress() );
   }

//========================================================================

   /*!
   @function _addUserRecipients( $iteratorUser )
   @param $iteratorUser - an IIterator< User >
   @discussion - adds all the Users in the iterator as recipients
      of this AEmail
   */

   function _addUserRecipients( &$iteratorUser )
   {
      for (
         $iteratorUser->reset();
         ! $iteratorUser->isEnd();
         $iteratorUser->moveNext() )
      {
         $this->_addUserRecipient(
            $iteratorUser->getCurrent() );
      }
   }

//========================================================================

   /*!
   @function _addRecipient( $emailAddress )
   @param $emailAddress - a String - email address of another recipient
   @discussion add a recipient using their email address
   */

   function _addRecipient( $emailAddress )
   {
      $this->recipientsHash{ $emailAddress } = true;
   }

//========================================================================

   /*!
   @function addExclusion( $emailAddress )
   @param $emailAddress - a String - email address
   @discusssion - excludes that email address from the recipients
      list (even if its explicitly added)
   */

   function _addExclusion( $emailAddress )
   {
      $this->exclusionsHash{ $emailAddress } = true;
   }

//========================================================================

   /*!
   @function _addUserExclusion( $user )
   @param $user - a User
   @discussion - excludes that User from the recipients list
      (even if its explicitly added)
   */

   function _addUserExclusion( &$user )
   {
      $this->_addExclusion( $user->getEmailAddress() );
   }

//========================================================================

   /*!
   @function getRecipientsList()
   @returns - a String - of recipients - delimited with angle brackets and commas
   */

   function getRecipientsList()
   {
      $this->recipientsHash =
         set::minus( $this->recipientsHash, $this->exclusionsHash );

      if ( EMAIL_USE_ANGLE_BRACKETS )
      {
         return '<' . implode( '>,<', array_keys( $this->recipientsHash ) ) . '>';
      }
      else
      {
         return implode( ',', array_keys( $this->recipientsHash ) );
      }
   }

//========================================================================

   /*!
   @function send()
   @discussion sends an email using
   */

   function send()
   {
      $recipients = trim( $this->getRecipientsList() );
      if ( $recipients == '' || $recipients == '<>' ) return;

      mail(
         $recipients,
         $this->getTitle(),
         $this->getBody(),
         $this->_getHeaders() );
   }

//========================================================================

   /*!
   @function getBody()
   @discussion - provides the body of the email
   @result - returns a String
   */

   /* abstract function getBody();  */

//========================================================================

   /*!
   @function getTitle()
   @discussion - provides a title for the email
   @result - returns a String
   */

   /* abstract function getTitle(); */

//========================================================================

   /*!
   @function getSenderEmail()
   @discussion - provides a sender for this email
   @result - returns a String
   */

   /* abstract function getSenderEmail(); */

//========================================================================

   /*!
   @function getSenderDisplayName()
   @discussion - provides a "pretty" name for the sender
      Default implementation returns empty String.
   @result - returns a user-presentable String
   */

   function getSenderDisplayName()
   {
      return '';
   }

//========================================================================

   /*!
   @function getReplyTo()
   @discussion - provides a reply to for this email
      Default implementation calls getSenderEmail
   @result - returns a String
   */

   function getReplyToEmail()
   {
      return $this->getSenderEmail();
   }

//========================================================================

   /*!
   @function _getHeaders()
   @discussion - provides the headers for this AEmail
   @result - returns a String
   */

   /* final */
   function _getHeaders()
   {
      $senderDisplayName = $this->getSenderDisplayName();

      //According to user comment at php.net on the mail function
      //Outlook gets annoyed without the space after the >
      return
         'From: ' . $senderDisplayName . ' <' . $this->getSenderEmail() . "> \r\n" .
         'Reply-to: <' . $this->getReplyToEmail() . '>';
   }

//========================================================================

};

?>