<?php

/*
CommandLine
$Author: dougqh $
$Revision: 1.2 $
$Date: 2002/11/14 15:27:31 $
$Log: CommandLine.php,v $
Revision 1.2  2002/11/14 15:27:31  dougqh
Minor revisions to work better with Executable.

Revision 1.1  2002/11/11 22:25:03  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/


//========================================================================

/*!
@class CommandLine
@abstract - a class that builds command lines
*/

class CommandLine extends Object
{

//========================================================================

   var $baseLine;
   
   var $parameters;
   var $numParameters;
   
   var $outRedirect;
   var $errorRedirect;

//========================================================================

   /*!
   @function CommandLine( $baseLine )
   @discussion - constructs a new CommandLine object
   @result - constructs a new CommandLine object
   */
   
   function CommandLine( $baseLine )
   {
      //require
      {
      }
      //body
      {
         $this->baseLine = $baseLine;
         
         $this->parameters = array();
         $this->numParameters = 0;
         
         $this->Object();
      }
      //ensure
      {
         assert( 'invariant( $this )' );         
      }
   }

//========================================================================

   /*!
   @function setFlagPrefix( $prefix )
   @discussion - sets the prefix used for flags
      - usually one of DEFAULT_FLAG_PREFIX or PHP_FLAG_PREFIX
   */
   
   function setFlagPrefix( $prefix )
   {
      $this->flagPrefix = $prefix;
   }
         
//========================================================================

   /*!
   @function addParameter( $value )
   @param $value - a String   
   @discussion - Adds <value> to the end of the command line.
   */

   /*!
   @function addParameter( $flag, $value )
   @param $flag - a String containing a flag without a dash prefix
   @param $value - a value that corresponds to the flag
   @discussion - 
   */
   
   function addParameter( $flagOrValue, $value = UNSPECIFIED )
   {
      if ( $value === UNSPECIFIED )
      {
         $this->parameters[ $this->numParameters ] = $flagOrValue;
      }
      else
      {
         $this->parameters[ $this->numParameters ] = $this->flagPrefix . $flagOrValue . ' ' . $value;
      }
   }
   
//========================================================================
   
   /*!
   @function addFlag( $flag )
   @param $flag - a String containing a flag without a dash prefix
   @discussion - a very thin wrapper around addParameter( $value )
      Adds -<flag> to the end of the command line.
   */
   
   function addFlag( $flag )
   {
      $this->addParameter( $this->flagPrefix . $flag );
   }

//========================================================================

   /*!
   @function setOutRedirect( $file )
   @param $file - a String containing a file name
   @discussion - redirects the standard out to the specified file
   */
   
   function setOutRedirect( $file )
   {
      $this->outRedirect = $file;
   }

//========================================================================
   
   /*!
   @function setErrorRedirect()
   @param $file - a String containing a file name
   @discussion - redirects the standard error to the specified file
   */
   
   function setErrorRedirect( $file )
   {
      $this->errorRedirect = $file;
   }

//========================================================================

   /*!
   @function getCommandLine( $redirectAll )
   @param $redirectAll - a boolean - if true getCommandLine will redirect
      all streams that have not already been redirected to /dev/null.
      - defaults to false
   @discussion - provides the actual command line String to be executed
   @result - returns a String
   */
   
   function getCommandLine( $redirectAll = UNSPECIFIED )
   {
      $redirectAll = ( $redirectAll === UNSPECIFIED ) ? false : $redirectAll;
      
      $command = $this->baseLine;
      
      $command .= implode( ' ', $this->parameters );
      
      if ( isset( $this->outRedirect ) )
      {
         $command .= ' 1>' . $this->outRedirect;
      }
      elseif ( $redirectAll )
      {
         $command .= ' 1>/dev/null';
      }
      
      if ( isset( $this->errorRedirect ) )
      {
         $command .= ' 2>' . $this->errorRedirect;
      }
      elseif ( $redirectAll )
      {
         $command .= ' 2>/dev/null';
      }
      
      return $command;
   }      

//========================================================================

   /*!
   @function start()
   @discussion - starts this CommandLine running in the background
   @result - returns the command's return value
   */
   
   function start()
   {
      process::start(
         $this->getCommandLine( true /* force redirect */ ) . ' &',
         false /* do not add redirects */ );
   }

//========================================================================

   /*!
   @function execute()
   @discussion - executes the command and waits for its output
   @result - returns an ordered pair
      - the first element is the command's return value
      - the second element is the output lines in an array
   */
   
   function execute()
   {
      process::execute( $this->getCommandLine() );
   }
      
//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */
    
   function _invariant()
   {
      return parent::_invariant();
   }

//========================================================================

};

?>