<?php

/*
Executable
$Author: dougqh $
$Revision: 1.2 $
$Date: 2003/07/10 13:15:22 $
$Log: Executable.php,v $
Revision 1.2  2003/07/10 13:15:22  dougqh
?

Revision 1.1  2002/11/14 15:27:31  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/

//========================================================================

/*!
@class Executable
@abstract - an interface for creating defining classes run from command line
   by the executor script
*/

class Executable extends Object
{

//========================================================================

   /*!
   @function Executable( $argumentsHash )
   @param $argumentsHash - a hash of arguments used to run this Executable
      Keys are argument names
      Values are argument values.
      
      UNSPECIFIED may be sent to create a prototype object which cannot be executed.
   @discussion - creates an Executable on which execute can be invoked
   */   
   
   function Executable( $argumentsHash )
   {
      //require
      {
         assert( 'isConcrete( $this, "Executable",
            array( "getArguments", "run" ) )' );
      }
      //body
      {         
         if ( $argumentsHash !== UNSPECIFIED ) $this->_setArguments( $argumentsHash );
         
         $this->Object();
      }
      //ensure
      {
         assert( 'invariant( $this )' );
      }
   }

//========================================================================

   /*!
   @function createCommandLine()
   @discussion - provides a CommandLine that will invoke this Executable
      - usually called on a prototype Executable
   @result - returns a new CommandLine
   */
   
   function &createCommandLine()
   {
      $commandLine =& new CommandLine( truePath( GP_BIN ) . PHP_EXECUTOR_SCRIPT );
      $commandLine->setFlagPrefix( PHP_FLAG_PREFIX );
      $commandLine->addParameter( $this->getAbsoluteClass() );
      
      return $commandLine;
   }
      
//========================================================================

   /*!
   @function _setArguments( $argumentsHash )
   @param $argumentsHash - a hash of arguments used to run this Executable
      Keys are argument names
      Values are argument values.
   @discussion - sets the arguments used to invoke this Executable
   */
   
   function _setArguments( $argumentsHash )
   {
      print 'Setting arguments' . NL;
      
      $this->argumentsHash = $argumentsHash;
   }
   
//========================================================================

   /*!
   @function hasArgument( $argumentName )
   @param $argumentName - a String containing an argument name
   @discussion - checks to see if the specified argument was supplied
   @result - returns a boolean
   */
   
   function hasArgument( $argumentName )
   {
      return isset( $this->argumentsHash{ $argumentName } );
   }
   
//========================================================================

   /*!
   @function getArgumentValue( $argumentName )
   @param $argumentName - a String containing an argument name
   @discussion - provides the value of the specified argument
   @result - returns a String
   */
   
   function getArgumentValue( $argumentName )
   {
      return $this->argumentsHash{ $argumentName };
   }
      
//========================================================================

   /*!
   @function getArguments()
   @discussion - provides a hash that represents the available arguments
      Keys are variable names.
      Values are variable descriptions.
      
      Example:
         return array(
            'user_id' => 'A user_id from the user table',
            'message' => 'A string containing a message to print' );
   
   @result - returns a hash of Strings
   */
   
   /* abstract static function getArguments(); */

//========================================================================

   /*!
   @function argumentsAreValid()
   @discussion - called to check if argument combination is valid
      
      NOTE: This default implementation always returns true if the
         arguments have been set.
   @result - returns a boolean
   */
   
   function argumentsAreValid()
   {
      return isset( $this->argumentsHash );
   }

//========================================================================

   /*!
   @function getArgumentErrors()
   @discussion - called when argumentsAreValid returns false
      - provides a plain text description of the what was wrong
      
      NOTE: If Executable::argumentsAreValid is overridden this method
      should also be overridden.
   @result - returns a String
   */
   
   function getArgumentErrors()   
   {
      return 'Arguments have not been set.' . NL;
   }
   
//========================================================================

   /*!
   @function getUsage()
   @discussion - provides a plain text message illustrating how
      to use this command
   @result - returns a String
   */
   
   function getUsage()
   {
      todo( 'Executable::getUsage', 'Change $this to self in PHP 5' );
      
      $string = '';
      foreach ( $this->getArguments() as $variable => $description )
      {
         $string .= $variable . ' - ' . $description . NL;
      }
      
      return $string;
   }
      
//========================================================================

   /*!
   @function execute()
   @discussion - c
   @result - 
   */
   
   function execute()
   {
      if (  ! $this->argumentsAreValid() )
      {
         print $this->getArgumentErrors();
         
         return INVALID_ARGUMENTS;
      }
      else
      {
         return $this->_execute();
      }
   }
   
//========================================================================
   
   /*!
   @function _execute()
   @discussion - called to execute this Executable
   @result - returns an Integer - 0 indicates success
   */
   
   /* abstract function _execute(); */
   
//========================================================================

   /*!
   @function _invariant()
   @discussion -
   @result - 
   */
   
   function _invariant()
   {
      return parent::_invariant();
   }
   
//========================================================================

};

?>