<?php

/*
DelimitedDelimitedFileGenerator
$Author: millz $
$Revision: 1.7 $
$Date: 2003/07/23 18:16:13 $
$Log: DelimitedFileGenerator.php,v $
Revision 1.7  2003/07/23 18:16:13  millz
- corrected problems with the use of array_walk

Revision 1.6  2003/01/11 19:30:23  millz
- found some more refs to escaper to kill

Revision 1.5  2003/01/10 22:30:22  millz
- removed need for escaper

Revision 1.4  2002/09/18 23:06:35  dougqh
Minor fix in addRow.

Revision 1.3  2002/09/13 22:43:38  millz
slight speed-ups and documentation-naming fixes

Revision 1.2  2002/09/13 19:08:51  millz
nothing

Revision 1.1  2002/09/13 19:07:15  millz
Initial revision



IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/

//========================================================================

importClass( "FileGenerators.FileGenerator" );

//========================================================================

/*!
@class DelimitedFileGenerator
@abstract - an abstract class that provides a framework for generating
   files
*/

class DelimitedFileGenerator extends FileGenerator
{

//========================================================================

   var $delimiter;
   var $contents;
   var $columnArray;

//========================================================================

   /*!
   @function DelimitedFileGenerator( $delimiter = "," )
   @param $delimiter - the string to seperate values by
      this may not contain a newline char
   @discussion - constructs a new DelimitedFileGenerator object
   @result - constructs a new DelimitedFileGenerator object
   */
   
   function DelimitedFileGenerator( $delimiter = UNSPECIFIED )
   {
      if ( $delimiter === UNSPECIFIED ) $delimiter = ",";
      
      //require
      {
         assert( "!( is_integer( strpos( $delimiter, '\n' ) ) )" ); // don't let it contain a \n
      }
      //body
      {
         $this->delimiter = $delimiter;
         $this->FileGenerator( );
      }
      //ensure
      {
         assert( "\$this->_invariant()" );
      }
   }


//========================================================================

   /*!
   @function setColumns( $columnArray )
   @param $columnArray - the array of column names
      these will also be added as the header row for the output file 
   @discussion - resets and adds a header row for the contents of the file
   @result - nothing returned
   */

   function setColumns( $columnArray )
   {
      assert( "is_array( \$columnArray )" );
      assert( "count(\$columnArray) > 0" );   

      $this->columnArray =& $columnArray;
      array_walk( $columnArray, array( $this, '_wrapWithQuotes' ) );
         
      $this->contents = implode( $this->delimiter, $columnArray ) . "\n";
   }

//========================================================================

   /*!
   @function addRow( $rowArray )
   @param $rowHash - the array of data values to add to the file, must be in proper order to match cols above
      no checks are done on array size other than > 0
   @discussion - adds row to output, HOWEVER it does nothing to output if array has no elements
   @result - returns nothing
   */

   function addRow( $rowArray )
   {
      assert( "is_array( \$rowArray )" );

      if( empty( $rowArray ) )return;
         
      array_walk ( $rowArray, array( $this, '_wrapWithQuotes' ) );
      $this->contents .= implode( $this->delimiter, $rowArray ) . "\n";
   }


//========================================================================

   /*!
   @function _wrapWithQuotes( &$valToEsc, $key )
   @param $valToEsc - the data value that should gets quotes
   @discussion - escapes special chars in the value
   @result - the value, properly quoted
   */

   function _wrapWithQuotes( &$valToEsc, $key )
   {
      // need to escape quotes with extra quotes
      // need to tack on quotes and return
      $valToEsc = '"'.str_replace('"', '""', $valToEsc).'"';      
   }

//========================================================================

   /*!
   @function _escapeDelimiter( $valToEsc )
   @param $valToEsc - the data value that may need some escape char interjects
   @discussion - escapes special chars in the value
   @result - the value, properly escaped
   */

//   function _escapeDelimiter( $valToEsc )
//   {
//      // let's reuse this creation since it would otherwise be created 1000's of times over and over
//      if( !isset($this->escapedDelimiter) )
//      {
//         // create the escaped version of the delimiter
//         $delCharArray = preg_split('//', $this->delimiter);
//         
//         $numCharsInDel = count($delCharArray);
//         
//         for( $i=0; $i < $numCharsInDel; $i++)
//         {
//            $this->escapedDelimiter .= $this->escaper.$delCharArray[$i];
//         }
//      }
//      
//      return str_replace($this->delimiter, $this->escapedDelimiter, $valToEsc);
//   
//   }
//  
//========================================================================

   /*!
   @function getExtension()
   @discussion - provides the extension to be used with the file being
      generated
      
      NOTE: This method must be defined by a concrete child class.
   @result - returns a String the extension - the String may be empty      
   */

   function getExtension()
   {
      if( $this->delimiter == "," )
         {return "csv";}
         
      elseif( $this->delimiter == "\t" )
         {return "tab";}
      
      // if nothing found above, then
      return "txt";   
   }
   
//========================================================================

   /*!
   @function getDefaultName()
   @discussion - provides the default name for the file being generated
      WITHOUT the extension
   
      NOTE: This method must be defined by a concrete child class.
   @result - returns a non-empty String
   */   
   
   function getDefaultName()
   {
      return "delimitedTextData";   
   }
   
//========================================================================

   /*!
   @function getContents()
   @discussion - provides the contents of the file being generated
      
      NOTE: This method must be defined by a concrete child class.
   @result - returns a String
   */   
   
   function getContents()
   {
      return $this->contents;   
   }

//========================================================================

   /*!
   @function isEmpty()
   @discussion - indicates if the contents of the file being generated
      will be empty
      
      NOTE: This method must be defined by a concrete child class.
   @result - returns true if the contents are empty;
      otherwise returns false
   */
   
   function isEmpty()
   {      
      return empty( $this->contents );      
   }
      
//========================================================================

   /*!
   @function getMIMEType()
   @discusion - provides the MIME type of the file being generated
   
      NOTE: This method must be defined by a concrete child class.
   @result - returns a non-empty String
   */
    
   function getMIMEType()
   {
      return "text/plain"; // prob should be application/download
   }
   
//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */
    
   function _invariant()
   {
      return parent::_invariant();
   }

//========================================================================

};

?>