<?php

/*
FileGenerator
$Author: dougqh $
$Revision: 1.2 $
$Date: 2002/11/11 14:24:52 $
$Log: FileGenerator.php,v $
Revision 1.2  2002/11/11 14:24:52  dougqh
Changed base class to HeavyObject.

Revision 1.1  2002/09/09 17:26:06  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/

/*!
@class FileGenerator
@abstract - an abstract class that provides a framework for generating
   files
*/

class FileGenerator extends HeavyObject
{

//========================================================================

   var $name;

//========================================================================

   /*!
   @function FileGenerator()
   @discussion - constructs a new FileGenerator object
   @result - constructs a new FileGenerator object
   */
   
   function FileGenerator()
   {
      //require
      {
         assert( "isConcrete( \$this, 'FileGenerator',
            new_array(
                'getDefaultName',
                'getContents',
                'getExtension',
                'getMIMEType',
                'isEmpty' ) )" );
      }
      //body
      {
         //FileGenerators frequently end up stored in restricted application
         //variables after being stored in session variables
         //therefore I am allowing FileGenerators to be copied at the time
         //of serialization         
         $this->HeavyObject( true );
      }
      //ensure
      {
         assert( "\$this->_invariant()" );
      }
   }

//========================================================================

   /*!
   @function setName( [ $name ] )
   @param $name - a non-empty Sting containing a name
      - the extension should NOT be included in the name
   @discussion - sets the name of the file being generated
      - if $name is ommitted the default name of the file will be used
   */
   
   function setName( $name = UNSPECIFIED )
   {
      //require
      {
         assert( "\$this->_invariant()" );
      }
      //body
      {
         if ( $name === UNSPECIFIED )
         {
            //unsetting the name will cause getName to use the value
            //returned by getDefaultName
            
            unset( $this->name );
         }
         else
         {
            require( "\$name != ''" );
            
            $this->name = $name;
            
            assert( "\$this->name == \$name" );
         }
      }
      //ensure
      {
         assert( "\$this->_invariant()" );
      }
   }
   
//========================================================================

   /*!
   @function getName()   
   @discussion - provides the name to be used WITHOUT the extension
   @result - returns a String containing the name
   */
   
   function getName()
   {
      //require
      {
         assert( "\$this->_invariant()" );
      }
      //body
      {
         if ( isset( $this->name ) )
         {
            $result =  $this->name;
         }
         else
         {
            $result = $this->getDefaultName();
         }
      }
      //ensure
      {
         assert( "\$result != ''" );
         
         assert( "\$this->_invariant()" );
      }
      
      return $result;
   }

//========================================================================

   /*!
   @function getFileName( $suffix )
   @param $suffix - an optional suffix to place after the basic file name,
      but before the extension
   @discussion - provides the name of a file to be generated WITH the
      extension
   @result - returns a String
   */
   
   function getFileName( $suffix = UNSPECIFIED )
   {
      //let getName do the assertion checks
      
      $extension = $this->getExtension();
      
      if ( $suffix === UNSPECIFIED )
      {
         $suffix = "";
      }
      else
      {
         $suffix = "_" . $suffix;
      }
      
      if ( $extension != '' )
      {
         return stripNonFileNameCharacters( $this->getName() . $suffix ) . "." . $extension;
      }
      else
      {
         return stripNonFileNameCharacters( $this->getName() ) . $suffix;
      }     
   }

  
//========================================================================

   /*!
   @function getExtension()
   @discussion - provides the extension to be used with the file being
      generated
      
      NOTE: This method must be defined by a concrete child class.
   @result - returns a String the extension - the String may be empty      
   */

   /* abstract static function getExtension(); */
   
//========================================================================

   /*!
   @function getDefaultName()
   @discussion - provides the default name for the file being generated
      WITHOUT the extension
   
      NOTE: This method must be defined by a concrete child class.
   @result - returns a non-empty String
   */   
   
   /* abstract function getDefaultName(); */
   
//========================================================================

   /*!
   @function getContents()
   @discussion - provides the contents of the file being generated
      
      NOTE: This method must be defined by a concrete child class.
   @result - returns a String
   */   
   
   /* abstract function getContents(); */

//========================================================================

   /*!
   @function isEmpty()
   @discussion - indicates if the contents of the file being generated
      will be empty
      
      NOTE: This method must be defined by a concrete child class.
   @result - returns true if the contents are empty;
      otherwise returns false
   */
   
   /* abstract function isEmpty(); */
      
//========================================================================

   /*!
   @function getMIMEType()
   @discusion - provides the MIME type of the file being generated
   
      NOTE: This method must be defined by a concrete child class.
   @result - returns a non-empty String
   */
    
   /* abstract static function getMIMEType(); */
   
//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */
    
   function _invariant()
   {
      return parent::_invariant();
   }

//========================================================================

};

?>