<?php

/*
ImageMap
$Author: dougqh $
$Revision: 1.1 $
$Date: 2003/09/10 13:33:22 $
$Log: ImageMap.php,v $
Revision 1.1  2003/09/10 13:33:22  dougqh
Initial revision

Revision 1.6  2002/10/09 14:10:14  dougqh
Stripping false off onclicks to avoid javascript errors.

Revision 1.5  2002/10/03 13:25:35  millz
update to addLinkArea to allow for onClick

Revision 1.4  2002/09/24 19:46:02  dougqh
Changing quotes to ticks.

Revision 1.3  2002/07/31 18:31:16  dougqh
Automatically generating map name to avoid naming conflicts.

Revision 1.2  2002/07/22 18:48:10  dougqh
Quick fix to make area HTML XHTML compliant.

Revision 1.1  2002/07/22 18:43:01  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/

/*!
@class ImageMap
@abstract - a concrete class represents an ImageMap
@discussion - ImageMap does NOT implement HTMLComponent; however, it
   does generate HTML which is used by the Image class when an ImageMap
   is specified.
*/

class ImageMap extends Object
{

//========================================================================

   var $name;
   var $areasHTML;

//========================================================================

   /*!
   @function ImageMap( [ $name ] )
   @param $name - an optional non-empty String - if not specified a unique
      name will be generated
   @discussion - constructs a new ImageMap object
   @result - constructs a new ImageMap object
   */
   
   function ImageMap( $name = UNSPECIFIED )
   {
      //require
      {
         assert( '$name != ""' );
      }
      //body
      {         
         //order is important because of reliance on object_id
         $this->Object();
         
         $this->name = ( $name === UNSPECIFIED ) ? $this->object_id : $name;
         $this->areasHTML = "";
      }
      //ensure
      {
         assert( '$this->_invariant()' );
      }
   }

//========================================================================

   /*!
   @function addLinkArea( $shape, $link, $alternateText )
   @param $shape - a Shape that defines the area
   @param $link - a Link object
   @param $alternateText - alternate text for this area
   @discussion - makes the specified area map to the URL defined
      by the Link object
   */
   
   function addLinkArea( &$shape, &$link, $alternateText )
   {
      //require
      {
         assert( 'isInstanceOf( $shape, "Shape" )' );
         assert( 'isInstanceOf( $link, "Link" )' );
         assert( '$alternateText != ""' );
         
         assert( '$this->_invariant()' );
      }
      //body
      {
         
         if( $link->hasOnClick() )
         { 
            $tempHref = 'javascript:'.$link->getOnClick();
            //need to remove any 'return false;' on the end of the string to
            //avoid a javascript error
            
            $length = strlen( $tempHref );
            
            //12 is strlen( 'return false' )
            //also check 13 - looking for 'return false;'
            
            $tempHref = str_replace(
               array( 'return false;' , 'return false' ),
               '',
               $tempHref );
         }
         else
         {
            $url =& $link->getURL();
            $tempHref = $url->getURLString();
         }
         
         $this->areasHTML .= '<area shape="' . $shape->getMapShape() .
            '" coords="' . $shape->getMapCoordinates() .
            '" href="' . $tempHref .
            '" alt="' . nl2br( htmlentities( $alternateText ) ) . '" />' . NL;
      }
      //ensure
      {
         assert( "\$this->_invariant()" );
      }
   }

//========================================================================

   /*!
   @function addOperationArea( &$shape, &$operation, [ $alternateText ] )
   @param $shape - a Shape that definies the area
   @param $operation - an Operation to be performed when the area is clicked
   @param $alternateText - alternate text for this area
      defaults to the name of the provided Operation
   @discussion - makes an area that when clicked executes the specified
      Operation
   */
   
   function addOperationArea( &$shape, &$operation, $alternateText = UNSPECIFIED )
   {
      assert( 'isInstanceOf( $operation, "Operation" )' );
      
      $this->addLinkArea(
         $shape,
         $operation->getLink(),
         ( $alternateText === UNSPECIFIED ? $operation->getName() : $alternateText ) );
   }
   
//========================================================================

   /*!
   @function getName()
   @discussion - provides the name of this ImageMap
   @result - returns a non-empty String
   */
   
   function getName()
   {
      return $this->name;
   }
   
//========================================================================

   /*!
   @function getMapHTML()
   @discussion - provides the HTML that makes up the map tag
   @result - returns a String containing HTML
   */
   
   function getMapHTML()
   {
      assert( '$this->_invariant()' );
      
      return '<map name="' . $this->name . '">' . NL . $this->areasHTML . '</map>' . NL;
   }
      
//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */
    
   function _invariant()
   {
      return parent::_invariant() &&
             ( $this->name != '' ) &&
             isset( $this->areasHTML );
   }

//========================================================================

};

?>