<?php

/*
ALightFormField
$Author: dougqh $
$Revision: 1.2 $
$Date: 2003/12/30 14:25:45 $
$Log: ALightFormField.php,v $
Revision 1.2  2003/12/30 14:25:45  dougqh
- 3925

Revision 1.1  2003/09/29 18:12:30  dougqh
Initial revision

Revision 1.1  2003/07/09 22:24:07  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

importClass( 'HTMLComponents.AHTMLComponent' );

//========================================================================

/*!
@class ALightFormField
@abstract - an abstract class from concrete LightFormFields are derived
*/

class ALightFormField extends AHTMLComponent
{

//========================================================================

   /*!
   @function ALightFormField( $name, $displayName )
   @param $name - an internal String - non-empty
   @param $displayName - a user-presentable String - non-empty
   @discussion - constructs a new ALightFormField object
   @result - constructs a new ALightFormField object
   */

   function ALightFormField( $name, $displayName )
   {
      $this->name = $name;
      $this->displayName = $displayName;
      $this->isEnabled = true;
      $this->isRequired = false;

      $this->AHTMLComponent();
   }

//========================================================================

   /*!
   @function getName()
   @discussion - provides the internal name for this field
   @result - returns a String
   */

   function getName()
   {
      return $this->name;
   }

//========================================================================

   /*!
   @function getDisplayName()
   @discussion - provides a user-presentable name for this field
   @result - returns a user-presentable String
   */

   function getDisplayName()
   {
      return $this->displayName;
   }

//========================================================================

   /*!
   @function setEnabled( $isEnabled )
   @param $isEnabled - a boolean containing the new enabled status of this
      field
   @discussion - sets the enabled status of this field
   */

   function setEnabled( $isEnabled )
   {
      $this->isEnabled = $isEnabled;
   }

//========================================================================

   /*!
   @function isEnabled()
   @discussion - indicates if this ALightField is enabled
   @result - returns a boolean
   */

   function isEnabled()
   {
      return $this->isEnabled;
   }

//========================================================================

   /*!
   @function setRequired( $isRequired )
   @param $isRequired - indicates whether or not this field is
      required
   @discussion - sets the required status of this field
   */

   function setRequired( $isRequired )
   {
      $this->isRequired = $isRequired;
   }

//========================================================================

   /*!
   @function isRequired()
   @discussion - indicates if this field is required
   @result - returns a boolean
   */

   function isRequired()
   {
      return $this->isRequired;
   }

//========================================================================

   /*!
   @function getHTML()
   @discussion - provides the HTML for this field
   @result - a String of HTML
   */

   /* final */
   function getHTML()
   {
      return ( $this->isEnabled ) ?
         $this->_getEnabledHTML() :
         $this->_getDisabledHTML();
   }

//========================================================================

   /*!
   @function _getEnabledHTML()
   @discussion - called to get the HTML of this field when its enabled
   @result - returns a String of HTML
   */

   /* abstract function _getEnabledHTML(); */

//========================================================================

   /*!
   @function _getDisabledHTML()
   @discussion - called to get the HTML of this field when its disabled
   @result - returns a String of HTML
   */

   /* abstract function _getDisabledHTML(); */

//========================================================================

   /*!
   @function setValue( $value )
   @param $value - Any
   @discussion - sets the value of this field
   */

   function setValue( &$value )
   {
      $this->value =& $value;
   }

//========================================================================

   /*!
   @function getValue()
   @discussion - provides the value provided in the REQUEST hash
   */

   function &getValue()
   {
      if ( ! isset( $this->value ) )
      {
         $this->value = $this->_getDefaultValue();
      }

      return $this->value;
   }

//========================================================================

   /*!
   @function loadFormData()
   @discussion - makes the value provided _getFormValue the current
      value of this field
   */

   function loadFormData()
   {
      if ( $this->isEnabled() )
      {
         $this->setValue( $this->_getFormValue() );
      }
   }

//========================================================================

   /*!
   @function _getFormValue()
   @discussion - provides the appropriate value from $_REQUEST into this
      field
   */

   /* abstract function &_getFormValue(); */

//========================================================================

   /*!
   @function _getDefaultValue()
   @discussion - provides the default value for this field
   @result - returns Any
   */

   /* abstract function &_getDefaultValue(); */

//========================================================================

   /*!
   @function _addError( $errorMsg )
   @param $errorMsg - a user-presentable String
   @discussion - adds an error message describing why the data in
      this field is invalid
   */

   function _addError( $errorMsg )
   {
      $this->errors .= $errorMsg . NL;
   }

//========================================================================

   /*!
   @function getErrors()
   @discussion - provides a user-presentable String describing why
      isValid returned false
   @result - returns a String
   */

   function getErrors()
   {
      return $this->errors;
   }

//========================================================================

   /*!
   @function isValid()
   @discussion - indicates if this field is in a valid state
   @result - returns a boolean
   */

   function isValid()
   {
      $this->errors = '';
      $valid = true;

      if ( $this->isRequired() && $this->isEmpty() )
      {
         $valid = false;

         $this->_addError( $this->getDisplayName() . ' is empty.' );
      }

      if ( ! $this->isEmpty() )
      {
         $valid = $valid && $this->isDataValid();
      }

      return $valid;
   }

//========================================================================

   /*!
   @function isEmpty()
   @discussion - indicates if this field is considered empty
   @result - returns a boolean
   */

   function isEmpty()
   {
      return isEmpty( $this->getValue() );
   }

//========================================================================

   /*!
   @function isDataValid()
   @discussion - does any additional checks that are necessary for
      this field

      Child can override if additional checks are necessary.  The method
      _addError should be used to add a user-presentable description of
      the problem.
   @result - returns a boolean
   */

   function isDataValid()
   {
      return true;
   }

//========================================================================

};

?>