<?php

/*
CheckBoxGroup
$Author: dougqh $
$Revision: 1.1 $
$Date: 2003/09/29 18:12:30 $
$Log: CheckBoxGroup.php,v $
Revision 1.1  2003/09/29 18:12:30  dougqh
Initial revision

Revision 1.4  2002/10/18 17:40:35  dougqh
Added methods getAllNames and getUncheckedNames.

Revision 1.3  2002/10/04 17:57:47  dougqh
Added method isChecked.

Revision 1.2  2002/10/03 19:01:29  millz
fix to addCheckbox

Revision 1.1  2002/10/03 18:05:21  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/

//========================================================================

importClass( 'HTMLComponents.LightForms.CheckBox' );

//========================================================================

/*!
@class CheckBoxGroup
@abstract - a LightForm element that represents a series of checkboxes
*/

class CheckBoxGroup extends Object
{

//========================================================================

   var $checkBoxHash;
   var $checkBoxes;
   var $numCheckBoxes;
   var $name;
   
//========================================================================

   /*!
   @function CheckBoxGroup( $name )
   @discussion - constructs a new CheckBoxGroup object
   @result - constructs a new CheckBoxGroup object
   */
   
   function CheckBoxGroup( $name )
   {
      //require
      {
      }
      //body
      {
         $this->name = $name;
         
         $this->checkBoxHash = new_hash();
         $this->checkBoxes = new_array();
         $this->numCheckBoxes = 0;
         
         $this->Object();
      }
      //ensure
      {
         assert( '$this->_invariant()' );
      }
   }

//========================================================================

   /*!
   @function getName()
   @discussion - provides the name of this CheckBoxGroup
   @result - returns a String
   */
   
   function getName()
   {
      assert( '$this->_invariant()' );
      
      return $this->name;
   }
   
//========================================================================

   /*!
   @function getAllNames()
   @discussion - provides an array of names for all the CheckBoxes
      in this CheckBoxGroup
   @result - returns an array of Strings
   */
   
   function getAllNames()
   {
      return array_keys( $this->checkBoxHash );
   }   
   
//========================================================================

   /*!
   @function addCheckbox( $name, [ $checked ] )
   @param $name - a name for the new Checkbox
   @param $checked - a boolean indicating if the box is checked
   @discussion - adds a new CheckBox to this CheckBoxGroup
   */
   
   function addCheckbox( $name, $checked = UNSPECIFIED )
   {
      //imported CheckBox unconditionally at top of file
      $checkbox =& new CheckBox( $this->name . '_' . $name, $checked );
      
      $this->checkBoxHash{ $name } =& $checkbox;
      $this->checkBoxes[ $this->numCheckBoxes++ ] =& $checkbox;
   }

//========================================================================

   /*!
   @function getCheckBoxes()
   @discussion - provides all the CheckBoxes in this CheckBoxGroup
   @result - returns an array of CheckBoxes
   */
   
   function &getCheckBoxes()
   {
      return $this->checkBoxes;
   }

//========================================================================

   /*!
   @function getCheckBoxAt( $i )
   @param $i - index of the CheckBox in this group
   @discussion - provides the CheckBox at the specified index
   @result - returns a CheckBox
   */
   
   function &getCheckBoxAt( $i )
   {
      return $this->checkBoxes[ $i ];
   }

//========================================================================

   /*!
   @function getCheckBox( $name )
   @param $name - the name of the CheckBox to retrieve
   @discussion - provides the CheckBox in this CheckBoxGroup with
      the specified name
   @result - returns a CheckBox
   */
   
   function &getCheckBox( $name )
   {
      return $this->checkBoxHash{ $name };
   }

//========================================================================

   /*!
   @function loadValues( $submitMethod )
   @param $submitMethod - a FormSubmitMethod - GET or POST
   @discussion - loads all the values of the check boxes
   */
   
   function loadValues( $submitMethod )
   {
      for ( $i = 0; $i < $this->numCheckBoxes; ++$i )
      {
         $this->checkBoxes[ $i ]->loadValue( $submitMethod );
      }
   }

//========================================================================

   /*!
   @function getCheckedNames()
   @discussion - provides an array of names of all the CheckBoxes
      where isChecked is true
   @result - returns an array of Strings
   */
   
   function getCheckedNames()
   {
      $checkedNames = array();
      $numChecked = 0;
      
      $prefixLength = strlen( $this->name ) + 1;
      
      for ( $i = 0; $i < $this->numCheckBoxes; ++$i )
      {
         if ( $this->checkBoxes[ $i ]->isChecked() )
         {
            $checkedNames[ $numChecked++ ] = substr(
               $this->checkBoxes[ $i ]->getName(),
               $prefixLength );
         }
      }
      
      return $checkedNames;
   }

//========================================================================

   /*!
   @function getUncheckedNames()
   @discussion - provides an array of names of all the CheckBoxes in this
      CheckBoxGroup that are not set
   @result - returns an array of Strings
   */
   
   function getUncheckedNames()
   {
      return array_diff( 
         $this->getAllNames(),
         $this->getCheckedNames() );
   }
   
//========================================================================

   /*!
   @function isChecked( $name )
   @param $name - 
   @result - returns a boolean
   */
   
   function isChecked( $name )
   {
      return $this->checkBoxHash{ $name }->isChecked();
   }
   
//========================================================================

   /*!
   @function getLastCheckBox()
   @discussion - provides the last CheckBox added to this CheckBoxGroup
   @result - returns a CheckBox
   */
   
   function &getLastCheckBox()
   {
      assert( '$this->_invariant()' );
      
      return $this->checkBoxes[ $this->numCheckBoxes - 1 ];
   }
   
//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */
    
   function _invariant()
   {
      return parent::_invariant();
   }

//========================================================================

};

?>