<?php

/*
ScriptBlock
$Author: millz $
$Revision: 1.9 $
$Date: 2003/03/27 19:57:56 $
$Log: ScriptBlock.php,v $
Revision 1.9  2003/03/27 19:57:56  millz
- bug fix

Revision 1.8  2003/03/27 19:20:20  millz
- updated to extend AHTMLComponent

Revision 1.7  2003/03/17 14:26:27  dougqh
New serializer.

Revision 1.7  2003/03/07 18:02:57  dougqh
Changed __customRestoreExcludedVariables to __customRestore.

Revision 1.6  2002/12/10 22:22:53  dougqh
- debugging

Revision 1.5  2002/11/22 16:43:43  millz
changed name of getJavaScriptBlock to createJavaScriptBlock

Revision 1.4  2002/09/17 15:50:00  dougqh
Debugging

Revision 1.3  2002/09/12 21:03:30  dougqh
Working on adding ScriptBlocks to the page head.

Revision 1.2  2002/09/04 15:31:20  millz
converted it to a heavyobject

Revision 1.1  2002/07/02 14:55:49  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/

//========================================================================

importClass( "HTMLComponents.AHTMLComponent" );

//========================================================================

/*!
@class ScriptBlock
@abstract - an HTMLComponent that generates JavaScript or
   VBScript script blocks 
*/

class ScriptBlock extends AHTMLComponent
{

//========================================================================

   var $scriptingLanguage;
   
   var $addedToHead;
   
//========================================================================

   /*!
   @function ScriptBlock( $scriptingLanguage )
   @param $scriptingLanguage - a ScriptingLanguage constant
   @discussion - constructs a new ScriptBlock object
      NOTE: This method should not be called directly.
      Call the static methods createJavaScriptBlock or getVBScriptBlock as
      needed.
   @result - constructs a new ScriptBlock object
   */
   
   /* private */
   function ScriptBlock( $scriptingLanguage )
   {
      //require
      {
         assert( "isInstanceOf( \$scriptingLanguage, 'ScriptingLanguage' )" );
      }
      //body
      {
         $this->scriptingLanguage = $scriptingLanguage;
         
         $this->addedToHead = false;
         $this->code = "";
         
         $this->AHTMLComponent();
      }
      //ensure
      {
         assert( "\$this->_invariant()" );
      }
   }
//========================================================================

   /*!
   @function createJavaScriptBlock()
   @discussion - provides a new ScriptBlock set up to accept
      JavaScript
   @result - returns a ScriptBlock
   */
   
   /* static */
   function &createJavaScriptBlock()
   {
      return new ScriptBlock( JAVASCRIPT );
   }

//========================================================================

   /*!
   @function getVBScriptBlock()
   @discussion - provides a new ScriptBlock set up to accept
      VBScript
   @result - returns a ScriptBlock
   */
   
   /* static */
   function &getVBScriptBlock()
   {
      return new ScriptBlock( VBSCRIPT );
   }

//========================================================================

   /*!
   @function getScriptingLanguage()
   @discussion - provides the ScriptingLanguage of this ScriptBlock
   @result - returns a ScriptingLanguage constant
   */
   
   function getScriptingLanguage()
   {
      assert( "\$this->_invariant()" );
      
      return $this->scriptingLanguage;
   }

//========================================================================

   /*!
   @function addCode( $codeString )
   @param $codeString - a String containing code to place in this
      ScriptBlock
   @discussion - adds raw code to this ScriptBlock
      NOTE: Use of this method is discouraged.  When possible use addFunction.
   */
   
   function addCode( $codeString )
   {     
      $this->code .= $codeString . "\n";
      
      assert( "\$this->_invariant()" );
   }

//========================================================================

   /*!
   @function addFunction( $scriptFunction )
   @param $scriptFunction - a ScriptFunction of the appropriate
      ScriptingLanguage
   @discussion - adds the code of the specified ScriptFunction to
      this ScriptBlock
   */
   
   function addFunction( $scriptFunction )
   {
      //require
      {
         assert( "isInstanceOf( \$scriptFunction, 'ScriptFunction' )" );
         assert( "\$scriptFunction->getScriptingLanguage() == \$this->getScriptingLanguage()" );
         
         assert( "\$this->_invariant()" );
      }
      //body
      {
         $this->code .= "\n" . $scriptFunction->getCode() . "\n";
      }
      //ensure
      {
         assert( "\$this->_invariant()" );
      }
   }

//========================================================================

   /*!
   @function getCode()
   @discussion - provides the code of this ScriptBlock
   */
   
   function getCode()
   {
      assert( "\$this->_invariant()" );
      
      return $this->code;
   }

//========================================================================

   /*!
   @function getHTML()
   @discussion - provides a String containing an HTML representation of
      this ScriptBlock
   @result - returns a non-empty String
   */
   
   function getHTML()
   {
      assert( "\$this->_invariant()" );
      
      if ( $this->scriptingLanguage == JAVASCRIPT )
      {
         $actualLanguage = "javascript";
         $comment = "//";
      }
      else // if ( $this->language == VBSCRIPT )
      {
         $actualLanguage = "vbscript";
         $comment = "";
      }
      
      $html = "<script language='" . $actualLanguage . "'>\n";
      $html .= "<!--\n";
      $html .= $this->code;
      $html .= $comment . "-->\n";
      $html .= "</script>\n";
      
      return $html;
   }

//========================================================================

   /*!
   @function addToPageHead()
   @discussion - adds this ScriptBlock into the head of the HTML
      page currently being generated
   */
   
   function addToPageHead()
   {
      if ( ! $this->addedToHead )
      {
         $this->addedToHead = true;
         
         ScriptBlock::_handleAddToHead( $this );
      }
   }

//========================================================================

   /*!
   @function __customRestore()
   @discussion - on the off chance a ScriptBlock has been serialized
      make sure it can still be added to the head of the new HTML page
      being generated
   */
   
   function __customRestore()
   {
      $this->addedToHead = false;
   }
   
//========================================================================

   /*!
   @function _handleAddToHead( $scriptBlock )
   @param $scriptBlock - a ScriptBlock object
   @discussion - notes that the specified ScriptBlock should be included
      in the page head
   */
   
   /* static */
   function _handleAddToHead( &$scriptBlock )
   {
      global $GL_scriptHeadHTML;
      
      if ( ! isset( $GL_scriptHeadHTML ) ) $GL_scriptHeadHTML = '';
      
      $GL_scriptHeadHTML .= $scriptBlock->getHTML();
   }

//========================================================================

   /*!
   @function getScriptHeadHTML()
   @discussion - provides the HTML for all Script blocks that
      have been registered using addToPageHead
   @result - returns a String
   */
   
   /* static */
   function getScriptHeadHTML()
   {
      global $GL_scriptHeadHTML;
      
      if ( ! isset( $GL_scriptHeadHTML ) ) return "";
      
      return $GL_scriptHeadHTML;
   }
   
//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */
    
   function _invariant()
   {
      return parent::_invariant() &&
             isInstanceOf( $this->scriptingLanguage, 'ScriptingLanguage' );
   }

//========================================================================

};

?>