<?php

/*
ScriptFunction
$Author: dougqh $
$Revision: 1.4 $
$Date: 2002/12/10 15:54:57 $
$Log: ScriptFunction.php,v $
Revision 1.4  2002/12/10 15:54:57  dougqh
- Minor correct to header comment.

Revision 1.3  2002/11/06 20:30:30  dougqh
?

Revision 1.2  2002/09/16 18:43:36  dougqh
Added static create functions.

Revision 1.1  2002/07/02 14:55:49  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/

/*!
@class ScriptFunction
@abstract - an abstract class that provides the basis for the
   concrete classes JavaScriptFunction and VBScriptFunction
*/

class ScriptFunction extends Object
{

//========================================================================

   /*!
   @function ScriptFunction( $name, [ $parameters ] ) 
   @param $name - a non-empty String containing a name for this ScriptFunction
   @param $parameters - an optional array of Strings that contains the names of
      the parameters for this ScriptFunction
   @discussion - constructs a new ScriptFunction object
   @result - constructs a new ScriptFunction object
   */
   
   /* protected */
   function ScriptFunction( $name, $parameters = UNSPECIFIED )
   {
      if ( $parameters === UNSPECIFIED )
      {
         $parameters = new_array();
      }
      
      //require
      {
         assert( "\$name != ''" );
         assert( "isInstanceOf( \$parameters, 'String[]' )" );
         
         assert( "isConcrete( \$this, 'ScriptFunction',
            new_array( 'getScriptingLanguage', '_getFunctionHead', '_getFunctionTail' ) )" );
      }
      //body
      {
         $this->name = $name;
         $this->parameters = $parameters;
         $this->body = "";
      
         $this->Object();
      }
      //ensure
      {
      }
   }

//========================================================================

   /*!
   @function getJavaScriptFunction( $name, [ $parameters ] )
   @param $name - a non-empty String containing a name for this ScriptFunction
   @param $parameters - an optional array of Strings that contains the names of
      the parameters for this ScriptFunction
   @param $body - a String containing the body code for this ScriptFunction
   @discussion - provides a new ScriptFunction to use with a JavaScript
      ScriptBlock
   @result - returns a ScriptFunction
   */
      
   /* static */
   function &getJavaScriptFunction( $name, $parameters = UNSPECIFIED )
   {
      return ScriptFunction::createJavaScriptFunction( $name, $parameters );
   }

//========================================================================

   /*!
   @function createJavaScriptFunction( $name, [ $parameters ] )
   @param $name - a non-empty String containing a name for this ScriptFunction
   @param $parameters - an optional array of Strings that contains the names of
      the parameters for this ScriptFunction
   @param $body - a String containing the body code for this ScriptFunction
   @discussion - provides a new ScriptFunction to use with a JavaScript
      ScriptBlock
   @result - returns a ScriptFunction
   */
      
   /* static */
   function &createJavaScriptFunction( $name, $parameters = UNSPECIFIED )
   {
      static $imported;
      
      if ( ! isset( $imported ) )
      {
         importClass( "HTMLComponents.Scripting.JavaScriptFunction" );
         
         $imported = true;
      }
      
      return new JavaScriptFunction( $name, $parameters );
   }

//========================================================================

   /*!
   @function createFormDataAppendJavaScriptFunction( $name, $formName )
   @discussion -
   @result - 
   */
   
   function &createFormDataAppendJavaScriptFunction( $name, $formName )
   {
      $jsFunction =& ScriptFunction::createJavaScriptFunction( $name, new_array( 'url' ) );
      $jsFunction->addCode( '//window.alert( url );' );
      $jsFunction->addCode( 'elements=window.document.forms.' . $formName . '.elements;' );
      $jsFunction->addCode( 'for ( i=0; i<elements.length; ++i )' );
      $jsFunction->addCode( '{' );
      $jsFunction->addCode( ' element = elements[ i ];' );
      $jsFunction->addCode( ' if ( element.type=="checkbox" || element.type=="radio")' );
      $jsFunction->addCode( ' {' );
      $jsFunction->addCode( ' if ( element.checked ) url += "&" + element.name + "=" + element.value;' );
      $jsFunction->addCode( ' } else {' );
      $jsFunction->addCode( '  if ( element.value )' );
      $jsFunction->addCode( '  {' );
      $jsFunction->addCode( '  url += "&" + element.name + "=" + element.value;' );
      $jsFunction->addCode( '  } else if ( element.selectedIndex ) {' );
      $jsFunction->addCode( '  url += "&" + element.name + "=" + element.options[ element.selectedIndex ].value;' );
      $jsFunction->addCode( '  }' );
      $jsFunction->addCode( ' }' );
      $jsFunction->addCode( '}' );
      $jsFunction->addCode( '//window.alert( url );' );
      $jsFunction->addCode( 'return url;' );
      
      return $jsFunction;
   }
      
//========================================================================

   /*!
   @function getVBScriptFunction( $name, [ $parameters ] )
   @param $name - a non-empty String containing a name for this ScriptFunction
   @param $parameters - an optional array of Strings that contains the names of
      the parameters for this ScriptFunction
   @param $body - a String containing the body code for this ScriptFunction
   @discussion - provides a new ScriptFunction to use with a VBScript
      ScriptBlock
   @result - returns a ScriptFunction
   */
   
   /* static */
   function &getVBScriptFunction( $name, $parameters = UNSPECIFIED )
   {
      return ScriptFunction::getVBScriptFunction( $name, $parameters );
   }

//========================================================================

   /*!
   @function createVBScriptFunction( $name, [ $parameters ] )
   @param $name - a non-empty String containing a name for this ScriptFunction
   @param $parameters - an optional array of Strings that contains the names of
      the parameters for this ScriptFunction
   @param $body - a String containing the body code for this ScriptFunction
   @discussion - provides a new ScriptFunction to use with a VBScript
      ScriptBlock
   @result - returns a ScriptFunction
   */
   
   /* static */
   function &createVBScriptFunction( $name, $parameters = UNSPECIFIED )
   {
      static $imported;
      
      if ( ! isset( $imported ) )
      {
         importClass( "HTMLComponents.Scripting.VBScriptFunction" );
         
         $imported = true;
      }
      
      return new VBScriptFunction( $name, $parameters );
   }
   
//========================================================================

   /*!
   @function addCode( $code )
   @param $code - a String containing code
   @discussion - adds $code to the end of the body of this ScriptFunction
   */
   
   function addCode( $code )
   {     
      $this->body .= "\n" . $code;
      
      assert( "\$this->_invariant()" );
   }
   
//========================================================================

   /*!
   @function getBody()
   @discussion - provides the code that is the body of this function
   @result - returns a String containing the code that comprises the
      body of this function
   */
   
   function getBody()
   {
      assert( "\$this->_invariant()" );
      
      return $this->body;
   }

//========================================================================

   /*!
   @function getCode()
   @discussion - provides all the code of this 
   @result - returns a String containing code
   */
   
   function getCode()
   {
      assert( "\$this->_invariant()" );
      
      // "\n" is contained at the beginning of $this->body
      return $this->_getFunctionHead() . 
             $this->body . "\n" .
             $this->_getFunctionTail();
   }

//========================================================================

   /*!
   @function getScriptLanguage()
   @discussion - indicates the ScriptingLanguage of this ScriptFunction
      NOTE: This method must be defined by a concrete subclass of ScriptFunction
   @result - returns a ScriptingLanguage constant
   */
   
   /* abstract function getScriptLanguage(); */   

//========================================================================

   /*!
   @function _getFunctionHead()
   @discussion - provides the portions of the function that precede
      the body of the function
   @result - returns a String
   */
   
   /* abstract function _getFunctionHead(); */
   
//========================================================================

   /*!
   @function _getFunctionTail()
   @discussion - provides the portions of the function that follow the
      body of the function
   @result - returns a String
   */
   
   /* abstract function _getFunctionTail(); */
   
//========================================================================

   /*!
   @function _getParameterList()
   @discussion - provides a comma separated list of parameters.
      This method is mainly provided as convenience for use by the
      concrete subclasses.
   @result - returns a String containing a comma separated list of
      parameters
   */
   
   function _getParameterList()
   {
      return implode( ", ", $this->parameters );
   }
      
//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */
    
   function _invariant()
   {
      return parent::_invariant() &&
             ( $this->name != '' );
   }

//========================================================================

};

?>