<?php

/*
FourColumnTable
$Author: apache $
$Revision: 1.1 $
$Date: 2003/12/18 20:39:26 $
$Log: FourColumnTable.php,v $
Revision 1.1  2003/12/18 20:39:26  apache
Initial revision

Revision 1.1  2003/07/09 21:46:24  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

importClass( 'HTMLComponents.AHTMLComponent' );

//========================================================================

/*!
@class FourColumnTable
@abstract - a wrapper around a standard Table
   Produces a table with four columns with the configuration.
   Typically of the form Label, Component, Label, Component
@discussion - Note this class produces a table, but does NOT
   extend the regular Table class.
*/

class FourColumnTable extends AHTMLComponent
{

//========================================================================

   //TODO: PHP5 make these constants
   //Ideally, these would be class constants.
   var $NUM_CELLS = 5;
   var $COL_NUM_CELLS = 2;

//========================================================================

   /*!
   @function FourColumnTable()
   @discussion - constructs a new FourColumnTable object
   @result - constructs a new FourColumnTable object
   */

   function FourColumnTable()
   {
      importClass( 'HTMLComponents.Table.Table' );
      $this->table =& new Table();
      $this->table->setCellPadding( 2 );
      $this->hasPropagatedDimensions = false;

      importClass( 'HTMLComponents.Text' );
      $this->labelText =& Text::createBoldText( 'dummy' );

      //set up some prototypical cells to reuse
      importClass( 'HTMLComponents.Table.TableCell' );
      $this->fullLabelCell =& new TableCell( toRef( NULL ) );
      $this->fullLabelCell->disableWrapping();
      $this->fullLabelCell->setColumnSpan( $this->NUM_CELLS );
      $this->fullLabelCell->setHorizontalAlignment( LEFT );
      $this->fullLabelCell->setVerticalAlignment( TOP );

      $this->fullCompCell =& new TableCell( toRef( NULL ) );
      $this->fullCompCell->setColumnSpan( $this->NUM_CELLS );

      $this->halfLabelCell =& new TableCell( toRef( NULL ) );
      //$this->halfLabelCell->disableWrapping();
      $this->halfLabelCell->setHorizontalAlignment( RIGHT );
      $this->halfLabelCell->setVerticalAlignment( TOP );

      $this->halfCompCell =& new TableCell( toRef( NULL ) );
      $this->halfCompCell->setVerticalAlignment( TOP );

      importClass( 'Display.Color' );
      $this->groupColor =& Color::getColor( 236, 236, 236 );

      $this->groupLabelCell =& new TableCell( toRef( NULL ) );
      $this->groupLabelCell->setBackgroundColor( $this->groupColor );
      $this->groupLabelCell->setHorizontalAlignment( CENTER );

      $this->isGroupMode = false;
      $this->activeGroupIndex = -1;

      $this->useNewRow = true;

      $this->AHTMLComponent();
   }

//========================================================================

   /*!
   @function forceNewRow()
   @discussion - forces the next add to be done on a new row
   */

   function forceNewRow()
   {
      $this->useNewRow = true;
   }

//========================================================================

   /*!
   @function isNextLeft()
   @discussion - indicates if the next add is to left pair of columns
   @result - returns a boolean
   */

   function isNextLeft()
   {
      return $this->useNewRow;
   }

//========================================================================

   /*!
   @function isNextRight()
   @discussion - indicates if the next add is to right pair of columns
   @result - returns a boolean
   */

   function isNextRight()
   {
      return ( ! $this->useNewRow );
   }

//========================================================================

   /*!
   @function add( $label, $htmlComponent, [ $forceFull ] )
   @param $label - a user presentable String
   $param $htmlComponent - an AHTMLComponent
   @param $forceFull - optional boolean - that indicates if $htmlComponent
      should be forced to take up a full row

      Not using $forceFull or setting it to false does NOT guarantee
      that $htmlComponent will NOT be put on its own row.
   @discussion - adds a label component pair
   */

   function add( $label, &$htmlComponent, $forceFull = UNSPECIFIED )
   {
      if ( ! $this->hasPropagatedDimensions )
      {
         $this->table->setRequestedDimensions(
            $this->getDimensions() );
         $this->hasPropagatedDimensions = true;
      }

      $forceFull = ( $forceFull === UNSPECIFIED ) ? false : $forceFull;

      if ( $this->isGroupMode )
      {
         $this->_addToGroup( $label, $htmlComponent, $forceFull );
      }
      else
      {
         $this->_addToTable( $label, $htmlComponent, $forceFull );
      }
   }

//========================================================================

   /*!
   @function addNamed( $namedHTMLComponent, [ $forceFull ] )
   $param $namedHTMLComponent - an AHTMLComponent that also
      supports getDisplayName
   @param $forceFull - optional boolean - that indicates if $htmlComponent
      should be forced to take up a full row

      Not using $forceFull or setting it to false does NOT guarantee
      that $namedHTMLComponent will NOT be put on its own row.
   @discussion - similar to add, but uses the displayName
      of the namedHTMLComponent as the label
   */

   function addNamed( &$namedHTMLComponent, $forceFull = UNSPECIFIED )
   {
      $this->add(
         $namedHTMLComponent->getDisplayName(),
         $namedHTMLComponent,
         $forceFull );
   }

//========================================================================

   /*!
   @function addField( $field, [ $forceFull ] )
   @param $field - an ALightFormField
   @discussion - similar to addNamed, but adds an "*" to the display name
      if the field is required
   */

   function addField( &$field, $forceFull = UNSPECIFIED )
   {
      $label = $field->getDisplayName();
      if ( $field->isRequired() ) $label .= '*';

      $this->add( $label, $field, $forceFull );
   }

//========================================================================

   /*!
   @function addSpacerRow()
   @discussion - adds a spacer row to the table
   */

   function addSpacerRow()
   {
      $this->table->addSpacerRow();
      $this->useNewRow = true;
   }

//========================================================================

   /*!
   @function addDividerRow()
   @discussion - adds a divider row to the table
   */

   function addDividerRow()
   {
      $this->table->addDividerRow();
      $this->useNewRow = true;
   }

//========================================================================

   /*!
   @function _addToTable( $label, $htmlComponent, $forceFull, [ $isGroup ] )
   @param $label - a user presentable String
   $param $htmlComponent - an AHTMLComponent
   @param $forceFull - boolean that indicates if $htmlComponent
      should be forced to take up a full row
   @param $isGroup - optional parameter indicating that this a group field
   @discussion - helper method of add - called when a group is
      not active
   */

   function _addToTable( $label, &$htmlComponent, $forceFull, $isGroup = false )
   {
      $this->labelText->_setText( $label . ':' );

      if ( $forceFull )
      {
         if ( $isGroup )
         {
            $this->fullLabelCell->setBackgroundColor( $this->groupColor );
            $this->fullCompCell->setBackgroundColor( $this->groupColor );
         }

         //Have to explicitly set it each time because labelCell cheats
         //and does NOT hold onto the object.
         $this->fullLabelCell->_setHTMLComponent( $this->labelText );

         $this->table->startRow();
         $this->table->_addCell( $this->fullLabelCell );

         $this->table->startRow();
         $this->fullCompCell->_setHTMLComponent( $htmlComponent );
         $this->table->_addCell( $this->fullCompCell );

         if ( $isGroup )
         {
            $this->fullLabelCell->unsetBackgroundColor();
            $this->fullCompCell->unsetBackgroundColor();
         }
      }
      else
      {
         if ( $isGroup )
         {
            $this->halfLabelCell->setBackgroundColor( $this->groupColor );
            $this->halfCompCell->setBackgroundColor( $this->groupColor );
         }

         if ( $this->useNewRow )
         {
            $this->table->startRow();
         }
         else
         {
            $this->table->addSpacerCell();
         }

         $this->useNewRow = ( ! $this->useNewRow );

         //Have to explicitly set it each time because labelCell cheats
         //and does NOT hold onto the object.
         $this->halfLabelCell->_setHTMLComponent( $this->labelText );
         $this->table->_addCell( $this->halfLabelCell );

         $this->halfCompCell->_setHTMLComponent( $htmlComponent );
         $this->table->_addCell( $this->halfCompCell );

         if ( $isGroup )
         {
            $this->halfLabelCell->unsetBackgroundColor();
            $this->halfCompCell->unsetBackgroundColor();
         }
      }
   }

//========================================================================

   /*!
   @function _addToGroup( $label, $htmlComponent, $forceFull )
   @param $label - a user presentable String
   $param $htmlComponent - an AHTMLComponent
   @param $forceFull - boolean that indicates if $htmlComponent
      should be forced to take up a full row
   @discussion - helper method of add - called when a group is
      active
   */

   function _addToGroup( $label, $htmlComponent, $forceFull )
   {
      $group =& $this->_getActiveGroup();

      $group{ 'forceFull' } = $group{ 'forceFull' } || $forceFull;

      if ( $group{ 'shouldRemove' } )
      {
         $element{ 'label' } =
            str_replace( $group{ 'label' }, '', $label );
      }
      else
      {
         $element{ 'label' } = $label;
      }

      $element{ 'htmlComponent' } =& $htmlComponent;
      $element{ 'forceFull' } = $forceFull;

      $group{ 'elements' }[] =& $element;
   }

//========================================================================

   /*!
   @function _getActiveGroup( [ $makeNew ] )
   @param $makeNew - optional boolean - that forces the creation
      of a new GroupStruct
   @discussion - provides the active group structure
   @result - returns a GroupStruct
   */

   function &_getActiveGroup( $makeNew = false )
   {
      if ( $makeNew )
      {
         $this->groups[ $this->activeGroupIndex ] = array();
      }

      return $this->groups[ $this->activeGroupIndex ];
   }

//========================================================================

   /*!
   @function startGroup( $label, $shouldRemove )
   @param $label - a user presentable String describing the group
   @param $shouldRemove - optional boolean - indicating if
      the group name should be removed if appears in the
      label of added label (case-sensitive) - defaults to false
   @discussion - starts a group within FourColumnTable
   */

   function startGroup( $label, $shouldRemove = false )
   {
      $this->activeGroupIndex = ( $this->activeGroupIndex + 1 ) % 2;

      $this->isGroupMode = true;

      //true - forces construction of a new GroupStruct
      $group =& $this->_getActiveGroup( true );
      $group{ 'label' } = $label;
      $group{ 'forceFull' } = false;
      $group{ 'shouldRemove' } = $shouldRemove;
      $group{ 'elements' } = array();
   }

//========================================================================

   /*!
   @function endGroup()
   @discussion - ends a group started by startGroup and
      adds it to the table
   */

   function endGroup()
   {
      $group =& $this->_getActiveGroup();

      if ( $group{ 'forceFull' } )
      {
         if ( $this->activeGroupIndex == 0 )
         {
            //left column group is too big - abandon using it in
            //a nicely formatted pair

            $this->_addGroupToTable( $group );

            //done with this group, so the next group can overwrite it
            $this->activeGroupIndex = -1;
         }
         else
         {
            //right column is too big - so just add both our groups
            //individually

            //add them both as individually
            $this->_addGroupToTable( $this->groups[ 0 ] );
            $this->_addGroupToTable( $this->groups[ 1 ] );
         }
      }
      else
      {
         //wait until we have to groups fully constructed and try
         //to put them in side-by-side columns
         if ( $this->activeGroupIndex == 1 )
         {
            $this->_addGroupsToTable( $this->groups[ 0 ], $this->groups[ 1 ] );
         }
      }

      $this->isGroupMode = false;
   }

//========================================================================

   /*!
   @function _addGroupToTable( $group )
   @param $group - a GroupStruct
   @discussion - used for the simple group add case where no
      coordination is needed between two groups
   */

   function _addGroupToTable( &$group )
   {
      $this->table->startRow();
      $this->_addGroupLabel( $group{ 'label' }, $this->NUM_CELLS );
      $this->forceNewRow();

      foreach ( $group{ 'elements' } as $curElement )
      {
         $this->_addToTable(
            $curElement{ 'label' },
            $curElement{ 'htmlComponent' },
            $curElement{ 'forceFull' },
            true );
      }
   }

//========================================================================

   /*!
   @function _addGroupsToTable( $groupA, $groupB )
   @param $groupA - a GroupStruct
   @param $groupB - a GroupStruct
   @discussion - handles the complicated situation where
      groupA and groupB can be placed in the table in side-by-side
      columns
   */

   function _addGroupsToTable( &$groupA, &$groupB )
   {
      $aElements =& $groupA{ 'elements' };
      $bElements =& $groupB{ 'elements' };

      $count = max(
         count( $aElements ),
         count( $bElements ) );

      $this->table->startRow();
      $this->_addGroupLabel( $groupA{ 'label' }, $this->COL_NUM_CELLS );
      $this->table->addSpacerCell();
      $this->_addGroupLabel( $groupB{ 'label' }, $this->COL_NUM_CELLS );
      $this->forceNewRow();

      for ( $i = 0; $i < $count; ++$i )
      {
         if ( isset( $aElements[ $i ] ) )
         {
            $this->_addToTable(
               $aElements[ $i ]{ 'label' },
               $aElements[ $i ]{ 'htmlComponent' },
               false,
               true );
         }
         else
         {
            $this->table->addEmptyCell( $this->COL_NUM_CELLS );
         }

         if ( isset( $bElements[ $i ] ) )
         {
            $this->_addToTable(
               $bElements[ $i ]{ 'label' },
               $bElements[ $i ]{ 'htmlComponent' },
               false,
               true );
         }
         else
         {
            $this->forceNewRow();
         }
      }
   }

//========================================================================

   /*!
   @function _addGroupLabel( $label, $colSpan )
   @param $label - a user-presentable String
   @param $colSpan - number of columns to span
   @discussion - helper used to add group headings to the table
   */

   function _addGroupLabel( $label, $colSpan )
   {
      importClass( 'HTMLComponents.Text' );

      $this->groupLabelCell->_setHTMLComponent(
         Text::createBoldText( $label ) );
      $this->groupLabelCell->setColumnSpan( $colSpan );

      $this->table->_addCell( $this->groupLabelCell );
   }

//========================================================================

   /*!
   @function getHTML()
   */

   function getHTML()
   {
      return $this->table->getHTML();
   }

//========================================================================

};

?>