<?php

/*
TwoColumnTable
$Author: dougqh $
$Revision: 1.3 $
$Date: 2004/04/01 18:09:30 $
$Log: TwoColumnTable.php,v $
Revision 1.3  2004/04/01 18:09:30  dougqh
- Added the addField method

Revision 1.2  2003/12/18 20:39:57  dougqh
- Removed a commented out block of debugging

Revision 1.1  2003/12/18 20:39:26  apache
Initial revision

Revision 1.10  2003/07/09 21:46:24  dougqh
Minor tweaks

Revision 1.9  2003/04/02 19:37:03  millz
- one more lowering of label_wrap_length

Revision 1.8  2003/04/02 18:44:54  millz
- adjusted the wrapping length

Revision 1.7  2003/04/01 22:03:29  millz
- debug

Revision 1.6  2003/03/28 15:30:02  millz
- added ability to detemrine when to make two rows for a given entry and when to wrap labels

Revision 1.5  2003/03/27 19:20:48  millz
- updated to extend AHTMLComponent

Revision 1.4  2002/07/30 20:06:47  dougqh
Added set methods for cell padding and cell spacing.

Revision 1.3  2002/07/25 19:34:24  dougqh
Minor modifications to implementation.
Added a new setWidths method that allows for width specifications
to be given to the two columns in the table.

Revision 1.2  2002/07/12 20:44:44  millz
updated table to have 60/40 cell widths

Revision 1.1  2002/07/08 21:24:24  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

   define( 'LABEL_WRAPPING_LENGTH', 20 );

//========================================================================

importClass( 'HTMLComponents.AHTMLComponent' );

//========================================================================

/*!
@class TwoColumnTable
@abstract - an HTMLComponent used for presenting two column tables
   where the left column is always a text label and the right column is
   any HTMLComponent
@discussion - TwoColumnTable is NOT a sub-class of Table, but its HTML
   representation is generated by using a Table internally.
*/

class TwoColumnTable extends AHTMLComponent
{

//========================================================================

   var $table;

   var $boldFont;

   var $leftCell;
   var $rightCell;

//========================================================================

   /*!
   @function TwoColumnTable()
   @discussion - constructs a new TwoColumnTable object
   @result - constructs a new TwoColumnTable object
   */

   function TwoColumnTable()
   {
      //require
      {
      }
      //body
      {
         importClass( "HTMLComponents.Table.Table" );
         $this->table =& new Table();
         $this->table->setCellSpacing( 3 );

         importClass( "HTMLComponents.Text" );
         $this->leftText =& new Text( "" );
         importClass( "Display.Font" );
         $this->leftText->setFont( Font::getFont( true ) );

         importClass( "HTMLComponents.Table.TableCell" );
         $null = NULL;
         $this->leftCell =& new TableCell( $null );
         $this->leftCell->disableWrapping();
         $this->leftCell->setHorizontalAlignment( RIGHT );
         $this->leftCell->setVerticalAlignment( TOP );

         $this->rightCell =& new TableCell( $null );

         $this->AHTMLComponent();
      }
      //ensure
      {
         assert( "\$this->_invariant()" );
      }
   }

//========================================================================

   /*!
   @function addField( $field )
   @param $field - an ALightFormField
   @discussion - similar to addNamed, but adds an "*" to the display name
      if the field is required
   */

   function addField( &$field )
   {
      $label = $field->getDisplayName();
      if ( $field->isRequired() ) $label .= '*';

      $this->addRow( $label, $field );
   }

//========================================================================

   /*!
   @function setCellPadding( $padding )
   @param $padding - the amount of padding to use for this TwoColumnTable
   @discussion - sets the cell padding for this TwoColumnTable
   */

   function setCellPadding( $padding )
   {
      $this->table->setCellPadding( $padding );
   }

//========================================================================

   /*!
   @function addSpacerRow( [ $color ] )
   @param $color - optional background Color for the spacer row
      - defaults to no Color specification
   @discussion - adds a row with a single cell containing a BR in
      it to the table
   */

   function addSpacerRow( $color = UNSPECIFIED )
   {
      $this->table->addSpacerRow( $color );
   }

//========================================================================

   /*!
   @function setCellSpacing( $spacing )
   @param $padding - the amount of spacing to use for this TwoColumnTable
   @discussion - sets the cell spacing for this TwoColumnTable
   */

   function setCellSpacing( $spacing )
   {
      $this->table->setCellSpacing( $spacing );
   }

//========================================================================

   /*!
   @function setWidth( $width )
   @param $width - a table width specifier absolute or by percentage
   @discussion - sets the width of this TwoColumnTable
   */

   function setWidth( $width )
   {
      $this->table->setWidth( $width );
   }

//========================================================================

   /*!
   @function setWidths( $leftWidth, $rightWidth )
   @param $leftWidth - a width specifier for the left column
   @param $rightWidth - a width specifier for the right column
   @discussion - sets the widths of the two columns to be used in
      this TwoColumnTable
   */

   function setWidths( $leftWidth, $rightWidth )
   {
      $this->leftCell->setWidth( $leftWidth );
      $this->rightCell->setWidth( $rightWidth );
   }

//========================================================================

   /*!
   @function disableRightColumnWrapping()
   @discussion - disables the wrapping of elements in the right column
   */

   function disableRightColumnWrapping()
   {
      $this->rightCell->disableWrapping();
   }

//========================================================================

   /*!
   @function addRow( $label, &$htmlComponent )
   @param $label - a String containing a label to display in the left cell
      - an empty string may be passed if no label is desired
   @param $htmlComponent - an HTMLComponent to display in the right cell
   @discussion - adds a row with the left cell containing the text $label
      and the right cell containing the HTMLComponent $htmlComponent
   */

   function addRow( $label, &$htmlComponent )
   {
      $this->table->startRow();

      $useTwoRows = false;
      $numCols = 1;
      $this->leftCell->setHorizontalAlignment( RIGHT );
      $this->rightCell->setHorizontalAlignment( LEFT );

      $useTwoRows = $this->_shouldUseTwoRows( $htmlComponent->getWidth() );

      if($useTwoRows)
      {
         $numCols = 2;
         $this->leftCell->setHorizontalAlignment( LEFT );
         $this->rightCell->setHorizontalAlignment( CENTER );
      }


      if ( $label == "" )
      {

         $this->table->addEmptyCell( $numCols );

      }
      else
      {
         $this->leftText->_setText( $label . ":" );
         $this->leftCell->_setHTMLComponent( $this->leftText );

         if( strlen($label) > LABEL_WRAPPING_LENGTH  && $numCols < 2 )
         {
            $this->leftCell->enableWrapping();
         }
         else
         {
            $this->leftCell->disableWrapping();
         }

         $this->leftCell->setColumnSpan( $numCols );

         $this->table->_addCell( $this->leftCell );
      }

      if( $useTwoRows ){$this->table->startRow();}

      $this->rightCell->setColumnSpan( $numCols );

      $this->rightCell->_setHTMLComponent( $htmlComponent );
      $this->table->_addCell( $this->rightCell );
   }

//========================================================================

   function _shouldUseTwoRows( $compWidth )
   {

      $useTwoRows = false;
      $thisWidth = $this->getWidth();

      // first we need to have a non-percent width that is actually specified for us
      // for the component in question and we have to know, in pixels, how big this TwoColumnTalbe is
      // without that info we have to assume false
      //print "We have integers of comp = $compWidth and this = $thisWidth".BR_NL;

      if(!isInteger($compWidth)){return false;}
      if(!isInteger($thisWidth)){return false;}



      $rightSideWidth = $this->rightCell->getWidth();


      if( $rightSideWidth !== UNSPECIFIED )
      {
         // let's see if we have a pixel width value for the rightside of the table
         // if we do we can use it for the comparison directly
         if( strpos( $rightSideWidth,'%' ) === false )  // this is not a percent val
         {
            if($compWidth >= $rightSideWidth )
            {
               $useTwoRows = true;
            }
         }
         else // we have a percent val for the right
         {
            $percentValRightSideWidth = str_replace( '%', '', $rightSideWidth );
            $fractionalRightSideWidth = $percentValRightSideWidth / 100;

            if($compWidth >= intval($fractionalRightSideWidth * $thisWidth) )
            {
               $useTwoRows = true;
            }

         }
      }
      elseif($compWidth >= 0.75 * $thisWidth )  // last catch for some things, we'll assume that if it is bigger
      {                                       // than 75% that we need two rows
         $useTwoRows = true;

      }

      return $useTwoRows;
   }

//========================================================================

   /*!
   @function getHTML()
   @discussion - provides the HTML representation of this TwoColumnTable
   @result - returns a String containing HTML
   */

   function getHTML()
   {
      assert( "\$this->_invariant()" );

      return $this->table->getHTML();
   }

//========================================================================

   /*
   @function _invariant()
   @discussion - defines a boolean expression that should always be true
      for this object
   @result - returns a boolean expression indicating if the object is in
      a valid state
   */

   function _invariant()
   {
      return parent::_invariant() &&
             isInstanceOf( $this->table, 'Table' ) &&
             isInstanceOf( $this->leftCell, 'TableCell' ) &&
             isInstanceOf( $this->leftText, 'Text' );
   }

//========================================================================

};

?>