<?php

/*
URLLibrary
$Author: dougqh $
$Revision: 1.5 $
$Date: 2004/01/08 15:27:18 $
$Log: URLLibrary.php,v $
Revision 1.5  2004/01/08 15:27:18  dougqh
- debugging

Revision 1.4  2003/12/19 14:09:34  dougqh
- Providing a user defined html_entities_decode if PHP is not providing one.

Revision 1.3  2003/12/17 14:42:23  dougqh
- Fixed an issue with array support of addParameterToURL

Revision 1.2  2003/12/16 20:24:45  dougqh
- Added array support to addParametersToURL.

Revision 1.1  2003/09/09 15:11:59  dougqh
Initial revision

Revision 1.1  2003/07/14 14:39:53  dougqh
Initial revision

Revision 1.1  2003/07/14 14:38:59  dougqh
Initial revision

IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

/*!
@function addParameterToURL( $url, $name, $value )
@param $url - a String containing the URL to add the parameter to
   - it may be empty for the purposes of building just a parameter string
@param $name - a non-empty String containing the name of the parameter
@param $value - a String containing the value of the parameter
@discussion - adds the specified parameter to the end of the URL String
@result - returns a new URL String with the parameter added to the end
*/

function addParameterToURL( $url, $name, $value )
{
   assert( '$name != ""' );

   if ( is_array( $value ) )
   {
      if ( empty( $value ) )
      {
         return $url;
      }
      else
      {
         foreach ( $value as $key => $subvalue )
         {
            $url = addParameterToURL( $url, $name . '[' . $key . ']', $subvalue );
         }
         return $url;
      }
   }
   else
   {
      if ( strpos( $url, '?' ) === false )
      {
         return $url . '?' . urlencode( $name ) . '=' . urlencode( $value );
      }
      else
      {
         return $url . '&' . urlencode( $name ) . '=' . urlencode( $value );
      }
   }
}

//========================================================================

/*!
@function addParametersToURL( $url, $params )
@param $url - a String containing the URL to add the parameter to

   NOTE: It may be empty for the purposes of building just a parameter string.
      In that case the parameter string will have a leading ? mark.
@param $params - a hash of the format { [ $name ] => $value, ... },
   where $name is the name of a parameter and $value is the value of that
   parameter
@discussion - adds all of the parameters contained in $params to the specified
   URL
@result - returns a new URL string with all the parameters added to the end
*/

function addParametersToURL( $url, $params )
{
   foreach ( $params as $name => $value )
   {
      $url = addParameterToURL( $url, $name, $value );
   }

   return $url;
}

//========================================================================

/*!
@function getServerOf( $url )
@param $url - a String containing a URL
@discussion - provides the server name portion of the URL string
@result - returns a String containing the just the server name from
   the URL String
*/

function getServerOf( $url )
{
   $url = str_replace( '\\', '/', $url );

   if ( isAbsoluteURL( $url ) )
   {
      //absolute URL

      //if it starts with https then startPos is strlen( 'https://' ) which is 8
      //otherwise strlen( 'http://' ) which is 7
      $startPos = ( strpos( $url, 'https://' ) === 0 ) ? 8 : 7;

      $slashPos = strpos( $url, '/', $startPos );

      if ( $slashPos === false )
      {
         //no slash read to the end
         $server = substr( $url, $startPos );
      }
      else
      {
         //slash found - read up to, but not including slash
         $server = substr( $url, $startPos, $slashPos - $startPos );
      }

      //TODO: Should really strip port 443 for https
      $server = _tryStripPort80( $server );
   }
   else
   {
      //relative URL

      $server = getServer();
   }

   return $server;
}

//========================================================================

/*!
@function isAbsoluteURL( $url )
@param $url - a String containing a URL
@discussion - indicates if the provided URL is an absolute URL
@result - returns true if the URL is an absolute URL;
   otherwise returns false
*/

function isAbsoluteURL( $url )
{
   return ( ( strpos( $url, 'http://' ) === 0 ) ||
            ( strpos( $url, 'https://' ) === 0 ) );
}

//========================================================================

/*!
@function getAbsoluteURL( $url )
@param $url - a String containing a URL
@discussion - if $url is an absolute URL $url is returned
   - if $url is a relative URL - an absolute URL based on the this server
     is returned
*/

function getAbsoluteURL( $url )
{
   if ( isAbsoluteURL( $url ) ) return $url;

   if ( isset( $_SERVER{ 'HTTPS' } ) && $_SERVER{ 'HTTPS' }  == 'on' )
   {
      $protocol = 'https';
      $portToIgnore = 443;
   }
   else
   {
      $protocol = 'http';
      $portToIgnore = 80;
   }

   return $protocol . '://' . getServer( $portToIgnore ) . $url;
}

//========================================================================

/*!
@function getServer()
@result - returns the current server including port number if necessary
*/

function getServer( $portToIgnore = 80 )
{
	if ( isset( $_SERVER{ 'SERVER_NAME' } ) )
	{
		if ( ! preg_match( '/:\d+/', $_SERVER{ 'SERVER_NAME' } ) )
		{
		   //the port is not already in the name, so add it

   		if ( $_SERVER{ 'SERVER_PORT' } == $portToIgnore )
   		{
   			return $_SERVER{ 'SERVER_NAME' };
   	   }
   	   else
   		{
   			return $_SERVER{ 'SERVER_NAME' } . ':' . $_SERVER{ 'SERVER_PORT' };
   	   }
		}

		return $_SERVER{ 'SERVER_NAME' };
	}
	else
	{
		unreachablePoint( '$_SERVER information not found' );
	}
}

//========================================================================

/*!
@function _tryStripPort80( $server )
@param $server - a server identifier including port number
@discussion - strips off trailing :80 if it is present
@result - returns a server name without the trailing port 80 specification
*/

function _tryStripPort80( $server )
{
   return preg_replace( '/\:80$/', '', $server );
}

//========================================================================

/*!
@function redirect( $url )
@param $url - a relative or absolute URL to which to redirect
@discussion - redirects the browser to the specified URL
   NOTE: Script execution will terminate when this function is
      called.
*/

function redirect( $url )
{
   header( 'Location:' . getAbsoluteURL( $url ) );
   niceExit();
}

//========================================================================

/*!
@function getBaseURLFromURL( $url )
@param $url - a URL
@discussion - provides the portion of the URL before the GET arguments
@result - returns a String
*/

function getBaseURLFromURL( $url )
{
   $pos = strpos( $url, '?' );

   if ( $pos !== false )
      return substr( $url, 0, $pos );
   else
      return $url;
}

//========================================================================


   function getAbsoluteBugPortURL( $tailOfUrl )
   {
      $partIncludingPhpDir = getContainingDirectory( $_SERVER['PHP_SELF'] );
      //return $partIncludingPhpDir;
      return getAbsoluteURL( getContainingDirectory($partIncludingPhpDir) . '/' . $tailOfUrl );
   }

//========================================================================

/*!
@function getParametersFromURL( $url )
@param $url - a URL
@discussion - provides the GET variables and their values from the URL
@result - returns a hash of Strings
   - the keys are variable names
*/

function getParametersFromURL( $url )
{
   $pos = strpos( $url, '?' );

   if ( $pos === false ) return new_hash();

   $queryString = substr( $url, $pos + 1 );

   $pairs = explode( '&', $queryString );

   $result = array();

   foreach ( $pairs as $curPair )
   {
      list( $name, $value ) = explode( '=', $curPair );

      $result{ urldecode( $name ) } = urldecode( $value );
   }

   return $result;
}

//========================================================================

/*!
@function extract_fill_object( $hash, $object, [ $key ] )
@param $hash - a hash of data
@param $object - an Object
@param $keyNames - optional set of key names
   If omitted all elements of $hash are extracted.
@discussion - Similar to PHP's built-in extract, but fills the provided
   object.  For each key-value pair in $hash, the member variable
   key will contain value.
*/

function extract_fill_object( $hash, &$object, $keyNames = UNSPECIFIED )
{
   if ( $keyNames === UNSPECIFIED )
   {
      foreach ( $hash as $key => $value )
      {
         $object->$key = $value;
      }
   }
   else
   {
      foreach ( array_intersect( $keyNames, array_keys( $hash ) )  as $key )
      {
         $object->$key = $hash{ $key };
      }
   }
}

//========================================================================

if ( ! function_exists( 'html_entity_decode' ) )
{
   //From: http://us4.php.net/manual/en/function.html-entity-decode.php
   function html_entity_decode( $given_html, $quote_style = ENT_QUOTES ) {
      $trans_table = get_html_translation_table( HTML_SPECIALCHARS, $quote_style );
      if( $trans_table["'"] != '&#039;' ) { # some versions of PHP match single quotes to &#39;
        $trans_table["'"] = '&#039;';
      }
      return ( strtr( $given_html, array_flip( $trans_table ) ) );
   }
}

//========================================================================
?>