<?php

/*
arrayLibrary
$Author: millz $
$Revision: 1.2 $
$Date: 2003/12/17 21:37:00 $
$Log: arrayLibrary.php,v $
Revision 1.2  2003/12/17 21:37:00  millz
- removing duplicate preg_split logic

Revision 1.1  2003/09/09 15:11:59  dougqh
Initial revision

Revision 1.16  2002/12/31 18:02:08  dougqh
- Untested fix to 323 - made second parameter of pushLeftReferences optional again.

Revision 1.15  2002/12/30 22:43:41  dougqh
- Debugging

Revision 1.14  2002/12/19 20:35:51  millz
- func name fix

Revision 1.13  2002/12/19 15:41:20  millz
- fixed and renamed the pushXrefs and pushXvals funcs - they are now right and left - not up and down

Revision 1.12  2002/12/13 15:50:00  millz
- added pushDownValues

Revision 1.11  2002/11/18 15:02:14  dougqh
Fixed function new_hash.

Revision 1.10  2002/10/21 19:27:21  dougqh
Added new functions that wrap calls to array_slice.

Revision 1.9  2002/10/09 20:32:48  dougqh
Added a new function objectHashDeepCopy

Revision 1.8  2002/09/25 15:18:14  dougqh
Quotes to ticks.

Revision 1.7  2002/09/17 00:49:45  dougqh
Added functions to aid in filtering an array of objects.

Revision 1.6  2002/08/29 22:14:55  dougqh
Added a quick sort, hash copy, and an arraySlice that handles references.

Revision 1.5  2002/08/26 19:57:49  dougqh
Added new function arrayMergeReferences.

Revision 1.4  2002/08/21 00:25:40  dougqh
Added new function arrayDifference used by Query.

Revision 1.3  2002/08/19 14:30:50  dougqh
Fixed bug in pushDownReferences.

Revision 1.2  2002/08/13 13:35:44  dougqh
Added new isObjectHash used by Serializer.

Revision 1.1  2002/07/01 20:21:39  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/

/*!
@library array
@abstract - this file contains functions for manipulating arrays
*/

//========================================================================

/*!
@function new_array( ... )
@params - a list of items to put in the new array - any length
@discussion - it is better to use this function than the PHP array()
   function, when handling arrays of references
   
   - using this function also supports stronger type checking
     in combination with isArray
   
   - this function is safe to use in place of array() when creating
     numerically indexed arrays
   
@result - returns a reference to a new array
*/

function &new_array()
{   
   //I would put design by contract in here, but the PHP
   //interpreter did not seem to like it the last time I tried.
   //It when into an infinite recursion.
   
   return func_get_args();
}

//========================================================================

/*!
@function new_hash( ... )
@params - a list of key, value pairs to put into the hash
   i.e. "keyName" => "valueOfKey"
@discussion - it is better to use this function than the PHP
   array() function, when creating hashes
   
   - using this function allows stronger type checking using
      isHash
      
   - this function is safe to use in place of array() when creating
     new hashes      
@result
*/

function &new_hash()
{  
   $arguments = func_get_args();   
   
   $numArguments = count( $arguments );  
   
   //require
   {
      assert( '( $numArguments % 2 ) == 0' );
   }
   //body
   {   
      $result = array();
      
      if ( $numArguments != 0 )
      {
         for ( $i = 0; $i <= ( $numArguments / 2 ); $i += 2 )
         {
            $result{ $arguments[ $i ] } = $arguments[ $i + 1 ];
         }
      }
   }
   //ensure
   {
      assert( 'isHash( $result )' );
   }

   return $result;
}

//========================================================================

/*!
@function isHash( $array )
@param $array - an array to test
@discussion - tests an array to see if it is a hash
   - it does not test the parameter to make sure it is an array
@result - returns true if the array is a hash
   otherwise returns false
   - always returns false for an empty array
*/

function isHash( $array )
{
   return is_array( $array );
}

//========================================================================

/*!
@function isArray( $array )
@param $array - an array to test
@discussion - tests an array to see if it is numerically indexed from 0
   - it does not test the parameter to make sure it is an array
@result - returns true if the array is a numerically indexed array
   otherwise returns false
   - always returns false for an empty array
*/

function isArray( $array )
{
   if ( is_array( $array ) )
   {
      return ( isset( $array[ 0 ] ) || ( count( $array ) == 0 ) );
   }   
   else
   {
      return false;
   }
}

//========================================================================

//Taken from user comment at http://us2.php.net/manual/en/function.array-merge.php
function array_merge_compat( $array1,$array2 )
{
   $return=array();
   foreach(func_get_args() as $arg){
       if(!is_array($arg)){
           $arg=array($arg);
       }
       foreach($arg as $key=>$val){
           if(!is_int($key)){
               $return[$key]=$val;
           }else{
               $return[]=$val;
           }
       }
   }
   return $return;
}

//========================================================================

/*!
@function arrayMerge( $array0, $array1 )
@param $array0 - a numerically array
@param $array1 - a numerically array
@discussion - merges $array0 and $array1 eliminating any duplicates
   - restores indices so they are consecutive
@result - returns a numerically indexed array
*/

function arrayMerge( $array0, $array1 )
{
   assert( 'isArray( $array0 )' );
   assert( 'isArray( $array1 )' );

   return array_values( array_unique( array_merge( $array0, $array1 ) ) );
}

//========================================================================

/*!
@function arrayTop( $array )
@discussion - returns the top of element of the stack without popping
@result - returns the top of element of the stack
*/

function arrayTop( $array )
{
   return $array[ count( $array ) - 1 ];
}

//========================================================================

   /*!
   @function pushRightReferences( &$array, $startingIndex, [ $endIndexPlusOne ] )
   @param $array - an array passed and modified by reference
   @param $startingIndex - the index to start the push up at;
      
      This element will be left with its previous value.
      
      Changing the value at this index is left to the caller.
      
   @param $endIndexPlusOne - the last index to process plus one or just the length
      - if not specified the array length will be used   
   @discussion - performs a push up on the array passed by reference
      - all assignments done are reference assignments
   */
   
   
   
   function pushRightReferences( &$array, $startingIndex = UNSPECIFIED, $endIndexPlusOne = UNSPECIFIED )
   {
      if ( $startingIndex === UNSPECIFIED ) $startingIndex = 0;
      if ( $endIndexPlusOne === UNSPECIFIED ) $endIndexPlusOne = count( $array );
      
      //require
      {
         assert( 'isInstanceOf( $array, "Any[]" )' );
         assert( '$startingIndex >= 0' );      
         assert( '$endIndexPlusOne >= $startingIndex' );
      }
      //body   
      {
         for ( $i = $endIndexPlusOne; $i > $startingIndex; /* $i-- - done below */ )
         {
            $array[ $i ] =& $array[ --$i ];
         }
      }
      //ensure
      {
         assert( 'isInstanceOf( $array, "Any[]" )' );
      }
   }


//========================================================================

   /*!
   @function pushLeftReferences( &$array, [ $startingIndex, [ $endIndexPlusOne ] ] )
   @param $array - an array passed and modified by reference
   @param $startingIndex - the index to start the push up at;
      the item at the starting index will be destroyed - defaults to 0
   @param $endIndexPlusOne - the last index to process plus one or just the length
      - if not specified the array length will be used
   @discussion - performs a push up on the array passed by reference
      - all assignments done are reference assignments
   */
   
   function pushLeftReferences( &$array, $startingIndex = UNSPECIFIED, $endIndexPlusOne = UNSPECIFIED )
   {
      if ( $startingIndex === UNSPECIFIED ) $startingIndex = 0;
      if ( $endIndexPlusOne === UNSPECIFIED ) $endIndexPlusOne = count( $array );
      
      //require
      {
         assert( 'isInstanceOf( $array, "Any[]" )' );
         assert( '$startingIndex >= 0' );
         assert( '$endIndexPlusOne >= $startingIndex' );
      }
      //body   
      {
         for ( $i = $startingIndex; $i < ($endIndexPlusOne - 1); /* $i++ - done below */ )
         {
            $array[ $i ] =& $array[ ++$i ];
         }      
      }
      //ensure
      {
         assert( 'isInstanceOf( $array, "Any[]" )' );
      }
   }

//========================================================================

/*!
@function pushRightValues( &$array, [ $startingIndex, [ $endIndexPlusOne ] ] )
@param $array - an array passed and modified by reference
@param $startingIndex - the index to start the push up at;
   the item at the starting index will be destroyed - defaults to 0
@param $endIndexPlusOne - the last index to process plus one or just the length
   - if not specified the array length will be used
@discussion - performs a push up on the array passed by reference
   - all assignments done are value assignments
*/

function pushRightValues( &$array, $startingIndex = UNSPECIFIED, $endIndexPlusOne = UNSPECIFIED )
{
   if ( $startingIndex === UNSPECIFIED ) $startingIndex = 0;
   if ( $endIndexPlusOne === UNSPECIFIED ) $endIndexPlusOne = count( $array );
   
   //require
   {
      assert( 'isInstanceOf( $array, "Any[]" )' );
      assert( '$startingIndex >= 0' );      
      assert( '$endIndexPlusOne >= $startingIndex' );
   }
   //body   
   {
      for ( $i = $endIndexPlusOne; $i > $startingIndex; /* $i-- - done below */ )
      {
         $array[ $i ] = $array[ --$i ];
      }
   }
   //ensure
   {
      assert( 'isInstanceOf( $array, "Any[]" )' );
   }
}

//========================================================================

/*!
@function pushLeftValues( &$array, [ $startingIndex, [ $endIndexPlusOne ] ] )
@param $array - an array passed and modified by reference
@param $startingIndex - the index to start the push up at;
   the item at the starting index will be destroyed - defaults to 0
@param $endIndexPlusOne - the last index to process plus one or just the length
   - if not specified the array length will be used
@discussion - performs a push up on the array passed by reference
   - all assignments done are value assignments
*/

function pushLeftValues( &$array, $startingIndex = UNSPECIFIED, $endIndexPlusOne = UNSPECIFIED )
{
   if ( $startingIndex === UNSPECIFIED ) $startingIndex = 0;
   if ( $endIndexPlusOne === UNSPECIFIED ) $endIndexPlusOne = count( $array );
   
   //require
   {
      assert( 'isInstanceOf( $array, "Any[]" )' );
      assert( '$startingIndex >= 0' );      
      assert( '$endIndexPlusOne >= $startingIndex' );
   }
   //body   
   {
      for ( $i = $startingIndex;  $i < ($endIndexPlusOne - 1); /* $i++ - done below */ )
      {
         $array[ $i ] = $array[ ++$i ];
         
      }
   }
   //ensure
   {
      assert( 'isInstanceOf( $array, "Any[]" )' );
   }
}

//========================================================================

/*!
@function swapIndicesByReferences( &$array, $index0, $index1 )
@param $array - an array that will be modified by reference
@param $index0 - one index to swap
@param $index1 - another index to swap
@discussion - swaps the item at array[index0] with the item at array[index1]
   using reference assignments
*/

function swapIndicesByReferences( &$array, $index0, $index1 )
{
   //print 'SwapIndicesByReferences: ' . $index0 . ' ' . $index1 . BR_NL;
   
   //require
   {
      assert( 'isInstanceOf( $array, "Any[]" )' );
      assert( '$index0 >= 0' );
      assert( '$index1 >= 0' );
   }
   //body
   {
      $temp =& $array[ $index0 ];
      $array[ $index0 ] =& $array[ $index1 ];
      $array[ $index1 ] =& $temp;
   }
   //require
   {
      assert( 'isInstanceOf( $array, "Any[]" )' );
   }
}

//========================================================================

/*!
@function swapIndicesByReferences( &$array, $index0, $index1 )
@param $array - an array that will be modified by reference
@param $index0 - one index to swap
@param $index1 - another index to swap
@discussion - swaps the item at array[index0] with the item at array[index1]
   using value assignments
*/

function swapIndicesByValues( &$array, $index0, $index1 )
{
   //require
   {
      assert( 'isInstanceOf( $array, "Any[]" )' );
      assert( '$index0 >= 0' );
      assert( '$index1 >= 0' );
   }
   //body
   {
      $temp = $array[ $index0 ];
      $array[ $index0 ] = $array[ $index1 ];
      $array[ $index1 ] = $temp;
   }
   //require
   {
      assert( 'isInstanceOf( $array, "Any[]" )' );
   }
}

//========================================================================

/*!
@function isObjectHash( $objectsHash )
@param $objectsHash - any
@discussion - checks to see if the provided variable a mono or poly-dimensional
   hash of Objects
@result - returns true if the pvovided value is a hash of Objects;
   otherwise returns false
*/

function isObjectHash( &$objectsHash )
{
   if ( ! is_array( $objectsHash ) ) return false;
   if ( empty( $objectsHash ) ) return false;
   
   list( $firstKey ) = array_keys( $objectsHash );
   
   if ( is_object( $objectsHash{ $firstKey } ) )
   {
      return true;
   }
   else 
   {
      return isObjectHash( $objectsHash{ $firstKey } );
   }   
}


//========================================================================

    function getNumericIDArrayFromString( $string )
    {
        return preg_split ("/\D+/", $string, -1, PREG_SPLIT_NO_EMPTY);
    }

//========================================================================

/*!
@function arrayDifference( $array0, $array1 )
@param $array0 - a numerically indexed Array
@param $array1 - a numerically indexed Array
@discussion - provides all the elements in $array0 that are not in $array1
   - similar to PHP's array_diff, but does NOT preserve keys
   
   It is assumed that the order of the elements in the array is NOT important.
@result - returns a numerically indexed array
*/

function arrayDifference( $array0, $array1 )
{
   $tempArray = array_diff( $array0, $array1 );
   
   $resultArray = array();
   
   $numElements = 0;
   foreach ( $tempArray as $curValue )
   {
      $resultArray[ $numElements++ ] = $curValue;
   }
   
   return $resultArray;   
}

//========================================================================

/*!
@function arrayMergeReferences( $array0, $array1 )
@param $array0 - a numerically-indexed array
@param $array1 - a numerically-indexed array
@discussion - merges two arrays of Objects preserving references
@result - returns a numerically indexed array
*/

function &arrayMergeReferences( &$array0, &$array1 )
{
   $results = new_array();
   $numResults = 0;
   
   $countArray0 = count( $array0 );
   for ( $i = 0; $i < $countArray0; /* i++ - in loop */ )
   {
      $results[ $numResults++ ] = $array0[ $i++ ];
   }
   
   $countArray1 = count( $array1 );
   for ( $i = 0; $i < $countArray1; /* i++ - in loop */ )
   {
      $results[ $numResults++ ] = $array1[ $i++ ];
   }
   
   return $results;
}

//========================================================================

/*!
@function quickSortObjects( $objects, $methodName, [ $sortAscending ] )
@param $objects - an array of Objects
@param $methodName - the name of a get accessor method
@param $sortAscending - optional boolean indicating if the sort is
   ascending - defaults to true
@discussion - sorts an array of objects by comparing the
   value generated by methodName
*/

function quickSortObjects( &$objects, $methodName, $sortAscending = true )
{
   _quickSortObjects( $objects, $methodName, $sortAscending, 0, count( $objects ) - 1 );
}

//========================================================================

/*!
@function _quickSortObjects( $objects, $methodName, $sortAscending, $left, $right )
@param $objects - an array of Objects
@param $methodName - the name of a method to compare members
@param $sortAscending - boolean indicating if the sort is ascending
@param $left - an index
@param $right - an index
@discussion - quick sort on an array of objects
*/

function _quickSortObjects( &$objects, $methodName, $sortAscending, $left, $right )
{ 
   if ( $left >= $right ) return; 
   
   swapIndicesByReferences( $objects, $left, intval( ( $left + $right ) / 2 ) ); 
   
   $last = $left; 
   
   if ( $sortAscending )
   {
      for ( $i = $left + 1; $i <= $right; $i++ )
      { 
         if ( $objects[ $i ]->$methodName() < $objects[ $left ]->$methodName() )
         { 
            swapIndicesByReferences( $objects, ++$last, $i ); 
         }
      }
   }
   else
   {
      for ( $i = $left + 1; $i <= $right; $i++ )
      { 
         if ( $objects[ $i ]->$methodName() > $objects[ $left ]->$methodName() )
         { 
            swapIndicesByReferences( $objects, ++$last, $i ); 
         }
      }
   }
   
   swapIndicesByReferences( $objects, $left, $last ); 

   _quickSortObjects( $objects, $methodName, $sortAscending, $left, $last - 1 ); 
   _quickSortObjects( $objects, $methodName, $sortAscending, $last + 1, $right );
}

//========================================================================

/*!
@function objectHashCopy( $objectHash )
@param $objectHash - a hash of Objects
@discussion - copies a hash of Objects - preserving references, but
   returning a new array
@result - returns a new Object hash
*/

function &objectHashCopy( &$objectHash )
{
   $result = new_array();
   foreach ( array_keys( $objectHash ) as $curKey )
   {
      $result{ $curKey } =& $objectHash{ $curKey };
   }
   
   return $result;
}

//========================================================================

/*!
@function objectHashDeepCopy( $objectHash )
@param $objectHash - a hash of Objects
@discussion - does a deep copy of the hash - i.e. gets a copy
   of every Object in the hash
@result - returns a new Object hash
*/

function &objectHashDeepCopy( &$objectHash )
{
   $result = new_array();
   foreach ( array_keys( $objectHash ) as $curKey )
   {
      $result{ $curKey } =& $objectHash{ $curKey }->getCopy();
   }
   
   return $result;
}

//========================================================================

/*!
@function arraySliceReferences( $array, $offset, $length )
@param $array - an array of references
@param $offset - an offset at which to start the slice
@param $length - the length of the slice
@discussion - identical to array_slice, but maintains references
@result - returns an array
*/

function &arraySliceReferences( &$array, $offset, $length )
{
   $results = new_array();
   $numResults = 0;
   
   $endLength = min( count( $array ) - $offset, $length );
   
   for ( $i = $offset; $numResults < $endLength; /* inside loop */ )
   {
      $results[ $numResults++ ] = $array[ $i++ ];
   }
   
   return $results;
}

//========================================================================

/*!
@function arraySliceRange( $array, $integerRange )
@param $array - an array to slice
@param $integerRange - an IntegerRange
@discussion - similar to array_slice but uses an IntegerRange
@result - returns an array
*/

function arraySliceRange( &$array, &$integerRange )
{
   return arraySlice(
      $array,
      $integerRange->getLowerBound(),
      $integerRange->getLength() );
}

//========================================================================

/*!
@function arraySlice( &$array, $startIndex, [ $length ] )
@param $array - an array to slice
@param $startIndex - an index at which to start
@param $length - number of characters to extract - defaults to all remaining characters
@discussion - similar to array_slice but slightly more robust
@result - returns an array
*/

function arraySlice( &$array, $startIndex, $length = INFINITY )
{
   if ( ( count( $array ) - $startIndex ) > $length )
   {
      return array_slice( $array, $startIndex, $length );
   }
   else
   {
      return array_slice( $array, $startIndex );
   }
}

//========================================================================

/*!
@function objectArrayFilter( $objects, $method, $value )
@param $objects - an array of Objects
@param $method - a method to call on the Objects
@param $value - a value to compare against
@discussion - filters an array according to the specified parameters
   - keeping only the elements where $objects[ $i ]->$method() == $value.
@result - returns an array of Objects
*/

function &objectArrayFilter( &$objects, $method, $value )
{
   $results = new_array();
   $numResults = 0;
   
   $numObjects = count( $objects );   
   for ( $i = 0; $i < $numObjects; ++$i )
   {
      if ( $objects[ $i ]->$method() == $value )
      {
         $results[ $numResults++ ] =& $objects[ $i ];
      }      
   }
   
   return $results;
}

//========================================================================

/*!
@function objectArraySplitFilter( $objects, $method, $value )
@param $objects - an array of Objects
@param $method - a method to call on the Objects
@param $value - a value to compare against
@discussion - filters an array according to the specified parameters
   - produces a pair of two object arrays 
   - the first consists elements where $objects[ $i ]->$method() == $value
   - the second consists of elements not satisfying the first condition
@result - a pair of object arrays
*/

function &objectArraySplitFilter( &$objects, $method, $value )
{
   $results = new_array();
   $results[ 0 ] = new_array();
   $numResults0 = 0;
   $results[ 1 ] = new_array();
   $numResults1 = 0;
   
   $results = new_array();
   $numResults = 0;
   
   $numObjects = count( $objects );   
   for ( $i = 0; $i < $numObjects; ++$i )
   {
      if ( $objects[ $i ]->$method() == $value )
      {
         $results[ 0 ][ $numResults0++ ] =& $objects[ $i ];
      }
      else
      {
         $results[ 1 ][ $numResults1++ ] =& $objects[ $i ];
      }      
   }
   
   return $results;
}

//========================================================================

?>