<?php

/*
debugLibrary
$Author: dougqh $
$Revision: 1.5 $
$Date: 2004/11/17 19:51:55 $
$Log: debugLibrary.php,v $
Revision 1.5  2004/11/17 19:51:55  dougqh
- Compatiblity fixes for PHP5 and windows

Revision 1.4  2004/04/01 22:05:50  dougqh
- Adding _FILES tracking

Revision 1.3  2003/12/18 20:34:05  dougqh
- Updated to work in PHP 4.2

Revision 1.2  2003/11/03 20:07:14  dougqh
- Added PRINT_EMAIL_RECIPIENTS_AND_EXIT for convenience in checking email rule changes.

Revision 1.1  2003/09/09 15:11:59  dougqh
Initial revision

Revision 1.1  2003/07/14 15:08:01  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//frequently used strings
//========================================================================

define( 'DEBUGNAME_ERROR', 'Error' );
define( 'TEST_FAILED', 'Test Failed' );
define( 'TEST_INFORMATION', 'Test Information' );

//file settings
//========================================================================

define( 'DEBUGGING_DIRECTORY', 'debugging' );

define( 'DEBUG_FILE', 'debug' );
define( 'LAST_DEBUG_FILE', 'lastDebug' );

define( 'QUERY_FILE', 'queries' );
define( 'TIMING_FILE', 'timing' );

//note the lead 0 makes it octal
define( 'DEBUG_FILE_PERMISSIONS', 0644 );

define( 'PHP_5_ERROR_LEVEL', 2048 );

//custom debugging cookie settings
//========================================================================

define( 'DEBUG_COOKIE', 'bugport' . VERSION_TYPE );
define( 'DEBUG_COOKIE_LIFE', intval( 365.25 * 24 * 60 * 60 ) /* seconds - one year*/ );
if ( ! defined( 'ALLOW_CUSTOM_DEBUGGING' ) )
{
   define( 'ALLOW_CUSTOM_DEBUGGING', false );
}

//========================================================================

define( 'MAX_HISTORY_SIZE', 20 );

//========================================================================

set_error_handler( '_errorHandler' );

loadDebugSettings();

//========================================================================

/*!
@function getDefaultDebugSettings()
@discussion - provides a hash of debug constant names and their default values
@result - returns a hash of strings
   keys are debug constant names
   values are default values for the constants
*/

function getDefaultDebugSettings()
{
   return array(
      'ENABLE_ASSERTIONS' => false,
      'ENABLE_DEBUGGING' => false,
      'ENABLE_INVARIANT' => false,
      'DEPRECATED_IS_ERROR' => false,
      'INCOMPLETE_IS_ERROR' => false,
      'PRINT_START_COMMAND_LINE_AND_EXIT' => false,
      'ENABLE_STRINGENT_CHECKS' => false,
      'PRINT_SERIALIZATION_TIMINGS' => false,
      'FORCE_DESIGNER' => false,
      'FORCE_QA' => false,
      'FORCE_DEVELOPER' => false,
      'FORCE_QA_LEAD' => false,
      'ALWAYS_ENABLE_ALL_FORMS' => false,
      'RUN_EXPERIMENTAL' => false,
      'PRINT_EMAIL_RECIPIENTS_AND_EXIT' => false,
      'PAUSE_ON_REDIRECT' => false,
      'PRINT_INSERT_UPDATE_SQL' => false );
}

//========================================================================

/*!
@function getAvailableDebugOptions()
@discussion - provides an array of debug constants that may be set
@result - returns an array of Strings
*/

function getAvailableDebugOptions()
{
   return array_keys( getDefaultDebugSettings() );
}

//========================================================================

/*!
@function getDebugSettings()
@discussion - provides a hash of the current debug settings
@result - returns a hash
   keys - are debugging constants
   values - are their values
*/

function getDebugSettings()
{
   //print_r( $_COOKIE );

   if ( ALLOW_CUSTOM_DEBUGGING && isset( $_COOKIE{ DEBUG_COOKIE } ) )
   {
      return _getCompleteDebugSettings( unserialize( $_COOKIE{ DEBUG_COOKIE } ) );
   }
   else
   {
      //print 'Using defaults <br />';
      return getDefaultDebugSettings();
   }
}

//========================================================================

/*!
@function _getCompleteDebugSettings( $settings )
@param $settings - a set of debugging settings
@discussion - adds default entries for any missing variables to the settings
@result - returns a set of debugging settings
*/

function _getCompleteDebugSettings( $settings )
{
   //use the default value for anything that's not set
   foreach ( getDefaultDebugSettings() as $option => $default )
   {
      if ( ! isset( $settings{ $option } ) )
      {
         $settings{ $option } = $default;
      }
   }

   return $settings;
}

//========================================================================

/*!
@function loadDebugSettings()
@discussion - loads the debug settings
*/

function loadDebugSettings()
{
   foreach( getDebugSettings() as $option => $value )
   {
      define( $option, $value );
      //print 'define( ' . $option . ', ' . $value . ' ) <br />';
   }
}

//========================================================================

/*!
@function setDebugOptions()
@param $settings - a set of debug settings
@discussion - sets the debugging settings
*/

function setDebugOptions( $settings )
{
   if ( ALLOW_CUSTOM_DEBUGGING )
   {
      $settings = _getCompleteDebugSettings( $settings );

      $serializedSettings = serialize( $settings );

      //kill the old cookie
      setcookie( DEBUG_COOKIE );

      //create a new cookie
      $success = setcookie(
         DEBUG_COOKIE,
         $serializedSettings,
         time() + DEBUG_COOKIE_LIFE,
         '/' );

      if ( ! $success )
      {
         print 'Saving the cookie failed <br />';
      }

      $_COOKIE{ DEBUG_COOKIE } = $serializedSettings;
   }
}

//========================================================================

/*!
@function _startDebugging()
@discussion - call to start the debugging process
*/

function _startDebugging()
{
  static $debuggingStarted;

   if ( session_id() && ! isset( $debuggingStarted ) )
   {
      $debuggingStarted = true;

      if ( ENABLE_DEBUGGING )
      {
         $debuggingFile = _getDebuggingFile( DEBUG_FILE );

         if ( file_exists( $debuggingFile ) )
         {
            $lastDebuggingFile = _getDebuggingFile( LAST_DEBUG_FILE );

            system( 'mv ' . $debuggingFile . ' ' . $lastDebuggingFile );
         }

         //delete the file
         unlink( $debuggingFile );

         chmod( $debuggingFile, DEBUG_FILE_PERMISSIONS );
      }

      if ( ENABLE_QUERY_LOGGING )
      {
         $queryFile = _getDebuggingFile( QUERY_FILE );

         if ( file_exists( $queryFile ) )
         {
            unlink( $queryFile );
         }
      }

      if ( ENABLE_PAGE_TIMING_LOGGING )
      {
         $timingFile = _getDebuggingFile( TIMING_FILE );

         if ( file_exists( $timingFile ) )
         {
            unlink( $timingFile );
         }
      }
   }
}

//========================================================================

/*!
@function debugQuery( $debugName, $queryStatement )
@param $debugName - a group name for the debugging being performed
   -usually a class or function name
@param $queryStatement - Any
@discussion - similar to debug, but also logs the data into a queries file
   if LOGGING_ENABLED is true
*/

function debugQuery( $debugName, $queryStatement )
{
   importClass( 'Common.Utilities.DatabaseConnection' );

   $output =
      $debugName . NL .
      'Query: ' . NL .
      DatabaseConnection::formatQuery( $queryStatement ) . NL .
      '---------------------------------------------------------------' . NL;

   if ( ENABLE_QUERY_PRINTING )
   {
      print getStringDepiction( $output );
   }

   if ( ENABLE_QUERY_LOGGING )
   {
      appendToFile( _getDebuggingFile( QUERY_FILE ), $output );
   }

   debug( $debugName, 'Query', $value );
}

//========================================================================

/*!
@function debug( $debugName, $description, $value )
@param $debugName - a group name for the debugging being performed
   -usually a class or function name
@param $description - a description of the value being displayed
@param $value - Any
@discussion - logs information into a file
*/

function debug( $debugName, $description, &$value )
{
   if ( ! ENABLE_DEBUGGING ) return;

   _startDebugging();

   if ( session_id() )
   {
      $debuggingFile = _getDebuggingFile( DEBUG_FILE );

      $fh = fopen( $debuggingFile, 'a' );

      if ( is_object( $value ) )
      {
         $string = $value->getDebugging();
      }
      else
      {
         $string = getStringDepiction( $value );
      }

      if ( $debugName == DEBUGNAME_ERROR )
      {
         $errorString = '-------------------------------------------------------------' . NL;
         $errorString .= 'ERROR: ' . NL;
         $errorString .= $description . NL;
         $errorString .= getSimpleStringDepiction( $string ) . NL;
         $errorString .= '-------------------------------------------------------------' . NL;
         $errorString .= NL;

         fwrite( $fh, $errorString );
      }
      else
      {
         $debuggingString = 'DebuggingGroup: ' . $debugName . NL;
         $debuggingString .= 'Description: ' . $description . NL;
         $debuggingString .= 'Value: ' . NL;
         $debuggingString .= getSimpleStringDepiction( $string ) . NL;
         $debuggingString .= NL;

         fwrite( $fh, $debuggingString );
      }

      fclose( $fh );
   }
}

//========================================================================

/*!
@function logError( $debugName, $error )
@param $debugName - a group name for the debugging being performed
   - usually a class or function name
@param $error - an error string
@discussion - log's an error that occurred
*/

function logError( $debugName, $error )
{
   $value = NL . $error;

   debug( DEBUGNAME_ERROR, 'Error with unit ' . $debugName, $value );
}

//========================================================================

/*
@function _errorHandler()
@param $errorNumber - the error code of the error that occurred
@param $errorMessage - a string describing the error
@param $errorFile - the name of the file in which in the error occurred
@param $errorLine - the line on which the error occurred
@discussion - handles PHP runtime errors by building a nice
   HTML representation of the error and passing it off to
   the debug function, so that the debug function can log the
   error
   - the debug function is written, so that it always instantaneously
   prints out errors
*/

function _errorHandler( $errorNumber, $errorMessage, $errorFile, $errorLine, $errorContext )
{
   if ( $errorNumber == PHP_5_ERROR_LEVEL ) return;
   if ( error_reporting() == 0 ) return;
   
   static $suppressHandling;
   if ( isset( $suppressHandling ) && $suppressHandling ) return;
   
   $suppressHandling = true;
   
   _actualErrorHandler( $errorNumber, $errorMessage, $errorFile, $errorLine, $errorContext );
   
   $suppresHandling = false;
}

function _actualErrorHandler( $errorNumber, $errorMessage, $errorFile, $errorLine, $errorContext )
{  
   //TODO: Clean-up this function

   $isGenePort3ClassFile = ( strpos( $errorFile, 'geneport3/' ) !== false );
   $isScript = ( strpos( $errorFile, '/php/' ) !== false );

   $isADO = ( strpos( $errorFile, '.inc.php' ) !== false );

   $isGenePort4 = ( ! $isGenePort3ClassFile ) && ( ! $isScript ) && ( ! $isADO );

   $errorString = BR_NL;
   $errorString .= 'Error Number: ' . $errorNumber . BR_NL;
   $errorString .= $errorMessage . BR_NL;
   $errorString .= 'File: ' . $errorFile . BR_NL;
   $errorString .= 'Line: ' . $errorLine . BR_NL;
   $errorString .= 'isGP4: ' . getStringDepiction( $isGenePort4 );

   if ( $errorNumber == E_WARNING )
      $errorName = 'ERROR - WARNING';
   elseif ( $errorNumber == E_NOTICE )
      $errorName = 'ERROR - NOTICE';

   debug( DEBUGNAME_ERROR, $errorName, $errorString );

   if ( $isGenePort4 )
   {
      if ( ( ! IS_WINDOWS ) && class_exists( 'process' ) )
      {
         list( $retval, $lines ) = process::execute( 'ls -al ' . $errorFile );
      }
      else
      {
         $lines[ 0 ] = 'Unable to retrieve file info';
      }

      if ( IS_COMMAND_LINE )
      {
         print NL;
         print $errorString . NL;
         print NL;

         if ( isset( $lines[ 0 ] ) )
         {
            print 'File Info: ' . $lines[ 0 ] . NL;
         }

         print NL;

         if ( strpos( $errorFile, 'importLibrary.php' ) !== false )
         {
            $location = getCallLocationOf( 'importClass' );

            if ( $location !== false )
            {
               print 'Probable importing error' . NL;
               print 'importClass called at ...' . NL;
               print $location . BR_NL;
            }
         }

         print 'Backtrace: ' . NL;
         print getBacktrace( 1 ) . NL;
      }
      else
      {
         print BR_NL;
         print '<font color="red" size="+1">' . NL;
         print '<b>' . $errorString . '</b>' . NL;
         print '</font>' . BR_NL;

         if ( isset( $lines[ 0 ] ) )
         {
            print '<font size="+1">File Info: ' . $lines[ 0 ] . '</font>' . NL;
         }

         if ( strpos( $errorFile, 'importLibrary.php' ) !== false )
         {
            $location = getCallLocationOf( 'importClass' );

            print BR_NL;
            print 'Probable importing error' . BR_NL;
            print 'importClass called at ...' . BR_NL;
            print $location . BR_NL;
         }

         print 'Backtrace: ' . BR_NL;
         print getStringDepiction( getBacktrace( 1 ) ) . BR_NL;
         print 'URL History: ' . BR_NL;
         print getStringDepiction( getHistory() ) . BR_NL;
      }
   }
}

//========================================================================

/*!
@function todo( $functionName, $task )
@param $functionName - the name of the function containing the TODO
@param $task - a description of the task
@discussion - logs TODOs that have been triggered to a file if
   ENABLE_TODO_LOGGING is true
*/

function todo( $functionName, $task )
{
   incomplete( 'todo' );
}

//========================================================================

/*!
@function incomplete( $functionName, [ $task ] )
@param $functionName - the incomplete function that was called
@param $task - a description of what is incomplete
@discussion - called by an incomplete function for tracking purposes
*/

function incomplete( $functionName, $task = UNSPECIFIED )
{
   if ( INCOMPLETE_IS_ERROR )
   {
      if ( $task === UNSPECIFIED )
      {
         unreachablePoint( 'Call to incomplete function: ' . $functionName );
      }
      else
      {
         unreachablePoint(
            'Call to incomplete function: ' . $functionName . NL .
            'Task: ' . NL .
            $task );
      }
   }
}

//========================================================================

/*!
@function deprecated( $functionName, [ $replacementName ] )
@param $functionName - the deprecated function that was called
@param $replacementName - name of the prefer function to use - defaults to nothing
@discussion - called by deprecated function passing its name
   - if the constant DEPRECATED_IS_ERROR is defined to be true
     then this function will call unreachablePoint
*/

function deprecated( $functionName, $replacementName = UNSPECIFIED )
{
   if ( DEPRECATED_IS_ERROR )
   {
      if ( $replacementName === UNSPECIFIED )
      {
         unreachablePoint(
            'Call to deprecated function: ' . $functionName .
            'Replacement function unknown' );
      }
      else
      {
         unreachablePoint(
            'Call to deprecated function: ' . $functionName .
            'Replacement function: ' . $replacementName );
      }
   }
}

//========================================================================

/*!
@function _getDebuggingFile( $baseName )
@param $baseName - a String
@discussion - provides a complete path to a unique file name
   that incorporates the specified $baseName
@result - returns a String
*/

function _getDebuggingFile( $baseName )
{
   return truePath( DEBUGGING_DIRECTORY ) . session_id() . '_' . $baseName . '.txt';
}

//========================================================================

/*!
@function getBacktrace( [ $startIndex ] )
@param $startIndex - the index in the trace at which to start relative to
   the caller - defaults to 0
@discussion - provides a nicely formatted backtrace
@result - returns a String
*/

function getBacktrace( $startIndex = 0 )
{
   if ( ! function_exists( 'debug_backtrace' ) )
   {
      return 'No backtrace available before PHP 4.3' . NL;
   }

   $trace = debug_backtrace();

   $numCalls = count( $trace );

   //skip element 0, since that's the call to getBacktrace
   for ( $i = $startIndex + 1, $output = ''; $i < $numCalls; ++$i )
   {
      $curCall =& $trace[ $i ];

      $output .= 'Call No: ' . ( $numCalls - 1 - $i ) . BR_NL;
      $output .= 'Called from File : ' . cleanUpSourceFileName( $curCall{ 'file' } ) . ' : Line : ' . $curCall{ 'line' } . NL;
      if ( isset( $curCall{ 'class' } ) )
      {
         $output .= 'Called ' . $curCall{ 'class' } . $curCall{ 'type' } . $curCall{ 'function' } . NL;
      }
      else
      {
         $output .= 'Called ' . $curCall{ 'function' } . NL;
      }

      if ( empty( $curCall{ 'args' } ) )
      {
         $output .= 'Arguments: NONE' . NL . NL;
      }
      else
      {
         $output .= 'Arguments: ' . NL;
         $numArgs = count( $curCall{ 'args' } );

         for ( $j = 0; $j < $numArgs; ++$j )
         {
            $curArg = $curCall{ 'args' }[ $j ];

            if ( is_null( $curArg ) )
            {
               $value = 'NULL';
            }
            elseif ( is_bool( $curArg ) )
            {
               $value = $curArg ? 'true' : 'false';
            }
            elseif ( is_array( $curArg ) )
            {
               $keys = array_keys( $curArg );
               if ( is_object( $curArg{ $keys[ 0 ] } ) )
               {
                  $value .= 'Object[]';
               }
               else
               {
                  $value = getSimpleStringDepiction( $curArg );
               }
            }
            elseif ( is_object( $curArg ) )
            {
               $value = $curArg->object_id;
            }
            elseif ( is_string( $curArg ) )
            {
               $value = '"' . $curArg . '"';
            }
            else
            {
               $value = $curArg;
            }

            $output .= '[' . $j . '] = ' . $value . NL;
         }

         $output .= NL;
      }
   }

   return $output;
}

//========================================================================

/*!
@function getLocation()
@discussion - provides the file name, line number, function name, and
   class name (if applicable) from which this function was called
@result - returns a String
*/

function getLocation()
{
   if ( function_exists( 'debug_backtrace' ) )
   {
      return _getPrettyLocation( debug_backtrace(), 0 );
   }
   else
   {
      return 'Call location unavaiable before PHP 4.3';
   }
}

//========================================================================

/*!
@function getCallLocationOf( $function )
@param $function - a String - a function name
@discussion - provides the information about where function was called
   by using debug_backtrace

   If no call to function is found in the backtrace, then false is returned.
@result - returns a String on success - otherwise returns false
*/

/*!
@function getCallLocationOf( $class, $method )
@param $class - a String - a class name
@param $method - a String - a method name
@discussion - provides the information about where the specified was
   called by using debug_backtrace

   If no call to function is found in the backtrace, then false is returned.
@result - returns a String on success - otherwise returns false
*/

function getCallLocationOf( $param0, $param1 = NULL )
{
   if ( ! function_exists( 'debug_backtrace' ) )
   {
      return 'Call location cannot be determined before PHP 4.3';
   }

   $trace = debug_backtrace();

   //print_r( $trace );

   $numCalls = count( $trace );

   if ( $param1 !== NULL )
   {
      $class = strtolower( $param0 );
      $method = strtolower( $param1 );

      for ( $i = 0; $i < $numCalls; ++$i )
      {
         $call = $trace[ $i ];

         if (
            isset( $call{ 'class' } ) &&
            $call{ 'class' } == $class &&
            $call{ 'function' } == $method )
         {
            $foundIndex = $i;
            break;
         }
      }
   }
   else
   {
      $function = strtolower( $param0 );

      for ( $i = 0; $i < $numCalls; ++$i )
      {
         $call = $trace[ $i ];

         if (
            ! isset( $call{ 'class' } ) &&
            $call{ 'function' } == $function )
         {
            $foundIndex = $i;
            break;
         }
      }
   }

   print 'FoundIndex: ' . $foundIndex . BR_NL;

   if ( isset( $foundIndex ) )
   {
      return _getPrettyLocation( $trace, $foundIndex );
   }
   else
   {
      return false;
   }
}

//========================================================================

/*!
@function cleanUpSourceFileName( $fileName )
@param $fileName - a String - source file name
@discussion - beautifies the file name
@result - returns a String
*/

function cleanUpSourceFileName( $fileName )
{
   return stripLeadSlash( str_replace( realpath( CLASS_DIR ), '', $fileName ) );
}

//========================================================================

/*!
@function _getPrettyLocation( $trace, $callIndex )
@param $trace - a backtrace
@param $callIndex - an unsigned integer - index of the relevant call
@discussion - nicely formats the information about the function call
@result - returns a String
*/

function _getPrettyLocation( &$trace, $callIndex )
{
   $curCall = $trace[ $callIndex ];
   $file = cleanUpSourceFileName( $curCall{ 'file' } );
   $line = $curCall{ 'line' };

   if ( isset( $trace[ $callIndex + 1 ] ) )
   {
      $prevCall = $trace[ $callIndex + 1 ];
      if ( isset( $prevCall{ 'class' } ) )
      {
         $function = $prevCall{ 'class' } . $prevCall{ 'type' } . $prevCall{ 'function' };
      }
      else
      {
         $function = $prevCall{ 'function' };
      }
   }
   else
   {
      $function = 'Global';
   }

   if ( $function == 'require_once' )
   {
      //actually called in global space of the file being loaded by require_once
      $function = 'Global space in file: ' . cleanUpSourceFileName( $prevCall{ 'args' }[ 0 ] );
   }

   return
      'File : ' . $file . ' : Line : ' . $line . BR_NL .
      'Function : ' . $function;

}

//========================================================================

/*!
@function logRequest()
@discussion - logs the page request in the session for debugging purposes
*/

function logRequest()
{
   if ( ! IS_COMMAND_LINE )
   {
      if ( ! isset( $_SESSION{ 'history' } ) )
      {
         $_SESSION{ 'history' } = array();
      }

      $entry = array(
         'script' => $_SERVER{ 'PHP_SELF' },
         'get' => $_GET,
         'post' => $_POST,
         'files' => $_FILES );

      array_unshift(
         $_SESSION{ 'history' },
         $entry );

      //Don't let it get too big, so unset at the MAX_HISTORY position
      //Just to be safe unset the next one, too.
      unset( $_SESSION{ 'history' }[ MAX_HISTORY_SIZE ] );
      unset( $_SESSION{ 'history' }[ MAX_HISTORY_SIZE + 1 ] );
   }
}

//========================================================================

/*!
@function getHistory()
@discussion - provides a history of the URL accesses during this session
@result - returns a String
*/

function getHistory()
{
   if ( isset( $_SESSION{ 'history' } ) )
   {
      return getSimpleStringDepiction( $_SESSION{ 'history' } );
   }
   else
   {
      return 'History not found' . BR_NL;
   }
}

//========================================================================

?>