<?php

/*
executionLibrary
$Author: dougqh $
$Revision: 1.1 $
$Date: 2003/09/09 15:11:59 $
$Log: executionLibrary.php,v $
Revision 1.1  2003/09/09 15:11:59  dougqh
Initial revision

Revision 1.2  2003/07/22 17:33:14  dougqh
- Setting session.gc_maxlifetime so form validation keys taking longer to expire.

Revision 1.1  2003/07/14 14:24:19  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

/*!
@library execution
@discussion - this file contains code for basic processes including
   scripts that should be run at the beginning of a PHP script
   and scripts that should run at the end of a PHP script
*/

//========================================================================

//These could be done as an enum, but for now this will suffice
define( 'LIGHT_EXECUTION', 'LIGHT_EXECUTION' );
define( 'STANDARD_EXECUTION', 'STANDARD_EXECUTION' );

//========================================================================

if ( ! defined( 'IS_COMMAND_LINE' ) ) define( 'IS_COMMAND_LINE', false );

//========================================================================

importLibrary( 'process' );
importLibrary( 'webServer' );

//========================================================================

//register_shutdown_function( '_shutdownHandler' );

//========================================================================

/*!
@function runStartUpProcesses()
@discussion - this function performs all actions that should be performed
   by standard scripts at startup.

   Standard scripts are any scripts that produce a full web page to
   be presented to the user or require Operation execution.
*/

function runStartUpProcesses()
{
   _runStartUpProcesses( STANDARD_EXECUTION );
}

//========================================================================

/*!
@function runEndProcesses()
@discussion - this function performs all actions that standard scripts
   should execute on shutdown

   Standard scripts are any scripts that produce a full web page to
   be presented to the user or require Operation execution.
*/

function runEndProcesses()
{
   _runEndProcesses( STANDARD_EXECUTION );
}

//========================================================================

/*!
@function runLightStartUpProcesses()
@discussion - this function performs all actions that lesser scripts
   should run at startup
*/

function runLightStartUpProcesses()
{
   _runStartUpProcesses( LIGHT_EXECUTION );
}

//========================================================================

/*!
@function runLightEndProcesses()
@discussion - this function performs all actions that lesser scripts
   should execute on shutdown
*/

function runLightEndProcesses()
{
   _runEndProcesses( LIGHT_EXECUTION );
}

//========================================================================

/*!
@function niceExit()
@discussion - exits the script in a "nice" fashion
*/

function niceExit()
{
   runLightEndProcesses();

   exit;
}

//========================================================================

/*!
@function _runStartUpProcesses( $executionType )
@param $executionType - one of the execution type constants defined
   above LIGHT_EXECUTION or STANDARD_EXECUTION
@discussion - runs the necessary start up processes based on the
   execution type specified

   NOTE: This function should NOT be called.  Call runStartUpProcesses
      or runLightStartUpProcesses instead.
*/

function _runStartUpProcesses( $executionType )
{
   //$startTime = getmicrotime();

   //seed the random number with the time
   list( $usec, $sec ) = explode( ' ', microtime() );
   $seed = floatval( $sec + ( floatval( $usec * 100000 ) ) );
   srand( $seed );

   if ( ! IS_COMMAND_LINE )
   {
      //not sure its loaded yet
      importLibrary( 'time' );
      gpTime::setTimeOut( GENEPORT4_TIMEOUT );
   }

   //just in case it has not already been done
   _importCommon();

   if ( ( ! defined( 'SID' ) ) && ( ! IS_COMMAND_LINE ) )
   {
      ini_set( 'session.gc_maxlifetime', 36000 );
      session_start();
   }

   logRequest();

   //set the assertion mode
   if ( ENABLE_ASSERTIONS )
   {
      turnOnAssertions();
   }
   else
   {
      turnOffAssertions();
   }

   if ( ! IS_COMMAND_LINE )
   {
      //start output buffering
      ob_start();
   }
}

//========================================================================

/*!
@function _runEndProcesses( $executionType )
@param $executionType - one of the execution type constants defined
   above LIGHT_EXECUTION or STANDARD_EXECUTION
@discussion - runs shutdown processes as appropriate for the execution
   process specified

   NOTE: Do NOT call this function.  Call runEndProcesses or runLightEndProcesses
      as appropriate.
*/

function _runEndProcesses( $executionType )
{
   //$startTime = getmicrotime();

   //finalize all registered Finalizeable and Disposable objects
   systemMethods::finalizeAll();
   systemMethods::disposeAll();

   if ( ! IS_COMMAND_LINE )
   {
      //flush the output buffer
      ob_end_flush();
   }

   if ( ! IS_COMMAND_LINE ) webServer::tryKillProcess();
}

//========================================================================

/*!
@function _shutdownHandler()
@discussion - called when script execution ends
   - this function handles different types of connection disconnect situations
*/

//function _shutdownHandler()
//{
//   $status = connection_status();
//
//   if ( $status == CONNECTION_STATUS_ABORT )
//   {
//      _runEndProcesses( LIGHT_EXECUTION );
//   }
//   elseif ( $status == CONNECTION_STATUS_TIMEOUT )
//   {
//   }
//}

//========================================================================
?>