<?php

/*
fileLibrary
$Author: dougqh $
$Revision: 1.4 $
$Date: 2004/04/28 01:14:18 $
$Log: fileLibrary.php,v $
Revision 1.4  2004/04/28 01:14:18  dougqh
- Fixed Windows compatibility bug in truePath

Revision 1.3  2003/12/18 20:15:40  dougqh
- Fixed header doc

Revision 1.2  2003/09/29 17:00:39  dougqh
Added functions to URL to file system (and vice versa) paths conversions

Revision 1.1  2003/09/09 15:11:59  dougqh
Initial revision

Revision 1.11  2003/08/29 21:06:11  dougqh
- Modified trueApachePath to get the DOCUMENT_ROOT from _SERVER instead of the old register global DOCUMENT_ROOT variable.

Revision 1.10  2003/07/09 22:05:54  dougqh
Modified makeDirectory to support make any necessary directories
in the path.

Revision 1.9  2003/01/03 14:32:09  millz
- added getDirectoryListing

Revision 1.8  2002/11/18 15:01:18  dougqh
Minor fix to stripSlash.

Revision 1.7  2002/11/06 20:30:52  dougqh
?

Revision 1.6  2002/09/30 23:15:27  dougqh
Quotes to ticks.

Revision 1.5  2002/08/05 19:22:36  dougqh
Added new function readEntireFile.

Revision 1.4  2002/07/22 17:17:07  dougqh
Minor change to stripNonFileNameCharacters.

Revision 1.3  2002/07/02 18:37:57  dougqh
Fixed generateSuffix to work with the GP4 UserSystem.

Revision 1.2  2002/07/02 16:55:39  dougqh
Added additional methods that were added during SRS I to the AlDev instance.


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

define( 'NEW_DIR_ACCESS', 0777 );

/*!
@library file
@abstract - a group of functions for working with files
*/

//========================================================================

/*!
@function makeDirectory( $directoryName )
@param $directoryName - the name of the directory to create
@discussion - creates a directory
*/

function makeDirectory( $directoryName )
{
   if ( ! file_exists( $directoryName ) )
   {
      //-m sets the permissions for the new directories
      //-p creates any necessary directories in the path
      exec( 'mkdir -m ' . decoct( NEW_DIR_ACCESS ) . ' -p ' . $directoryName );
   }
}

//========================================================================

/*!
@function trueApachePath( $path )
@param $path - a URL path
@discussion - translates a URL path to the actual server path

   NOTE: this function is based on information I found on php.net
   - if it necessary to modify this function the URL below may be of help
   http://www.php.net/manual/en/function.realpath.php

   NOTE: trueApachePath relies only works for directories stored in
      the Apache htdocs directory.  Cache directories should NOT be
      stored in htdocs.  When getting the actual path of a cache directory
      use truePath.
@result - returns the actual server path of the specified URL with a
   trailing /
*/

function trueApachePath( $path )
{
   // '/' inside realpath because its a URL
   // but append DIRECTORY_SEPARATOR to the result because its a file system path
   return realpath( $_SERVER{ 'DOCUMENT_ROOT' } . '/' . $path ) . DIRECTORY_SEPARATOR;
}

//========================================================================

/*!
@function urlSlashesToFileSlashes( $path )
@param $path - a String
@discussion - changes URL slashes to file system slashes in the path
@result - returns a String
*/

function urlSlashesToFileSlashes( $path )
{
   $path = str_replace( '/', DIRECTORY_SEPARATOR, $path );
   $path = str_replace( '\\', DIRECTORY_SEPARATOR, $path );
   return $path;
}

//========================================================================

/*!
@function fileSlashesToURLSlashes( $path )
@param $path - a String
@discussion - changes file system slashes to URL slashes in the path
@result - returns a String
*/

function fileSlashesToURLSlashes( $path )
{
   return str_replace( DIRECTORY_SEPARATOR, '/', $path );
}

//========================================================================

/*!
@function truePath( $path )
@param $path - a path relative to the GenePort base path
@param $addSlash - indicates if a trailing slash should be added
   - defaults to false
@discussion - translates a path relative to the GenePort base path into
   an absolute path

   NOTE: trueApachePath relies only works for directories stored in
      the Apache htdocs directory.  Cache directories should NOT be
      stored in htdocs.  When getting the actual path of a cache directory
      use truePath.

   NOTE: It would better if this function did NOT add a trailing slash
      by default.  Unfortunately, other code is using this function and
      changing it as this time is not reasonable.
@result - returns the actual server path of the specified URL with a
   trailing DIRECTORY_SEPARATOR
*/

//TODO: Make addSlash default to false
function truePath( $path, $addSlash = true )
{
   //not sure if isEmpty is loaded yet
   //empty apparently does NOT work on constants
   if ( BASE_DIRECTORY == '' )
   {
      //Calling getLocation could lead to infinite recursion
      unreachablePoint( 'truePath - BASE_DIRECTORY is empty!' );
   }

   $result = BASE_DIRECTORY . $path;
   //clean up any unncecessary slashes
   $result = str_replace(
      str_repeat( DIRECTORY_SEPARATOR, 3 ), DIRECTORY_SEPARATOR,
      $result );
   $result = str_replace(
      str_repeat( DIRECTORY_SEPARATOR, 2 ), DIRECTORY_SEPARATOR,
      $result );

   if ( $addSlash )
   {
      if ( strrpos( $result, DIRECTORY_SEPARATOR ) !== strlen( $result ) - 1 )
      {
         //ensure that there is a trailing /
         return $result . DIRECTORY_SEPARATOR;
      }
      else
      {
         return $result;
      }
   }
   else
   {
      return stripSlash( $result );
   }
}

//========================================================================

/*!
@function appendToFile( $file, $string )
@param $file - absolute file name
@param $string - a String to write to append to the file
@discussion - opens, appends, and closes the specified file
*/

function appendToFile( $file, $string )
{
   $fh = fopen( $file, 'a' );
   fwrite( $fh, $string );
   fclose( $fh );
}

//========================================================================

/*!
@function getDirectoryListing( $directory, $matchingStringsArray )
@param $directory - the path where the directory to search resides
@param $matchingStringsArray - an array of strings, if one is found to match the filename,
   the filename is considered valid.  If the array is empty, all files in the dir will pass the test and be returned
@discussion - an array of file names WITHOUT the path
*/

function getDirectoryListing( $directory, $matchingStringsArray )
{
   $needToTest = count($matchingStringsArray);
   // find all files we need for portlets
   if ( $handle = opendir( $directory ) )
   {
      /* This is the correct way to loop over the directory. */
      while (false !== ($fileName = readdir($handle)) ) {

         if( $needToTest )
         {
            foreach($matchingStringsArray as $matcher)
            {
               if( strpos($fileName, $matcher ) !== false )
               {
                  // print "adding $fileName to the list<br>";
                  $outArray[] = $fileName;
                  break;
               }
            }
         }
         else
         {
            $outArray[] = $fileName;
         }

      }
      closedir( $handle );
   }

   return $outArray;
}

//========================================================================

/*!
@function getExtension( $file )
@discussion - returns the extension of the file specified
@result - returns the extension of the file specified
*/

function getExtension( $file )
{
	return substr( $file, strrpos( $file, '.' ) + 1 );
}

//========================================================================

/*!
@function baseNameWithoutExtension( $file )
@discussion - returns the just the name of the file without the extension
@result - returns a string containing just the name of the file without
   the extension
*/

function baseNameWithoutExtension( $file )
{
   $baseName = baseName( $file );

   return substr( $baseName, 0, strrpos( $baseName, '.' ) );
}

//========================================================================

/*!
@function stripSlash( $directory )
@param $directory - a directory specifier
@discussion - strips off the trailing slash if one is present
@result - returns the directory less any trailing slash
*/

function stripSlash( $directory )
{
   $length = strlen( $directory );

   if ( strrpos( $directory, DIRECTORY_SEPARATOR ) == $length - 1 )
   {
      $directory = substr( $directory, 0, $length - 1 );
   }

   return $directory;
}

//========================================================================

/*!
@function readEntireFile( $file )
@param $file - the name of file to overwrite or create
@discusion - opens, reads a file, and closes it
@result - returns the contents of the file
*/

function readEntireFile( $file )
{
   $fh = fopen( $file, 'r' );
   $contents = fread( $fh, filesize( $file ) );
   fclose( $fh );

   return $contents;
}

//========================================================================

/*!
@function writeFile( $file, $contents )
@param $file - the name of file to overwrite or create
@param $contents - the contents to write to the file
@discusion - opens, writes, and closes the file
*/

function writeFile( $file, $contents )
{
   $fh = fopen( $file, 'w' );
   fwrite( $fh, $contents );
   fclose( $fh );
}

//========================================================================

/*!
@function generateSuffix( $useUser, $useIP )
@param $useUser - a boolean indicating if the suffix should be
   based on the current logged in user
@param $useIP - indicates if the suffix should be based on the
   request IP
@discussion - generates a file suffix based on username, ip, or both
@result - returns a String containing a file suffix
*/

function generateSuffix( $useUser, $useIP )
{
   $suffix = '';

   if ( $useUser )
   {
      importClass( 'User.UserSystem' );
      $userSystem =& UserSystem::getInstance();

      $user =& $userSystem->getCurrentUser();
      $suffix .= $user->getUserName();
   }

   if ( $useIP )
   {
      $suffix .= stripNonFileNameCharacters( $_SERVER{ 'REMOTE_ADDR' } );
   }

   debug( 'generateSuffix', 'result', $suffix );

   return $suffix;
}

//========================================================================

/*!
@function stripNonFileNameCharacters( $string )
@param $string - a String
@discussion - strips out any characters that are not valid in a file name
@result - returns a String
*/

function stripNonFileNameCharacters( $string )
{
   return preg_replace( '/\W/', '_', $string );
}

//========================================================================

/*!
@function getRelativePath( $path, $basePath )
@param $path - a String containing a path or URL
@param $basePath - a String containing a path or URL
@discussion - provides a path to $path relative to $basePath
@result - returns a String
*/

function getRelativePath( $path, $basePath )
{
   return str_replace( $basePath, '', $path );
}

//========================================================================

/*!
@function getContainingDirectory( $filePath )
@param $filePath - a String containing a path like /usr/local/filename.txt
@discussion - returns the directory containing the file
@result - returns a String
*/

function getContainingDirectory( $filePath )
{
//   print 'got: ' . $filePath .BR_NL;
//   print 'container: ' . getFileNameFromFullPath( $filePath ) . BR_NL;
//   print 'returning: ' . str_replace ( getFileNameFromFullPath( $filePath ), '', $filePath) .BR_NL;

   $toRet = str_replace( getFileNameFromFullPath( $filePath ), '', $filePath );
   //return $toRet;
   if( $toRet[ strlen($toRet) - 1 ] == '/' ){ $toRet = substr(  $toRet, 0, strlen($toRet) - 1 ); }
   return $toRet;
}

//========================================================================

/*!
@function getFileNameFromFullPath( $filePath )
@param $filePath - a String containing a path like /usr/local/filename.txt
@discussion - returns the directory containing the file (/usr/local) - this works for directories too
@result - returns a String
*/

function getFileNameFromFullPath( $filePath )
{
   if( $filePath[ strlen($filePath) - 1 ] == '/' ){ $filePath = substr(  $filePath, 0, strlen($filePath) - 1 ); }
   return substr( $filePath, strrpos( $filePath, DIRECTORY_SEPARATOR ) + 1 );
}

//========================================================================

/*!
@function stripLeadSlash( $string )
@param $string - a String containing a path
@discussion - strips the lead slash off $string if one is present
@result - returns $string without a lead slash
*/

function stripLeadSlash( $string )
{
   if ( strpos( $string, DIRECTORY_SEPARATOR ) === 0 )
   {
      return substr( $string, 1 );
   }

   return $string;
}

//========================================================================

?>