<?php

/*
importLibrary
$Author: dougqh $
$Revision: 1.2 $
$Date: 2003/09/29 17:00:39 $
$Log: importLibrary.php,v $
Revision 1.2  2003/09/29 17:00:39  dougqh
Using DIRECTORY_SEPARATOR for Windows compatibility

Revision 1.1  2003/09/09 15:11:59  dougqh
Initial revision

Revision 1.11  2003/07/09 22:05:54  dougqh
Tweaking for BugPort.

Revision 1.10  2002/12/31 15:59:09  dougqh
- Debugging

Revision 1.9  2002/10/01 21:26:06  dougqh
Debugging

Revision 1.8  2002/09/27 18:28:24  dougqh
Optimizing class loader.

Revision 1.7  2002/09/24 23:07:07  dougqh
Quotes to ticks

Revision 1.6  2002/09/03 13:51:40  dougqh
Debugging and a little clean up.

Revision 1.5  2002/07/23 18:47:48  dougqh
Minor change to allow HeavyObject to extend Object.

Revision 1.4  2002/07/18 17:44:13  dougqh
Just debugging.

Revision 1.3  2002/07/15 15:40:59  dougqh
Minor change to account for change in location of configuration.php.

Revision 1.2  2002/07/11 22:17:54  dougqh
*** empty log message ***

Revision 1.1  2002/07/01 20:21:39  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

/*!
@library import
@abstract - a collection of functions that handle file importation
*/

//========================================================================

define( 'IS_GENEPORT4', true );

require_once(
   CLASS_DIR . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR .
   DIRECTORY_SEPARATOR . 'conf' . DIRECTORY_SEPARATOR . 'configuration.php' );

$libraryDir = CLASS_DIR . DIRECTORY_SEPARATOR . 'Libraries' . DIRECTORY_SEPARATOR;
require_once( $libraryDir . 'arrayLibrary.php' );
require_once( $libraryDir . 'assertionLibrary.php' );
require_once( $libraryDir . 'CONSTANTS.php' );

turnOffAssertions();

//error_reporting( E_ERROR );

//========================================================================

/*
@function importClass( $class )
@param $class - a class specifier which may include wild-cards
   - a package specification is optional

   EXAMPLES:
      PackageA.* - load all classes in the package PackageA
         does not load classes in subpackages of PackageA

      PackageA.ClassA - load the class ClassA from the package PackageA

      ClassA - load the class ClassA in the current package

@discussion - loads the specified class - noting that the class is
   needed by the current class if there is one
*/

function importClass( $class )
{
   //print 'Class: ' . $class . '<br />' . NL;

   _importClass( $class, false );
}

//========================================================================

/*!
@function importClasses( $classes )
@param $classes - an array of class specifiers
@discussion - imports all the classes specified in the array
*/

function importClasses( $classes )
{
   foreach ( $classes as $class )
   {
      _importClass( $class, false );
   }
}

//========================================================================

/*
@function tryImportClass( $class )
@param $class - a class specifier which may include wild-cards
   - a package specification is optional

   EXAMPLES:
      PackageA.* - load all classes in the package PackageA
         does not load classes in subpackages of PackageA

      PackageA.ClassA - load the class ClassA from the package PackageA

      ClassA - load the class ClassA in the current package

@discussion - TRIES to load the specified class
@result - if all specified classes loaded properly returns true
   - otherwise returns false
*/

function tryImportClass( $class )
{
   // print "trying to import $class - ";
   return _importClass( $class, true );
}

//========================================================================

/*!
@function primeImport()
@discussion - a function that imports the elements of GenePort 4

   NOTE: Generally, calling this function is unnecessary.  Loading any
   class or library will cause the same actions that this function causes
   followed by the loading of the requested file.
*/

function primeImport()
{
   _importCommon();
}

//========================================================================

/*!
@function _importClass( $class, $test )
@param $class - a class specifier which may include wild-cards
@param $test - a boolean indicating if an existence check
   should be performed before loading the file
@discusion - loads the specified classes
@result - returns true if all files were loaded successfully
*/

function _importClass( $class, $test )
{
   assert( '$class != ""' );

   $package = _getPackage( $class );

   $class = absoluteClassToClass( $class );

   if ( $class == '*' )
   {
      $loadedOkay = ( ( ! $test ) | packageExists( $package ) );

      if ( $loadedOkay )
      {
         $classes = getClassesInPackage( $package );

         //should NOT fail in here, so no testing
         foreach ( $classes as $curClass )
         {
            _loadClass( $package, $curClass, false );
         }
      }
   }
   else
   {
      $loadedOkay = _loadClass( $package, $class, $test );
   }

   return $loadedOkay;
}

//========================================================================

/*!
@function importLibrary( $libraryName )
@param $libraryName - a valid library name
@abstract - imports the specified library
@discussion - libraries should be stored in the Libraries directory
   with the name <library>Library.php
*/

function importLibrary( $libraryName )
{
   //require
   {
      assert( '$libraryName != ""' );
   }
   //body
   {
      static $libraryDirectory;

      if ( ! isset( $libraryDirectory ) )
      {
         $libraryDirectory = _getPackageDirectory( 'Libraries' );
      }

      require_once( $libraryDirectory . DIRECTORY_SEPARATOR . $libraryName . 'Library.php' );
   }
   //ensure
   {
   }
}

//========================================================================

/*!
@function getLoadedClasses()
@discussion - provides an array of all the classes that have been loaded
@result - returns an array of string containing full class specifiers
*/

function getLoadedClasses()
{
   global $GL_loadedClassesHash;

   if ( ! isset( $GL_loadedClassesHash ) )
   {
      return array();
   }
   else
   {
      return array_keys( $GL_loadedClassesHash );
   }
}

//========================================================================

/*!
@function isLoadedClass( $class )
@param $name - an absolute class specifier without wild-cards

   EXAMPLE: PackageA.ClassA

@discussion - indicates if a class has been loaded
*/

function isLoadedClass( $class )
{
   assert( '$class != ""' );

   global $GL_loadedClassesHash;

   return isset( $GL_loadedClassesHash{ $class } );
}

//========================================================================

/*!
@function classToAbsoluteClass( $class )
@param $class - takes a basic class name and translates it to a
   full absolute class specifier
@result - returns an absolute class specifier
*/

function classToAbsoluteClass( $class )
{
   global $GL_classToAbsoluteClassHash;

   assert( '$class != ""' );

   assert( 'isset( $GL_classToAbsoluteClassHash{ strtolower( "$class" ) } )' );

   return $GL_classToAbsoluteClassHash{ strtolower( $class ) };
}

//========================================================================

/*!
@function absoluteClassToClass( $absoluteClass )
@param $absoluteClass - an absolute class specifier
@result - returns the bare class name
*/

function absoluteClassToClass( $absoluteClass )
{
   assert( '$absoluteClass != ""' );

   return substr( $absoluteClass, strrpos( $absoluteClass, '.' ) + 1 );
}

//========================================================================

/*!
@function importConstants( $packageName )
@param $packageName - the name of a package
@discussion - loads the constants contained in the CONSTANTS.php file
   of the specified package

   The constants file is expected to exist if this function is called.
*/

function importConstants( $packageName, $test = false )
{
   assert( '$packageName != ""' );

   if ( ! isset( $GL_importedConstants{ $packageName } ) )
   {
      $GL_importedConstants{ $packageName } = true;

      $fileName = _getPackageDirectory( $packageName ) . DIRECTORY_SEPARATOR . 'CONSTANTS.php';

      if ( ( ! $test ) || file_exists( $fileName ) )
      {
         //print 'FileName: ' . $fileName . BR_NL;
         require_once( $fileName );
      }
   }
}

//========================================================================

/*!
@function packageExists( $package )
@package - a package specifier
@discussion - indicates if the specified package exists
@result - returns true if the specified package exists
*/

function packageExists( $package )
{
   return is_dir( _getPackageDirectory( $package ) );
}

//========================================================================

/*!
@function getClassesInPackage( $packageName )
@param $packageName - the name of a package
@discussion - generates a list of classes contained within
   the specified package ( excluded class that end in Test )

   NOTE: only the class names are returned - not full class specifiers

@result - returns an array of class names
*/

function getClassesInPackage( $packageName )
{
   assert( '$packageName != ""' );

   $packageDirectory = _getPackageDirectory( $packageName );

   $packageHandle = opendir( $packageDirectory );

   $classes = array();
   $numClasses = 0;

   for ( $curFile = readdir( $packageHandle );
         $curFile !== false;
         $curFile = readdir( $packageHandle ) )
   {
      $fileNameLengthMinusFour = strlen( $curFile ) - 4;
      $fileNameLengthMinusEight = $fileNameLengthMinusFour - 4;

      //print "Looking at file " . $curFile . "<br/>\n";

      if ( ( strpos( $curFile, '.php' ) == $fileNameLengthMinusFour ) &&
           ( strpos( $curFile, 'AUTOLOAD' ) !== 0 ) &&
           ( strpos( $curFile, 'Test.php' ) !== $fileNameLengthMinusEight ) )
      {
         $classes[ $numClasses++ ] = substr( $curFile, 0, $fileNameLengthMinusFour );
      }
   }

   closedir( $packageHandle );

   return $classes;
}

//========================================================================

/*!
@function _getPackageDirectory( $packageName )
@param $packageName - the name of a package
@discussion - indicates the directory in which the package is stored
@result - returns a string containing the directory path
*/

function _getPackageDirectory( $packageName )
{
   assert( '\$packageName != ""' );

//   static $packageDirectoryHash;
//   if ( ! isset( $packageDirectoryHash{ $packageName } ) )
//   {
//      $packageDirectoryHash{ $packageName } = CLASS_DIR . '/' . str_replace( '.', '/', $packageName );
//   }
//
//   return $packageDirectoryHash{ $packageName };

   return CLASS_DIR . DIRECTORY_SEPARATOR . str_replace( '.', DIRECTORY_SEPARATOR, $packageName );
}

//========================================================================

/*!
@function _loadClass( $package, $class, $test )
@param $package - a valid package name
@param $class - a valid class name
@param $test - boolean indicating if an existence check should performed
   before requiring the file
@discussion - loads a class noting the current active classes relationship
   to the class in question
*/

function _loadClass( $package, $class, $test )
{
   assert( '$package != ""' );
   assert( '$class != ""' );

   $fullClass = $package . '.' . $class;

   $loadedOkay = isLoadedClass( $fullClass );

   if ( ! $loadedOkay )
   {
      //implicitly load common
      _importCommon();

      $packageDirectory = _getPackageDirectory( $package );

      //load this package's AUTOLOAD if it has one

//      $autoloadFile = $packageDirectory . "/AUTOLOAD.php";
//      if ( file_exists( $autoloadFile ) )
//      {
//         //print "Including autoload file for " . $package . "<br/>\n";
//
//         require_once( $autoloadFile );
//      }

      importConstants( $package, true );

      //load this package's CONSTANTS if it has any

      $file = $packageDirectory .  DIRECTORY_SEPARATOR . $class . '.php';

      if ( ( ! $test ) || file_exists( $file ) )
      {
         require_once( $file );
         $loadedOkay = true;
      }

      _addLoadedClass( $fullClass );
   }

   return $loadedOkay;
}

//========================================================================

/*!
@function _importCommon()
@discussion - loads all the classes in the common directory if
   they have not already been loaded and any autoloaded libraries
*/

function _importCommon()
{
   global $GL_commonLoaded;

   if ( ! isset( $GL_commonLoaded ) )
   {
       //now load the Common package

      $GL_commonLoaded = true;

      //the common classes will need the libraries, so load those first

      $libraryDirectory = _getPackageDirectory( 'Libraries' );

      //load the libraries constants file
      $constantsFile = $libraryDirectory . DIRECTORY_SEPARATOR . 'CONSTANTS.php';
      if ( file_exists( $constantsFile ) )
      {
         require_once( $constantsFile );
      }

      //load the libraries autoload file
      $autoloadFile = $libraryDirectory . DIRECTORY_SEPARATOR . 'AUTOLOAD.php';
      if ( file_exists( $autoloadFile ) )
      {
         require_once( $autoloadFile );
      }

      _loadClass( 'Common', 'Object', false );
      _loadClass( 'Common', 'HeavyObject', false );

      _importClass( 'Common.*', false );

      $postloadFile = _getPackageDirectory( 'Common' ) . DIRECTORY_SEPARATOR . 'POSTLOAD.php';

      if ( file_exists( $postloadFile ) )
      {
         require_once( $postloadFile );
      }
   }
}
//========================================================================

/*!
@function _addLoadedClass( $class )
@param $class - an absolute class specifier
@discussion - notes that the specified class has been loaded
*/

function _addLoadedClass( $class )
{
   //require
   {
      assert( '$class != ""' );
   }
   //body
   {
      global $GL_loadedClassesHash;
      global $GL_classToAbsoluteClassHash;

      if ( ! isset( $GL_loadedClassesHash ) )
      {
         $GL_loadedClassesHash = array();
         $GL_classToAbsoluteClassHash = array();
      }

      $GL_loadedClassesHash{ $class } = true;
      $GL_classToAbsoluteClassHash{
         strtolower( absoluteClassToClass( $class ) ) } = $class;
   }
   //ensure
   {
   }
}

//========================================================================

/*!
@function _getPackage( $class )
@param $class - an absolute class specifier
@discussion - gets the package name from an absolute package specifier
@result - returns a package name
*/

function _getPackage( $class )
{
   assert( '$class != ""' );

   return substr( $class, 0, strrpos( $class, '.' ) );
}

//========================================================================

?>