<?php

/*
process
$Author: dougqh $
$Revision: 1.1 $
$Date: 2003/09/09 15:11:59 $
$Log: processLibrary.php,v $
Revision 1.1  2003/09/09 15:11:59  dougqh
Initial revision

Revision 1.3  2002/11/11 15:22:19  dougqh
Using new debugging constant in process::start.

Revision 1.2  2002/11/06 20:30:52  dougqh
debugging

Revision 1.1  2002/10/28 22:44:23  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/

//========================================================================

/*!
@class process
@abstract - used to find information about processes running on the process
   - also used to execute commands
@discussion - this library should ALWAYS be used instead of making calls
   directly to PHP's execute functions to ensure proper operation under
   PHP-Accelerator
   
   PHP-Accelerator must be set to NOT cache this file.
*/

class process
{

//========================================================================

   /*!
   @function getUser( $process_id )
   @param $process_id - a process id
   @discussion - provides the name of the user running the specified process
   @result - returns a String
      - returns false if the specified process does not exist
   */
   
   function getUser( $process_id )
   {
      $lastLine = exec( 'ps -o user -p ' . $process_id );
       
      if ( $lastLine != '' )
      {
         return $lastLine;
      }
      else
      {
         return false;
      }
   }
   
//========================================================================

   /*!
   @function getProcessList( $username )
   @param $username - a String containing a username
   @discussion - provides a list of information describing the
      processes the specified user is running
   @result - returns a formatted String
   */
   
   function getProcessList( $username )
   {
      list( $retval, $outputLines ) = process::execute( 'ps -ef | grep ' . $username . ' | grep -v grep | grep -v bash | grep -v ps' );
      
      return implode( NL, $outputLines );
   }

//========================================================================

   /*!
   @function getMemoryUsage( $process_id )
   @param $process_id - a process id
   @discussion - provides the memory usage in KB of the specified process
   @result - returns an integer
      - returns false if the specified process does not exist
   */
   
   function getMemoryUsage( $process_id )
   {
      $lastLine = exec( 'ps -o vsz -p ' . $process_id );
       
      if ( $lastLine != '' )
      {
         return $lastLine;
      }
      else
      {
         return false;
      }
   }
   
//========================================================================
   
   /*!
   @function execute( $command )
   @param $command - a command to execute
   @discussion - executes an arbitrary command and waits for it to return
      - output is captured
   @result - returns an ordered pair
      - the first element is the command's return value
      - the second element is the output lines in an array
   */
   
   function execute( $command )
   {
      $outputLines = '';
      $returnVar = 0;

      exec( $command, $outputLines, $returnVar );
      
      return array( $returnVar, $outputLines );
   }
   
//========================================================================

   /*!
   @function start( $command, [ $addRedirects ] )
   @param $command - a command to execute
   @param $addRedirects - indirects if redirects to /dev/null should be added
       - defaults to true
   @discussion - executes an arbitrary command and does not wait for
      it to return
      
      - All output is redirect to /dev/null.
      
      NOTE: The redirects to /dev/null should NOT be included in the command line.
   @result - returns the command's return value
   */
   
   function start( $command, $addRedirects = true )
   {
      if ( $addRedirects )
      {
         $command .= ' 1>/dev/null 2>/dev/null &';
      }
      
      if ( PRINT_START_COMMAND_LINE_AND_EXIT )
      {
         print 'Starting Command: ' . $command . NL . '<br />';
         exit;
      }
      
      $returnVar = 0;      
      
      system( $command, $returnVar );
      
      return $returnVar;
   }
   
//========================================================================

};

?>