<?php

/*
set
$Author: millz $
$Revision: 1.2 $
$Date: 2004/01/06 19:11:16 $
$Log: setLibrary.php,v $
Revision 1.2  2004/01/06 19:11:16  millz
- added arrayToSet

Revision 1.1  2003/09/09 15:11:59  dougqh
Initial revision

Revision 1.2  2002/12/20 23:50:58  dougqh
Debugging ?

Revision 1.1  2002/09/28 16:02:12  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/

/*!
@library set
@discussion - a library of functions that uses hashes as mathematical
   sets
*/

class set
{
   
//========================================================================
   
   /*!
   @function namedObjectsToSet( $namedObjects )
   @param $namedObjects - takes an array of NamedObjects
      - those implementing getName and turns them into a set
   @discussion - turns an array of NamedObjects into a set
      that may be used by other functions in this library
   @result - returns a Set
   */
   
   function &namedObjectsToSet( &$namedObjects )
   {      
      $numNamedObjects = count( $namedObjects );
      
      $set = array();
      for ( $i = 0; $i < $numNamedObjects; ++$i )
      {
         $set{ $namedObjects[ $i ]->getName() } = true;
      }
      
      return $set; 
   }
   
//========================================================================
   
   /*!
   @function stringsToSet( $strings )
   @param $strings - an array of Strings
   @discussion - converts an array of Strings into a set
   @result - returns a Set
   */
   
   function &stringsToSet( &$strings )
   {
      $set = array();
      foreach( $strings as $curString )
      {
         $set{ $curString } = true;
      }
      
      return $set;
   }
   
//========================================================================
   
   /*!
   @function hashToSet( $hash )
   @param $hash - a hash of anything
   @discussion - converts the keys of a hash into a Set
   @result - returns a Set
   */
   
   function &hashToSet( &$hash )
   {
      $keys = array_keys( $hashToSet );
      
      return stringsToSet( $keys );
   }

//========================================================================
   
   /*!
   @function inSet( $set, $item )
   @param $set - a Set
   @param $item - an item
   @discussion - checks to see if an item is in a set
   @result - returns a boolean
   */
   
   function inSet( &$set, $element )
   {
      return isset( $set{ $item } );
   }
   
//========================================================================
   
   /*!
   @function intersection( $setA, $setB )
   @param $setA - a Set
   @param $setB - a Set
   @discussion - finds the intersection of the two specified sets
   @result - returns a Set
   */
   
   function intersection( &$setA, &$setB )
   {
      $intersection = array();
      
      if ( count( $setA ) < count( $setB ) )
      {      
         foreach ( array_keys( $setA ) as $element => $dummy )
         {
            if ( isset( $setB{ $element } ) ) $intersection{ $element } = true;
         }
      }
      else
      {
         foreach ( array_keys( $setB ) as $element => $dummy )
         {
            if ( isset( $setA{ $element } ) ) $intersection{ $element } = true;
         }
      }
      
      return $intersection;
   }
   
//========================================================================

   /*!
   @function arrayToSet( $array )
   */

    function arrayToSet( $array )
    {
        foreach ( $array as $value )
        {
            $result{ $value } = true;
        }
        return $result;
    }

//========================================================================
   
   /*!
   @function equals( $setA, $setB )
   @param $setA - a Set
   @param $setB - a Set
   @discussion - indicates if the two sets are equivalent
   @result - returns a boolean
   */
   
   function equals( &$setA, &$setB )
   {
      $aKeys = array_keys( $setA );
      
      return ( count( $aKeys ) == count( array_intersect( array_keys( $setB ), $aKeys ) ) );
   }

//========================================================================

   /*!
   @function minus( &$setA, &$setB )
   @param $setA - a Set
   @param $setB - a Set
   @discussion - provides a Set with all the elements in setA not in setB
   @result - returns a Set
   */
   
   function minus( &$setA, &$setB )
   {
      $minus = array();
      foreach ( $setA as $element => $dummy )
      {
         if ( ! isset( $setB{ $element } ) ) $minus{ $element } = true;
      }
      
      return $minus;
   }
   
//========================================================================

   /*!
   @function isSubSet( $setA, $setB )
   @param $setA - a Set
   @param $setB - a Set
   @discussion - indicates if setA is a subset of setB
   @result - returns a boolean
   */
   
   function isSubSet( &$setA, &$setB )
   {
      foreach ( $setA as $element => $dummy )
      {
         if ( ! isset( $setB{ $element } ) ) return false;
      }
      
      return true;      
   }
   
//========================================================================

};

?>