<?php

/*
typeLibrary
$Author: dougqh $
$Revision: 1.1 $
$Date: 2003/09/09 15:11:59 $
$Log: typeLibrary.php,v $
Revision 1.1  2003/09/09 15:11:59  dougqh
Initial revision

Revision 1.10  2002/12/31 20:42:59  dougqh
- Added convenience function toRef.

Revision 1.9  2002/11/18 14:48:32  dougqh
Minor correction in hasMethods.

Revision 1.8  2002/10/14 15:33:13  dougqh
Added new function isEmpty which provides a better definition of empty than the built-in empty function provided by PHP.

Revision 1.7  2002/10/02 14:50:42  dougqh
Combatting duplicate definition conflicts with GP3 code.

Revision 1.6  2002/09/25 15:18:54  dougqh
Quotes to ticks.

Revision 1.5  2002/09/18 21:49:11  dougqh
Optimizing

Revision 1.4  2002/07/23 22:14:09  dougqh
Added isEmailAddress which is cleaned up version of GP3 isEmail.

Revision 1.3  2002/07/18 17:42:29  dougqh
nothing

Revision 1.2  2002/07/09 13:29:24  dougqh
Added the GP3 isNumeric function.

Revision 1.1  2002/07/01 20:21:39  dougqh
Initial revision


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a 
legal AGREEMENT between you (either as a 
registered individual or as the 
registered representative and on 
behalf of a single entity) and INCOGEN, Inc., 
for the software product identified above, which 
product includes computer software and may 
include associated media, printed materials, 
and "online" or electronic documentation 
("SOFTWARE PRODUCT").  By installing, copying, 
or otherwise using the SOFTWARE PRODUCT, you 
agree to be bound by the terms of this EULA. 
If you do not agree to the terms of this EULA, 
then DO NOT install or use the SOFTWARE 
PRODUCT; in such event the original purchaser 
may, however, return it to the place of 
purchase within thirty days of the date of 
original purchase for a full refund.

Please read the remaining text of this agreement 
in the accompanying license.txt file
*/

//========================================================================

/*!
@library type
@abstract - this library defines functions for doing strong type
   checking in PHP
   - it also includes functions for defining interfaces and enumerations
*/

//========================================================================

/*!
@function isInstanceOf( &$value, $type )
@param $value - a variable which may be of a given type
@param $type - a string containing the name of a defined type

   NOTE: An array of a certain type of class can be indicated by using <class>[].
   
   NOTE: A hash of a certain type of class can be indicated by using <class>{}.
   
   NOTE: An array of hashes of type can be indicated using <class>[]{} and vice versa.
   
@discussion - tests to see if object is an instance of the class $class
@result - true if object is of type $class
          false if the object is not of type $class
*/

function isInstanceOf( &$value, $type )
{
   //Figure out if we need to check for an array or hash
   
   $checkHash = false;
   $checkArray = false;
   
   $arrayPos = strpos( $type, '[]' );
   $hashPos = strpos( $type, '{}' );   
   
   if ( $arrayPos !== false )
   {
      if ( ( $hashPos !== false ) && ( $hashPos < $arrayPos ) )
      {
         $checkHash = true;
         $removePos = $hashPos;
      }
      else
      {
         $checkArray = true;
         $removePos = $arrayPos;
      }
   }
   elseif ( $hashPos !== false )
   {
      $checkHash = true;
      $removePos = $hashPos;
   }
   
   if ( $checkArray )
   {         
      $elementType = substr_replace( $type, '', $removePos, 2 );
      return _isInstanceOfArray( $value, $elementType );          
   }
   elseif ( $checkHash )
   {
      $elementType = substr_replace( $type, '', $removePos, 2 );
      return _isInstanceOfHash( $value, $elementType );
   }
   elseif ( class_exists( $type ) )
   {
      return  _isInstanceOfClass( $value, $type );
   }
   elseif ( enumExists( $type ) )
   {
      return _isInstanceOfEnum( $value, $type );
   }
   elseif ( interfaceExists( $type ) )
   {
      return _isInstanceOfInterface( $value, $type );
   }
   elseif ( ruleTypeExists( $type ) )
   {
      return _isInstanceOfRuleType( $value, $type );
   }
   else
   {
      //unloaded type, so return false
      
      return false;
   }     
}

//========================================================================

/*!
@function typeExists( $typeName )
@param $typeName - the type 
@discussion - indicates if a type is defined
@result - returns true if the type $typeName is defined
*/

function typeExists( $typeName )
{
   //require
   {
      assert( '$typeName != ""' );
   }
   //body
   {      
      //remove array/hash specifiers
      //if the base type exists the array/hash type exists
      
      $typeName = str_replace( array( '[]', '{}' ), '', $typeName );
      
      $result = classExists( $typeName ) || interfaceExists( $typeName ) ||
                enumExists( $typeName ) || ruleTypeExists( $typeName );
   }
   //ensure
   {
   }

   return $result;
}

//========================================================================

/*!
@function classExists( $className )
@param $clasName - the name of a class
@discussion - checks to see if a class has been defined
@result - returns true if the class $className has been defined
   otherwise returns false
*/

function classExists( $className )
{
   return class_exists( $className );
}

//========================================================================

/*!
@function toRef( $constant )
@param $constant - a constant - i.e. a number, string literal, or constant
@discussion - provides a reference to a variable containing the value in the constant
@result - returns a reference to a constant
*/

function &toRef( $constant )
{
   return $constant;
}

//========================================================================

/*!
@function _isInstanceOfClass( $value, $className )
@param $value - a value to test
@param $className - the name of defined class
@discussion - indicates if $value is an instance of the class $className
@result - returns true if $value is an instance of the class $className
   otherwise returns false
*/

function _isInstanceOfClass( &$value, $className )
{
   assert( 'classExists( $className )' );

   //PHP 4.2 has is_a, see change log - undocumented on PHP.net
   return is_a( $value, strtolower( $className ) );
}

//========================================================================

/*!
@function defineTypeByRule( $typeName, $ruleFunction )
@param $typeName - a nonempty string containing a name for this type
@param $ruleFunction - the name of standalone type checking function

   NOTE: The function specified must accept a single value and returned
     a boolean indicating if that value belong to the specified type.
@discussion - creates a new type classification based on a rule
*/

function defineRuleType( $typeName, $ruleFunction )
{
   //require
   {
      assert( '$typeName != ""' );
      
      assert( 'function_exists( $ruleFunction )' );
      assert( '! typeExists( $typeName )' );
   }
   //body
   {
      global $GL_ruleTypesHash;
      
      if ( ! isset( $GL_ruleTypesHash ) )
      {
         $GL_ruleTypesHash = new_hash();
      }
      
      $GL_ruleTypesHash{ $typeName } = $ruleFunction;
   }
   //ensure
   {
      assert( '$GL_ruleTypesHash{ $typeName } == $ruleFunction' );
      assert( 'ruleTypeExists( $typeName )' );
   }
}

//========================================================================

/*
@function ruleTypeExists( $typeName )
@param $typeName - 
@discussion - indicates if the specified rule type exists
@result - returns true if the rule type $typeName exists
   otherwise returns false
*/

function ruleTypeExists( $typeName )
{
   assert( '$typeName != ""' );

   global $GL_ruleTypesHash;
   
   return isset( $GL_ruleTypesHash{ $typeName } );
}

//========================================================================

/*
@function _isInstanceOfRuleType( $value, $ruleTypeName )
@param $value - a value to test
@param $ruleTypeName - a valid rule type name
@discussion - indicates if the valid belongs to the
   specified rule type classification
@result - returns true if the value does belong
   otherwise returns false
*/

function _isInstanceOfRuleType( &$value, $ruleTypeName )
{
   assert( 'ruleTypeExists( $ruleTypeName )' );
   
   global $GL_ruleTypesHash;
      
   $ruleFunction = $GL_ruleTypesHash{ $ruleTypeName };
      
   return $ruleFunction( $value );
}

//========================================================================

/*!
@function defineInterface( $interfaceName, $methods )
@param $interfaceName - the name of the interface being defined
@param $methods - an array of strings containing the names of the methods
   a class must implement to be in compliance with this interface
@discussion - since PHP lacks interfaces and multiple inheritance - 
   this function has been provided as a way to define interfaces
   - any class defining the methods listed in the $methods array
   will be considered to implement the interface $name
   - frequently it will still be better to define a class, because
   interfaces can not impose any design by contract constraints
   
   - when an interface is used by a class it is prefered that the class
   name be flagged with a comment indicated that
   Example: class x // implements <interface>
*/

function defineInterface( $interfaceName, $methods )
{
   global $GL_interfaceDefinitions;
   
   if ( ! isset( $GL_interfaceDefinitions ) )
   {
      $GL_interfaceDefinitions = new_hash();
   }
      
   //require
   {
      assert( '$interfaceName != ""' );
      
      assert( '! typeExists( $interfaceName )' );
      
      assert( 'isArray( $methods )' );
   }
   //body
   {      
      $numMethods = count( $methods );
      
      for ( $i = 0; $i < $numMethods; $i++ )
      {
         $methods[ $i ] = strtolower( $methods[ $i ] );
      }
      
      $GL_interfaceDefinitions{ $interfaceName } = $methods;
   }  
   //ensure
   {
      assert( 'getInterfaceMethods( $interfaceName ) == $methods' );
      assert( 'interfaceExists( $interfaceName )' );
   }
}

//========================================================================

/*!
@function interfaceExists( $interfaceName )
@param $name - a string containing the name of an interface
@discussion - indicates if an interface is defined
@result - returns true if the interface $name is defined
   otherwise returns false
*/

function interfaceExists( $interfaceName )
{
   assert( '$interfaceName != ""' );
   
   global $GL_interfaceDefinitions;
   
   if ( ! isset( $GL_interfaceDefinitions ) )
   {
      $GL_interfaceDefinitions = new_hash();
   }
   
   return isset( $GL_interfaceDefinitions{ $interfaceName } );
}

//========================================================================

/*!
@function getInterfaceMethods( $interfaceName )
@param $interfaceName - a string containing the name of an interface
@discussion - indicates what methods an object must define to
   be in compliance with a given interface
@result - returns an array containg the names of the methods
   a class must define to be in compliance with the interface
   $name
*/

function getInterfaceMethods( $interfaceName )
{
   assert( 'interfaceExists( $interfaceName )' );

   global $GL_interfaceDefinitions;
      
   return $GL_interfaceDefinitions{ $interfaceName };
}

//========================================================================

/*!
@function _isInstanceOfInterface( $value, $interfaceName )
@param $value - a value to check to see if it is an instance of
   the interface
@param $interfaceName - the name of a defined interface
@discussion - indicates if $value is an instance of the
   interface $interfaceName
@result - returns true if value is an instance of the specified
   interface
*/

function _isInstanceOfInterface( &$value, $interfaceName )
{
   assert( 'interfaceExists( $interfaceName )' );

   if ( is_object( $value ) )
   {
      $interfaceMethods = getInterfaceMethods( $interfaceName );
   
      return hasMethods( $value, $interfaceMethods );
   }
   else
   {
      return false;
   }
}

//========================================================================

/*!
@function defineEnum( $enumName, $members ... )
@param $enumName - the name of this enumerated type
@param $members - a list of strings that define the constants for
   this enumeration - there must at least one member in the
   enumeration
   - each of the members of the enumeration will be defined as a constant
@discussion - defines a new enumerated type
*/

function defineEnum( $enumName, $member0 )
{
   //require
   {
      assert( '$enumName != ""' );
      assert( '! typeExists( $enumName )' );
      
      assert( '$member0 != ""' );
   }
   //body
   {
      global $GL_definedEnums;
      if ( ! isset( $GL_definedEnums ) )
      {
         $GL_definedEnums = new_array();
      }
      $GL_definedEnums{ $enumName } = new_array();
      
      $args = func_get_args();
      
      $numArgs = count( $args );
      
      for ( $i = 1; $i < $numArgs; ++$i )
      {
         $string = 'enumeration_' . $enumName . '_' . $args[ $i ];
         
         define( $args[ $i ], $string );
         
         $GL_definedEnums{ $enumName }[ $i - 1 ] = $string;
      }
   }
   //ensure
   {
      assert( 'enumExists( $enumName )' );
   }
}

//========================================================================

/*!
@function enumExists( $enumName )
@param $enumName - the name of an enumerated type
@discussion - indicates whether or not an enumeration with the name
   $enumName has been defined
@result - returns true if an enum with the name $enumName has
   been defined
   - otherwise returns false
*/

function enumExists( $enumName )
{
   assert( '$enumName != ""' );

   global $GL_definedEnums;
   
   if ( isset( $GL_definedEnums ) )
   {
      return isset( $GL_definedEnums{ $enumName } );         
   }
   else
   {
      return false;
   }
}

//========================================================================

/*!
@function _isInstanceOfEnum( $value, $enumName )
@param $value - a value to test
@param $enumName - the name of a defined enumeration
@discussion - indicates if $value is a member of the enumeration $enumName
@result - returns true if $value is a member of the enumeration $enumName
   otherwise returns false
*/

function _isInstanceOfEnum( $value, $enumName )
{
   assert( 'enumExists( $enumName )' );

   if ( is_string( $value ) )
   {
      global $GL_definedEnums;

      return in_array( $value, $GL_definedEnums{ $enumName } );
   }
   else
   {
      return false;
   }
}

//========================================================================

/*!
@function isConcrete( $object, $abstractParentClass, [ $methods ] )
@param $object - the object you are checking to ensure is concrete
@param $abstractParentClass - a string containing the name of the
   abstract parent class you are checking to make sure is not being
   instantiated
@param $methods - an array of strings containing method names that
   a concrete class must define - this parameter is optional if
   ommitted it is assumed no methods need be defined
@discussion - checks to make sure an object is concrete
   - calls to this function should be made by the constructor of the
     abstract parent as part of its constructors precondition assertions
   - This function considers the object to be concrete if the following
     conditions are met ...
     1.  The object is from a subclass of $abstractParentClass
     2.  The object is not directly instantiated from the $abstractParentClass
     3.  The object has all of the specified methods
@result - returns true if the object is from a concrete child class
   otherwise returns false
*/

function isConcrete( &$object, $abstractParentClass, $methods = array() )
{
   assert( 'is_object( $object )' );
   assert( 'is_string( $abstractParentClass )' );
   assert( 'is_array( $methods )' );
         
   return ( get_class( $object ) != strtolower( $abstractParentClass ) ) &&
          isInstanceOf( $object, $abstractParentClass ) &&
          hasMethods( $object, $methods );
}

//========================================================================

/*!
@function hasMethods( $object, $methods )
@param $object - an object of any type
@param $methods - an array of method names
@discussion - checks to make sure an object has the specified methods
@result - returns true if the object has all the specified methods
   otherwise returns false
*/

function hasMethods( &$object, $methods )
{
   assert( 'is_object( $object )' );
   assert( 'is_array( $methods )' );

   $actualMethods = get_class_methods( $object );
   
   foreach ( $methods as $curIndex => $curMethod )
   {
      $methods[ $curIndex ] = strtolower( $curMethod );
   }

   $intersectionMethods = array_intersect( $methods, $actualMethods );               
      
   return ( count( $methods ) == count( $intersectionMethods ) );
}

//========================================================================

/*!
@function isSubclassOf( $testClass, $parentClass )
@param $testClass - a string containing the name of the class in question
@param $parentClass - a string containg the name of the potential parent class
@discussion - checks to see if $testClass is inherited from $parentClass
@result - returns true if $testClass is a subclass of $parentClass
   otherwise returns false
   
   NOTE: class A is considered to be a subclass of class A.
*/

function isSubclassOf( $testClass, $parentClass )
{
   //require
   {
      assert( '$testClass != ""' );
      assert( '$parentClass != ""' );
   }
   //body
   {
      $curClass = strtolower( $testClass );
      $parentClass = strtolower( $parentClass );
      
      for ( $found = ( $curClass == $parentClass );
            ( ! $found ) && ( $curClass !== false );
            $found = ( $curClass == $parentClass ) )
      {
         $curClass = get_parent_class( $curClass );
      }
   }
   //ensure
   {
   }

   return $found;
}

//========================================================================

/*!
@function _isInstanceOfArray( $value, [ $elementType ] )
@param $value - a value to check to see it is an array
@param $elementType - a type that all the members of the array should be
   
   NOTE: If type is unspecified - a type check is not perform on the
   elements of the array.
@discussion - indicates if $value is an array and if all the elements
   are of the optionally specified type
@result - returns true if value is an array and the elements are of
   the specified type; otherwise returns false
*/

function _isInstanceOfArray( &$value, $elementType = UNSPECIFIED )
{
   assert( '( $elementType === UNSPECIFIED ) || ( $elementType != "" )' );

   if ( isArray( $value ) )
   {
      if ( $elementType === UNSPECIFIED )
      {
         return true;
      }
      else
      {
         $result = true;
         
         $numValues = count( $value );
         for ( $i = 0; ( $i < $numValues ) && $result; ++$i )
         {
            $result = isInstanceOf( $value[ $i ], $elementType );
         }
         
         return $result;
      }
   }
   else
   {
      return false;
   }
}

//========================================================================

/*!
@function _isInstanceOfHash( $value, [ $type ] )
@param $value - a value to check to see it is an array
@param $type - a type that all the members of the array should be
   
   NOTE: If type is unspecified - a type check is not perform on the
   elements of the array.
@discussion - indicates if $value is an array and if all the elements
   are of the optionally specified type
@result - returns true if value is an array and the elements are of
   the specified type; otherwise returns false
*/

function _isInstanceOfHash( &$value, $elementType = UNSPECIFIED )
{
   assert( '( $elementType === UNSPECIFIED ) || ( $elementType != "" )' );
   
   if ( isHash( $value ) )
   {
      if ( $elementType === UNSPECIFIED )
      {
         return true;
      }
      else
      {
         $result = true;
         
         $keys = array_keys( $value );
         
         //print_r( $keys );
         
         foreach ( $keys as $key )
         {
            $result = isInstanceOf( $value{ $key }, $elementType );
         }
         
         return $result;
      }
   }
   else
   {
      return false;
   }
}

//========================================================================

if ( ! function_exists( 'isnumeric' ) )
{
   
/*!
@function isNumeric( $string )
@param $string - a String
@discussion - uses php's built in is_numeric, but also does a trim
	so leading and trailing white space is accepted
@result - returns true if the String is numeric
*/

function isNumeric( $string )
{
	return is_numeric( trim( $string ) );
}

}

//========================================================================

/*!
@function isEmpty( $value )
@discussion - a minor variation on PHP's built-in empty
   isEmpty does not consider 0 or "0" to be empty, which is different
   from the built-in empty
   
   NOTE: This function cannot be used on variables that have NOT been set.
@result - returns a boolean
*/

function isEmpty( $value )
{
   //use strong equivalence to ensure the desired meaning
   return empty( $value ) &&
          ( $value !== 0 ) &&
          ( $value !== "0" ) &&
          ( $value !== false );
}

//========================================================================

/*!
@function isEmailAddress( $string )
@param $string - a String
@discussion - checks to see if the specified String is an email
   address
@result - returns true if the String is an email address;
   otherwise returns false
*/

function isEmailAddress( $string )
{
	// Must contain '@' and '.' 
	if ( strpos( $string, '@' ) === false ) return false;
	if ( strpos( $string, '.' ) === false ) return false;

	// Must be at least 7 characters long (x@y.com) 
   if ( strlen( $string ) < 7 ) return false;

	list( $user, $domain ) = explode( '@', $string, 2 );

   //Checks for user validity
   //--------------------------------------------------------

   // User must be at least 1 character long
   if ( strlen( $user ) < 1 ) return false;

   // User cannot contain a comma
   if ( strpos( $user, ',' ) !== false ) return false;
   
   //Checks for domain validity
   //--------------------------------------------------------
   
   $dotPos = strpos( $domain, '.' );
   
	// Domain must contain at least 1 dot	
	if ( $dotPos === false ) return false;
	
	// Domain may not start with a dot 
	if ( $dotPos === 0 ) return false;

   // Domain must be a least 4 characters (z.com) 
   if ( strlen( $domain ) < 4 ) return false;

   // May not have more than one dot in sequence 
   if ( strpos( $domain, '..' ) !== false ) return false;

   // established presence of dot, so this is okay
   list( $domainName, $domainExtension ) = explode( '.', $domain, 2 );
   
   // There must be at least two parts to a domain (a sub domain would have 3 - a.b.com) 
   // This was established by checking for dot earlier.
	
   // follows from check that dot is not the first character
   //if ( $domainName = "" ) return false;
   
   //domain name may contain up to 26 characters ? 
   if ( strlen( $domainName ) > 26 ) return false;
   
   // first character of domain must be alphabetic or _
   if ( ! isAlphabetic( substr( $domainName, 0, 1 ), '_' ) ) return false;
   
   // only alphanumeric characters _ - and . may be in the domain name
   if ( ! isAlphaNumeric( $domainName, '_-.' ) ) return false;

   // Extention must be 2 or 3 characters (.nu - .com) 
   $extensionLength = strlen( $domainExtension );
   if ( $extensionLength > 3 || $extensionLength < 2 ) return false;

   if ( ! isAlphabetic( $domainExtension ) ) return false;
   
   //Everything checked out return true
   //--------------------------------------------------------
   return true; 
}

//========================================================================

?>