<?php

/*
FutureDateField
$Author: dougqh $
$Revision: 1.1 $
$Date: 2004/05/19 14:18:08 $
$Log: FutureDateField.php,v $
Revision 1.1  2004/05/19 14:18:08  dougqh
- A LightFormField to handle dates


IMPORTANT-READ CAREFULLY. This INCOGEN
End-User License Agreement ("EULA") is a
legal AGREEMENT between you (either as a
registered individual or as the
registered representative and on
behalf of a single entity) and INCOGEN, Inc.,
for the software product identified above, which
product includes computer software and may
include associated media, printed materials,
and "online" or electronic documentation
("SOFTWARE PRODUCT").  By installing, copying,
or otherwise using the SOFTWARE PRODUCT, you
agree to be bound by the terms of this EULA.
If you do not agree to the terms of this EULA,
then DO NOT install or use the SOFTWARE
PRODUCT; in such event the original purchaser
may, however, return it to the place of
purchase within thirty days of the date of
original purchase for a full refund.

Please read the remaining text of this agreement
in the accompanying license.txt file
*/

//========================================================================

importClass( 'HTMLComponents.LightForms.ALightFormField' );
importConstants( 'Time' );

//========================================================================
/*!
@class FutureDateField
@abstract -
*/

class FutureDateField extends ALightFormField
{

//========================================================================

   /*!
   @function FutureDateField()
   @discussion - constructs a new FutureDateField object
   @result - constructs a new FutureDateField object
   */
   function FutureDateField( $name, $displayName )
   {
      $this->ALightFormField( $name, $displayName );
   }

//========================================================================

   /*!
   @function _getDefaultValue()
   @discussion - defaults to NULL
   */
   function _getDefaultValue()
   {
      if ( $this->isRequired() )
      {
         return time();
      }
      else
      {
         return NULL;
      }
   }

//========================================================================

   /*!
   @function _getMonthValue()
   @discussion - month value used by _getEnabledHTML and _getDisabledHTML
   @result - returns a String
   */
   function _getMonthValue()
   {
      $value = $this->getValue();
      if ( is_null( $value ) ) return NULL;
      return intval( date( 'm', $value ) );
   }

//========================================================================

   /*!
   @function _getMonthSelect()
   */
   function &_getMonthSelect()
   {
      $months = array(
          'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
          'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec' );

      $month = $this->_getMonthValue();
      importClass( 'HTMLComponents.LightForms.SelectComponent' );
      $select =& new SelectComponent( $this->getName() . '_month', false );
      if ( ! $this->isRequired() ) $select->addOption( '', '-' );
      for ( $i = 1; $i <= 12; ++$i )
      {
         $select->addOption( $i, $months[ $i - 1 ], ( $i == $month ) );
      }
      return $select;
   }

//========================================================================

   /*!
   @function _getDayValue()
   @discussion - month value used by _getEnabledHTML and _getDisabledHTML
   @result - returns a String
   */
   function _getDayValue()
   {
      $value = $this->getValue();
      if ( is_null( $value ) ) return NULL;
      return intval( date( 'd', $value ) );

   }

//========================================================================

   /*!
   @function _getDaySelect()
   */
   function &_getDaySelect()
   {
      $day = $this->_getDayValue();
      importClass( 'HTMLComponents.LightForms.SelectComponent' );
      $select =& new SelectComponent( $this->getName() . '_day', false );
      if ( ! $this->isRequired() ) $select->addOption( '', '-' );
      for ( $i = 1; $i <= 31; ++$i )
      {
         $select->addOption( $i, $i, ( $i == $day ) );
      }
      return $select;
   }

//========================================================================

   /*!
   @function _getYearValue()
   @discussion - month value used by _getEnabledHTML and _getDisabledHTML
   @result - returns a String
   */
   function _getYearValue()
   {
      $value = $this->getValue();
      if ( is_null( $value ) ) return NULL;
      return intval( date( 'Y', $value ) );

   }

//========================================================================

  /*!
   @function _getYearSelect()
   */
   function &_getYearSelect()
   {
      $year = $this->_getYearValue();
      importClass( 'HTMLComponents.LightForms.SelectComponent' );
      $select =& new SelectComponent( $this->getName() . '_year', false );
      if ( ! $this->isRequired() ) $select->addOption( '', '-' );

      $curYear = intval( date( 'Y' ) );
      if ( is_null( $year ) )
      {
         $startYear = $curYear;
      }
      else
      {
         $startYear = min( $year, $curYear );
      }
      $endYear = max( $startYear + 10, $curYear + 10 );
      for ( $i = $startYear; $i <= $endYear; ++$i )
      {
         $select->addOption( $i, $i, ( $i == $year ) );
      }
      return $select;
   }

//========================================================================

   /*!
   @function _getEnabledHTML()
   @discussion -
   @result - html string
   */
   function _getEnabledHTML()
   {
      $monthSelect =& $this->_getMonthSelect();
      $daySelect =& $this->_getDaySelect();
      $yearSelect =& $this->_getYearSelect();

      return
          $monthSelect->getHTML() .
          $daySelect->getHTML() .
          $yearSelect->getHTML();
   }

//========================================================================

   /*!
   @function _getDisabledHTML()
   @discussion - provides HTML when this field is disabled
   @result - returns HTML
   */
   function _getDisabledHTML()
   {
      unreachablePoint( 'FutureDateField::_getDisabledHTML' );
   }

//========================================================================

   /*!
   @function &_getFormValue()
   @discussion -
   @result - TimeDuration object
   */
   function &_getFormValue()
   {
      $monthName = $this->getName() . '_month';
      $dayName = $this->getName() . '_day';
      $yearName = $this->getName() . '_year';

      if ( ! isset( $_REQUEST{ $monthName } ) ||
           ! isNumeric( $_REQUEST{ $monthName } ) ||
           ! isset( $_REQUEST{ $dayName } ) ||
           ! isNumeric( $_REQUEST{ $dayName } ) ||
           ! isset( $_REQUEST{ $yearName } ) ||
           ! isNumeric( $_REQUEST{ $yearName } ) )
      {
         return NULL;
      }

      $time = strtotime(
         $_REQUEST{ $yearName } . '-' .
         $_REQUEST{ $monthName } . '-' .
         $_REQUEST{ $dayName } );

      //Error code from strtotime
      if ( $time == -1 ) return NULL;

      return $time;
   }

//========================================================================

   /*!
   @function isValid()
   @discussion -
   @result - bool
   */
   function isValid()
   {
       //TODO: Finish this
       return parent::isValid();
   }

//========================================================================

}

?>